/*
 * $Id: netgear_mib.h,v 1.7 2007/02/08 10:22:45 madhu Exp $
 *
 * File Name - netgear_mib.h
 * Purpose   - Callback related enums and data structures.
 */
#ifndef _NETGEAR_MIB_H
#define _NETGEAR_MIB_H

#include <ucd-snmp/mib_debug.h>
#include <ucd-snmp/common_funcs.h>
#include <ucd-snmp/mibgroup/mib_defines.h>
#include <ucd-snmp/mibgroup/ng700ss/netgear_mib_defs.h>

/* 0-5 in eCos, 6 for System,Interface and Dot3, 
   7 for bridge, 8 for enterprise  */
#define SNMPD_CALLBACK_NETGEAR     8

typedef enum ngrmib_id_e {
    /* Host group */
    SS_HOST_INDEX,
    SS_HOST_NAME,
    SS_HOST_IP,
    SS_HOST_COMM,
    SS_HOST_ATTRIBUTE,
    SS_HOST_TRAPSNMPBOOTUP,
    SS_HOST_TRAPSNMPLINKUPDOWN,
    SS_HOST_TRAPTRAPINSERTREMOVE,
    SS_HOST_TRAPTRAPABNORMALTPRXERROR,
    SS_HOST_TRAPTRAPABNORMALTPTXERROR,
    SS_HOST_TRAPTRAPABNORMALFIBERRXERROR,
    SS_HOST_TRAPTRAPABNORMALFIBERTXERROR,
    SS_HOST_ADMIN,
    SS_HOST_STATUS,
    SS_HOST_WRITE,

    /* Igmp */
    SS_IGMP_SNOOPINGCFG,
    SS_IGMP_FILTER_UNKNOWNMULTICAST,

    /* Management VLAN */
    SS_M_VLANID,

    /*Pvlan*/
    SS_PVLAN_ID,
    SS_PVLAN_NAME,
    SS_PVLAN_MEMBER,
    SS_PVLAN_STATUS,
    SS_PVLAN_COUNT,

    /*QOS */
    SS_QOS_BASEMODE,
    SS_QOS_PORTIDVALUE,
    SS_QOS_PORTPRIORITYQUEUE,
    SS_QOS_1PCLASSVALUE,
    SS_QOS_1PPRIORITYQUEUE,
    SS_QOS_PRECEDANCEVALUE,
    SS_QOS_TOSPRIORITYQUEUE,
    SS_QOS_DSCPVALUE,
    SS_DSCP_PRIORITYQUEUE,

    /*Misc Group*/
    SS_MISC_RESET,
    SS_MISC_STATS_RESET,


    /*Spanning Tree */
    SS_SPAN_ONOFF,

    /* Static Multicast */
    SS_STATICMCAST_GROUPID,
    SS_STATICMCAST_GROUPMEMBER,
    SS_STATICMCAST_VLANID,
    SS_STATICMCAST_MAC,
    SS_STATICMCAST_STATUS,
	SS_STATICMCAST_COUNT,

    /* Trust MAC */
    SS_TRUSTEDMAC_GLOBALCFG,
    SS_TRUSTEDID,
    SS_TRUSTEDMAC,
    SS_TRUSTEDPORT,
    SS_TRUSTEDVID,
    SS_TRUSTEDSTATUS,
	SS_TRUSTEDMAC_COUNT,
    
    /* TVLAN */
    SS_TVLAN_ID,
    SS_TVLAN_NAME,
	SS_TVLAN_VID_QUERY,
    SS_TVLAN_MEMBER,
    SS_TVLAN_UNTAGGEDPORTS,
    SS_TVLAN_STATUS,
    SS_TVLAN_PORTPORTID,
    SS_TVLAN_PORTVLANID,
    SS_TVLAN_PORTPRIORITY,
    SS_TVLAN_PORTTAGGEDMODE,
    SS_TVLAN_COUNT,

    /* Trunk */
    SS_PORT_TRUNK_MAX,
    SS_PORT_TRUNK_ID,
    SS_PORT_TRUNK_MEMBER,
    SS_PORT_TRUNK_MEMBERVALUE,
    SS_PORT_TRUNK_ENABLE,

    /* Config */
    SS_CONFIG_VERSWPRIMARY,
    SS_CONFIG_VERHWCHIPSET,
    SS_CONFIG_BOOTTFTPOPERATION,
    SS_CONFIG_BOOTTFTPSERVERIP,
    SS_CONFIG_BOOTIMAGEFILENAME,
    SS_CONFIG_IPASSIGNMENTMODE,
    SS_CONFIG_PHYSADDRESS,
    SS_CONFIG_PASSWORDADMIN,
    SS_CONFIG_IPADDRESS,
    SS_CONFIG_NETMASK,
    SS_CONFIG_GATEWAY,
    SS_CONFIG_SAVE,
    SS_CONFIG_VERBOOTROMIMAGE,
    SS_CONFIG_RESTOREDEFAULTS,
    SS_CONFIG_TFTPSERVERIPADDRESS,
    SS_CONFIG_TFTPSERVERFILENAME,
    SS_CONFIG_TFTPOPERATION,
    SS_CONFIG_USERNAME,//79
    SS_CONFIG_PORT,
    SS_CONFIG_PORTSPEED,
    SS_CONFIG_PORTOPERSTATUS,
    SS_CONFIG_PORTFLOWCONTROL,
    SS_CONFIG_PORTFLOWCONTROLOPER,
    SS_CONFIG_PORTPRIORITY,
    SS_CONFIG_MIRRORID,
    SS_CONFIG_MIRRORMODE,
    SS_CONFIG_MIRRORMON,
    SS_CONFIG_MIRRORSRC,
    SS_CONFIG_MIRRORSTATUS,
    SS_CONFIG_MIRROR_COUNT,
    SS_CONFIG_COUNT,
    SS_CONFIG_VLANMODE,
    SS_CONFIG_IPCOUNT,
    SS_CONFIG_IPFILTER,
    SS_CONFIG_IPFILTERIPADDRESS,
    SS_CONFIG_IPFILTERSTATUS,

    /*Rate Limiting*/
    SS_INGRESSRATESELECT,
    SS_EGRESSRATESELECT,

    /*Storm Control*/
    SS_CONFIGSTORMCONTROLSETTING,
    SS_CONFIGLIMITSELECTION,
    SS_CONFIGLIMITVALUE,
    SS_STORMCONFIGPERPORTCTRLSTATUS,
    SS_STORMCTRLCURRENTRATELIMIT

} ngrmib_id_t;

struct ngrmib_parameters_e { 
    int group;
    #define GROUP_HOST         0
    #define GROUP_IGMPSNOOP    1
    #define GROUP_MGMT_VLAN    2
    #define GROUP_PVLAN        3
    #define GROUP_QOS          4
    #define GROUP_SPANTREE     5
    #define GROUP_STAT_MCAST   6
    #define GROUP_RATE         7
    #define GROUP_TRUST_MAC    8 
    #define GROUP_CONFIG       9 
    #define GROUP_TVLAN        10 
    #define GROUP_TRUNK        11 
    #define GROUP_STROM        12 
    #define GROUP_SSMISC       13
    int callback_rw;
    #define CALLBACK_ACT_READ  0
    #define CALLBACK_ACT_WRITE 1

    ngrmib_id_t group_obj;
    ngr_cbk_param_t group_data;
    int index; /* generic, can be index, port, qos class */
    unsigned int value;
    unsigned char buf[128];
    unsigned char ret_val; /* for error notification */
}; 
typedef struct ngrmib_parameters_e ngrmib_parameters_t;

#endif /* _NETGEAR_MIB_H */
