#ifndef _NETGEAR_MIB_DEFS_H_
#define _NETGEAR_MIB_DEFS_H_

#define MAX_PORTS_SUPPORTED 64

/* Host entry create structure */
struct ngr_host_entry_s {
    unsigned int   hindex;
    char           hname[20];
    unsigned char  hip[4];
    char           hcomm[20];
    unsigned char  hattrib;
    unsigned char  htrapbootup;
    unsigned char  htraplupd;
    unsigned char  htrapinsrem;
    unsigned char  htrapabntprx;
    unsigned char  htrapabntptx;
    unsigned char  htrapabnfbrrx;
    unsigned char  htrapabnfbrtx;
    unsigned char  hadmstat;
    int            hstatus; /*1 or 3 or 5 or 6 */
};
typedef struct ngr_host_entry_s ngr_host_entry_t;

/* Configure TFTP Server create structure */
struct ngr_tftpserver_entry_s {
    int            tftpmode;
    char           file_name[64];
    unsigned char  tftp_ipaddr[4];
};
typedef struct ngr_tftpserver_entry_s ngr_tftpserver_entry_t;

/*configure BOOTTFTP Server Create structure */
struct ngr_boottftpserver_entry_s {
    int           boottftpmode;
    char          file_name[64];
    unsigned char boottftp_ipaddr[4];
};
typedef struct ngr_boottftpserver_entry_s ngr_boottftpserver_entry_t;


/* IP Access list create structure */
struct ngr_ipfilter_entry_s {
    int            cfgindex;
    unsigned char  cfgip[4];
};
typedef struct ngr_ipfilter_entry_s ngr_ipfilter_entry_t;

/* Tagged VLAN create structure */
struct ngr_vlan_entry_s {
    unsigned short vid;
    char           vname[9];
    unsigned char  vports[MAX_PORTS_SUPPORTED/8]; /* 128 port support */
    unsigned char  vuntagports[MAX_PORTS_SUPPORTED/8]; /* untagged ports */
    int            vstatus; /*1 or 3 or 5 or 6 */
};
typedef struct ngr_vlan_entry_s ngr_vlan_entry_t;

/* Port VLAN creation structure */
struct ngr_pvlan_entry_s {
    unsigned short pvid;
    char           pvname[9];
    unsigned char  pvports[MAX_PORTS_SUPPORTED/8]; /* 128 port support */
    int            pvstatus; /*1 or 3 or 5 or 6 */
};
typedef struct ngr_pvlan_entry_s ngr_pvlan_entry_t;

/* Trust MAC entry creation structure */
struct ngr_trustmac_entry_s {
    int            tmid;
    unsigned char  tmac[6];
    int            tmport;
    unsigned short tmvid;
    int            tmstatus;
};
typedef struct ngr_trustmac_entry_s ngr_trustmac_entry_t;

/* Mirror entry create structure */
struct ngr_mirror_entry_s {
   unsigned int    mid;
   unsigned int    mmode;
   unsigned int    mmon_port;
   unsigned char   msrc_plist[MAX_PORTS_SUPPORTED/8];
   int             mstatus;
};
typedef struct ngr_mirror_entry_s ngr_mirror_entry_t;


/* Static Multicast entry creation structure */
struct ngr_statmcast_entry_s {
    int            smgid;
    unsigned char  smports[MAX_PORTS_SUPPORTED/8];
    unsigned short smvid;
    unsigned char  smmac[6];
    int            smstatus;
};
typedef struct ngr_statmcast_entry_s ngr_statmcast_entry_t;

/* Parameter that will be passed to the backend */
union ngr_cbk_param_s {
    ngr_host_entry_t        host;
    ngr_ipfilter_entry_t    ipfilter;
    ngr_pvlan_entry_t       pvlan;
    ngr_vlan_entry_t        tvlan;
    ngr_trustmac_entry_t    trustmac;
    ngr_statmcast_entry_t   smcast;
    ngr_mirror_entry_t      mirror;
    ngr_tftpserver_entry_t  tftpserver;
    ngr_boottftpserver_entry_t  boottftpserver;

};
typedef union ngr_cbk_param_s ngr_cbk_param_t;

#endif /* _NETGEAR_MIB_DEFS_H_ */
