/*
 * $Id: common_funcs.c,v 1.2 2006/12/11 06:29:08 ganesh Exp $
 *
 * Purpose: Contains functions that will be used across MIB groups
 */

#include <config.h>
#include "mibincl.h"

#include <ucd-snmp/mibgroup/mibII/group_mib.h>

#include "mibgroup/ng700ss/ssPVlanGroup.h"
#include "mibgroup/ng700ss/ssStaticMulticastGroup.h"
#include "mibgroup/ng700ss/ssTrustMACGroup.h"
#include "mibgroup/ng700ss/ssTVlanGroup.h"

#include "mibgroup/mibII/dot1qVlan.h"

/* count of number of interfaces on switch */
int interface_count;

void snmp_mib_setInterfaceCount (void)
{
    group_parameters_t        callback_parms;
    memset(&callback_parms, 0, sizeof(group_parameters_t));
    if (!interface_count) {
        callback_parms.group     = GROUP_INTERFACES;
        callback_parms.group_obj = IF_NUMBER;
        callback_parms.callback_rw = CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                            SNMPD_CALLBACK_GROUPS,
                            &callback_parms);
        AGENT_DBG(("Number of ports %d", callback_parms.value));
        interface_count = callback_parms.value;
    }
}

int snmp_mib_getVlanMode (void)
{
    group_parameters_t        callback_parms;
    memset(&callback_parms, 0, sizeof(group_parameters_t));
    callback_parms.group     = GROUP_SYSTEM;
    callback_parms.group_obj = SYSTEM_VLANMODE;
    callback_parms.callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_GROUPS,
                        &callback_parms);
    AGENT_DBG(("VLAN Mode %d", callback_parms.value));
    return callback_parms.value;
}

int snmp_get_bit_position_set(unsigned char  *bitmap, int start, int size)
{
    int    index, position;
    while(start < size)
    {
        index = (start >> 3);
        /* Get the position of the bit */
        position = (start & 0x7);/* To get modulo 8 */
        if((bitmap[index]) & (0x80 >> position)) {
            return ((index * 8) + position);
        }
        start++;
    }
    return -1;
}

int snmp_set_t1_trap (int status)
{
    group_parameters_t        callback_parms;
    memset(&callback_parms, 0, sizeof(group_parameters_t));
    callback_parms.group     = GROUP_SYSTEM;
    callback_parms.value     = (status == 1?1:0);
    callback_parms.group_obj = SYSTEM_SNMPTRAP;
    callback_parms.callback_rw = CALLBACK_ACT_WRITE;
    if(snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_GROUPS,
                        &callback_parms) == 1) {
        AGENT_EXIT;
        return 1;
    }
    return 0;
}

int snmp_get_t1_trap_status (int *status)
{
    group_parameters_t        callback_parms;
    memset(&callback_parms, 0, sizeof(group_parameters_t));
    callback_parms.group     = GROUP_SYSTEM;
    callback_parms.group_obj = SYSTEM_SNMPTRAP;
    callback_parms.callback_rw = CALLBACK_ACT_READ;
    callback_parms.value = 2; /* return disabled by default */
    if(snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_GROUPS,
                        &callback_parms) == 1) {
        AGENT_EXIT;
        return 1;
    }

    *status = callback_parms.value;
    return 0;
}

int snmp_mib_get_stats (int stat_type)
{
    group_parameters_t        callback_parms;
    memset(&callback_parms, 0, sizeof(group_parameters_t));
    
    callback_parms.group     = GROUP_DOT3;
    callback_parms.value     = stat_type;
    callback_parms.group_obj = DOT3_SYSSTATS;
    callback_parms.callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_GROUPS,
                        &callback_parms);
    return callback_parms.value;
}

int netgear_notification_callback(int majorID, int minorID,
                                  void *serverarg, void *clientarg)
{
    /* Free memory allocated during createAndWait */
    free_pvlan_createAndWaitMemory();
    free_tvlan_createAndWaitMemory();
    free_statmcast_createAndWaitMemory();
    free_trustmac_createAndWaitMemory();
    free_dot1qVlan_createAndWaitMemory();
    return 1;
}

int snmp_set_ipforwarding(int value)
{
    group_parameters_t        callback_parms;
    memset(&callback_parms, 0, sizeof(group_parameters_t));
    callback_parms.group     = GROUP_SYSTEM;
    callback_parms.group_obj = SYSTEM_IPFWD;
    callback_parms.callback_rw = CALLBACK_ACT_WRITE;
    callback_parms.value = value;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_GROUPS,
                        &callback_parms);
    return 1;
}

int snmp_get_ipforwarding (int *value)
{
    group_parameters_t        callback_parms;
    memset(&callback_parms, 0, sizeof(group_parameters_t));
    callback_parms.group     = GROUP_SYSTEM;
    callback_parms.group_obj = SYSTEM_IPFWD;
    callback_parms.callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_GROUPS,
                        &callback_parms);
    *value = callback_parms.value;
    return 1;
}

int snmp_get_ipttl(int *value)
{
    group_parameters_t        callback_parms;
    memset(&callback_parms, 0, sizeof(group_parameters_t));
    callback_parms.group     = GROUP_SYSTEM;
    callback_parms.group_obj = SYSTEM_IPTTL;
    callback_parms.callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_GROUPS,
                        &callback_parms);
    *value = callback_parms.value;
    return 1;
}

int snmp_set_ipttl(int value)
{
    group_parameters_t        callback_parms;
    memset(&callback_parms, 0, sizeof(group_parameters_t));
    callback_parms.group     = GROUP_SYSTEM;
    callback_parms.group_obj = SYSTEM_IPTTL;
    callback_parms.callback_rw = CALLBACK_ACT_WRITE;
    callback_parms.value = value;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_GROUPS,
                        &callback_parms);
    return 1;
}

