/*
 * $Id: dot1dBase.c,v 1.1 2006/11/23 14:48:41 jeffc Exp $
 */

/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include <ucd-snmp/mibgroup/mibII/bridge_mib.h>
#include "mibgroup/mibII/dot1dBase.h"

/* callback argument */
static bridge_parameters_t *dot1dbase_parms = NULL;

/* Path to this MIB WRT global MIB tree */
oid dot1dBase_variables_oid[] = { 1,3,6,1,2,1,17,1 };

/* Variables that this MIB supports */
struct variable4 dot1dBase_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define DOT1DBASEBRIDGEADDRESS      1
    {DOT1DBASEBRIDGEADDRESS,  ASN_OCTET_STR,  RONLY ,  var_dot1dBase, 1,  { 1 }},
#define DOT1DBASENUMPORTS       2
    {DOT1DBASENUMPORTS,  ASN_INTEGER,  RONLY ,  var_dot1dBase, 1,  { 2 }},
#define DOT1DBASETYPE       3
    {DOT1DBASETYPE,  ASN_INTEGER,  RONLY ,  var_dot1dBase, 1,  { 3 }},

#define DOT1DBASEPORT       4
    {DOT1DBASEPORT,  ASN_INTEGER,  RONLY,   var_dot1dBasePortTable, 3,  { 4, 1, 1 }},
#define DOT1DBASEPORTIFINDEX        5
    {DOT1DBASEPORTIFINDEX,  ASN_INTEGER,  RONLY,   var_dot1dBasePortTable, 3,  { 4, 1, 2 }},
#define DOT1DBASEPORTCIRCUIT        6
    {DOT1DBASEPORTCIRCUIT,  ASN_OBJECT_ID,  RONLY,   var_dot1dBasePortTable, 3,  { 4, 1, 3 }},
#define DOT1DBASEPORTDELAYEXCEEDEDDISCARDS      7
    {DOT1DBASEPORTDELAYEXCEEDEDDISCARDS,  ASN_COUNTER,  RONLY,   var_dot1dBasePortTable, 3,  { 4, 1, 4 }},
#define DOT1DBASEPORTMTUEXCEEDEDDISCARDS        8
    {DOT1DBASEPORTMTUEXCEEDEDDISCARDS,  ASN_COUNTER,  RONLY,   var_dot1dBasePortTable, 3,  { 4, 1, 5 }},
};

/** Initializes the dot1dBase module */
void
init_dot1dBase(void)
{
    /* Allocate memory */
    dot1dbase_parms = (bridge_parameters_t *) \
                sal_alloc(sizeof(bridge_parameters_t), MIBII_DOT1DBASE_MIB);

    /* let us set the group identifier and register ourselves*/
    if (dot1dbase_parms) {
        dot1dbase_parms->group = BRIDGE_DOT1DBASE;

        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(MIBII_DOT1DBASE_MIB, dot1dBase_variables, variable4,
                   dot1dBase_variables_oid);
    }
}

/* Retrieve global dot1d parameters */
unsigned char *
var_dot1dBase(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    AGENT_ENTER;
    AGENT_OID_PRINT(*length, name);

    static long long_ret;
    static unsigned char string[SPRINT_MAX_LEN];

    /* Validate */
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    /* Get and assign the values. No writable OIDs here */
    switch(vp->magic) {
        case DOT1DBASEBRIDGEADDRESS:
            dot1dbase_parms->group_obj = DOT1D_BASEBRIDGEADDRESS;
            break;

        case DOT1DBASENUMPORTS:
            /* This will be equal to number of ports, but let us get from
               backend for future modification support */
            dot1dbase_parms->group_obj = DOT1D_BASENUMPORTS;
            break;

        case DOT1DBASETYPE:
            dot1dbase_parms->group_obj = DOT1D_BASETYPE;
            break;

        default:
          ERROR_MSG("Undefined MIB reference in dot1dBase");
    	  AGENT_EXIT;
	  return NULL;
    }

    /* Callback */
    dot1dbase_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1dbase_parms);
    
    switch(vp->magic) {
        case DOT1DBASEBRIDGEADDRESS:
            *var_len = MAC_ADDRESS_LEN;/* Copy 6 bytes */
            memcpy (string, dot1dbase_parms->buf, *var_len);
            return (unsigned char *) string;
    }

    /* Give the value back */
    long_ret = (long)dot1dbase_parms->value;
    return (unsigned char *)&long_ret;
}

/*
 * Validation function for dot1dBasePortEntry
 */
static int
header_dot1dBasePortEntry(struct variable *vp,
           oid *name,
           size_t *length,
           int exact,
           size_t *var_len,
           WriteMethod **write_method)
{
#define DOT1DBASEPORTENTRY_NAME_LENGTH  11
    AGENT_ENTER;
    oid newname[MAX_OID_LEN];
    int i, result;
    int port_number;
    int found = FALSE;

    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    for(i = 1; i <= interface_count; i++) {
        /* Build an OID with this port, and loop */
        newname[DOT1DBASEPORTENTRY_NAME_LENGTH] = (oid)i;
        result = snmp_oid_compare(
                    name, *length, newname, (int)vp->namelen + 1);
        if ((exact && (result == 0)) || (!exact && (result < 0))) {
            found = TRUE;
            AGENT_DBG(("Found a match at %d", i));
            port_number = i;
            break;
        }
    }

    /* TODO: Get AGG port info and loop through that also? */

    /* Let us return if we have not found anything */
    if ( found == FALSE ) {
        DEBUGMSGTL(("dot1dBase", "Invalid index\n"));
        AGENT_EXIT;
        return MATCH_FAILED;
    }

    /* Set up the OID */
    memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));

    *length = vp->namelen + 1;
    *write_method = 0;
    *var_len = sizeof(long);

    AGENT_EXIT;
    return port_number;
}


/*
 * var_dot1dBasePortTable(): This function provides access to values stored 
 * in tabular format.
 */
unsigned char *
var_dot1dBasePortTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    AGENT_ENTER;
    AGENT_OID_PRINT(*length, name);
    static long long_ret;

    /* Get number of ports on the switch */
    snmp_mib_setInterfaceCount();
    AGENT_DBG(("Interface count %d", interface_count));

    /* Validate */
    if ((dot1dbase_parms->index =
                header_dot1dBasePortEntry(vp,name,length,exact,
                                    var_len,write_method)) == MATCH_FAILED) {
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
        case DOT1DBASEPORT:
            /* TODO: We need to do something for AGG port? */
            dot1dbase_parms->group_obj = DOT1D_BASEPORT;
            break;

        case DOT1DBASEPORTIFINDEX:
            dot1dbase_parms->group_obj = DOT1D_BASEPORTIFINDEX;
            break;

        case DOT1DBASEPORTCIRCUIT:
            dot1dbase_parms->group_obj = DOT1D_BASEPORTCIRCUIT;
            break;

        case DOT1DBASEPORTDELAYEXCEEDEDDISCARDS:
            dot1dbase_parms->group_obj = DOT1D_BASEPORTDELAYEXCEEDEDDISCARDS;
            break;

        case DOT1DBASEPORTMTUEXCEEDEDDISCARDS:
            dot1dbase_parms->group_obj = DOT1D_BASEPORTMTUEXCEEDEDDISCARDS;
            break;

        default:
            ERROR_MSG("Undefined MIB access in dot1dPortTable");
            break;
    }

    /* Callback */
    dot1dbase_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1dbase_parms);
    
    /* Give the value back */
    long_ret = (long)dot1dbase_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}


