/*
 * $Id: dot1dExtBase.c,v 1.1 2006/11/23 14:48:41 jeffc Exp $
 */

/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include <ucd-snmp/mibgroup/mibII/bridge_mib.h>
#include "mibgroup/mibII/dot1dExtBase.h"

/* Callback parameter passed to application */
static bridge_parameters_t *dot1dextbase_parms = NULL;

/* Path to this MIB WRT the global MIB tree */
oid dot1dExtBase_variables_oid[] = { 1,3,6,1,2,1,17,6,1,1 };

/* Variables that this MIB supports */
struct variable4 dot1dExtBase_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define DOT1DDEVICECAPABILITIES     1
    {DOT1DDEVICECAPABILITIES,  ASN_OCTET_STR,  RONLY ,  var_dot1dExtBase, 1,  { 1 }},
#define DOT1DTRAFFICCLASSESENABLED  2
    {DOT1DTRAFFICCLASSESENABLED,  ASN_INTEGER,  RWRITE,  var_dot1dExtBase, 1,  { 2 }},
#define DOT1DGMRPSTATUS             3
    {DOT1DGMRPSTATUS,  ASN_INTEGER,  RWRITE,  var_dot1dExtBase, 1,  { 3 }},
#define DOT1DPORTCAPABILITIES       4
    {DOT1DPORTCAPABILITIES,  ASN_OCTET_STR,  RONLY,   var_dot1dPortCapabilitiesTable, 3,  { 4, 1, 1 }},
};

/** Initializes the dot1dExtBase module */
void
init_dot1dExtBase(void)
{
    AGENT_ENTER;

    dot1dextbase_parms = (bridge_parameters_t *) \
                sal_alloc(sizeof(bridge_parameters_t),MIBII_DOT1DEXTBASE_MIB); 

    if (dot1dextbase_parms) {
        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(MIBII_DOT1DEXTBASE_MIB, dot1dExtBase_variables, variable4,
                   dot1dExtBase_variables_oid);

        dot1dextbase_parms->group = BRIDGE_DOT1DEXTBASE; 
    }

    AGENT_EXIT;
}

/*
 * var_dot1dExtBase():
 *   This function is called when a Get/Set is done on
 *   a MIB object. */
unsigned char *
var_dot1dExtBase(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
#define C_BYTE_LEN 1
    AGENT_ENTER;
    AGENT_OID_PRINT(*length, name);

    static long long_ret;
    static unsigned char byte;

    /* Validate */
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
        case DOT1DDEVICECAPABILITIES:
            dot1dextbase_parms->group_obj = DOT1D_DEVICECAPABILITIES;
            break;

        case DOT1DTRAFFICCLASSESENABLED:
            *write_method = write_dot1dTrafficClassesEnabled;
            dot1dextbase_parms->group_obj = DOT1D_TRAFFICCLASSESENABLED;
            break;

        case DOT1DGMRPSTATUS:
            *write_method = write_dot1dGmrpStatus;
            dot1dextbase_parms->group_obj = DOT1D_GMRPSTATUS;
            break;

        default:
            ERROR_MSG("Undefined MIB reference in dot1dExtbase Table");
            AGENT_EXIT;
            return NULL;
    }

    /* Callback */
    dot1dextbase_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1dextbase_parms);

    /* Return values back */
    switch(vp->magic) {
        case DOT1DDEVICECAPABILITIES:
            *var_len = C_BYTE_LEN;
            byte = (unsigned char)dot1dextbase_parms->value;
            return (u_char *)&byte;
    }
    
    long_ret = (long)dot1dextbase_parms->value;
    AGENT_EXIT;
    return (u_char *)&long_ret;
}

/*
 * Validate dot1dPortCapabilitiesTable 
 */
static int
header_dot1dPortCapabilitiesTable(struct variable *vp,
                                   oid *name,
                                   size_t *length,
                                   int exact,
                                   size_t *var_len,
                                   WriteMethod **write_method)
{
#define DOT1DPORTCAP_ENTRY_LEN  13
    AGENT_ENTER;
    oid newname[MAX_OID_LEN];
    int i, result;
    int found = FALSE;
    int port_number;

    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    for(i = 1; i <= interface_count; i++) {
        newname[DOT1DPORTCAP_ENTRY_LEN] = (oid)i;
        result = snmp_oid_compare(name, *length, newname, (int)vp->namelen + 1);
        if ((exact && (result == 0)) || (!exact && (result < 0))) {
            AGENT_DBG(("Found at index %d", i));
            found = TRUE;
            port_number = i;
            break;
        }
    }

    /* TODO: Get Agg port information? */

    if ( found == FALSE ) {
        DEBUGMSGTL(("dot1dExtBase", "... index out of range\n"));
        AGENT_EXIT;
        return MATCH_FAILED;
    }

    memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    *length = vp->namelen + 1;

    *write_method = 0; /* No write method */
    *var_len = sizeof(long);

    AGENT_EXIT;
    return port_number;
}


/*
 * var_dot1dPortCapabilitiesTable():
 *  This function contains Multi-Index tables
 *   The workings of this are basically the same as for var_dot1dExtBase above.
 */
unsigned char *
var_dot1dPortCapabilitiesTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    AGENT_ENTER;
    static unsigned char byte;

    /* Get number of ports on the switch */
    snmp_mib_setInterfaceCount();

    /* Validate */
    if (header_dot1dPortCapabilitiesTable(vp,name,length,exact,var_len,
                                        write_method)  == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

  /* Start the MIB assignments */
    switch(vp->magic) {
        case DOT1DPORTCAPABILITIES:
            dot1dextbase_parms->group_obj = DOT1D_PORTCAPABILITIES;
            break;
        default:
            ERROR_MSG("Undefined MIB reference in Ext Port Capab table");
            AGENT_EXIT;
            return NULL;
    }

    /* Callback */
    dot1dextbase_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1dextbase_parms);

    AGENT_EXIT;
    byte = (unsigned char) dot1dextbase_parms->value;
    *var_len = C_BYTE_LEN; /* All in one byte */
    return (u_char *)&byte;
}


int
write_dot1dTrafficClassesEnabled(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    AGENT_ENTER;
    int value = 0;

    dot1dextbase_parms->group_obj = DOT1D_TRAFFICCLASSESENABLED;
    if (var_val) {
        value = * (int *) var_val;
    }

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to dot1dExtBase not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to dot1dExtBase: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          if (value != MIB_STATUS_ENABLED && value != MIB_STATUS_DISABLED) {
              fprintf(stderr,"write to dot1dExtBase: bad value\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
            dot1dextbase_parms->value = value;
            dot1dextbase_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                                SNMPD_CALLBACK_BRIDGE,
                                dot1dextbase_parms);
            if (dot1dextbase_parms->rv == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
          break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


int
write_dot1dGmrpStatus(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    int value = 0;

    if (var_val) {
        value = * (int *) var_val;
    }

    dot1dextbase_parms->group_obj = DOT1D_GMRPSTATUS;
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to dot1dExtBase not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to dot1dExtBase: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          if (value != MIB_STATUS_ENABLED && value != MIB_STATUS_DISABLED) {
              fprintf(stderr,"write to dot1dExtBase: bad value\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
            dot1dextbase_parms->value = value;
            dot1dextbase_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                                SNMPD_CALLBACK_BRIDGE,
                                dot1dextbase_parms);
            if (dot1dextbase_parms->rv == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }

          break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

