/*
 * $Id: dot1dGarp.c,v 1.1 2006/11/23 14:48:41 jeffc Exp $
 */
/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include "mibgroup/mibII/dot1dGarp.h"
#include <ucd-snmp/mibgroup/mibII/bridge_mib.h>

/* Argument to callback */
static bridge_parameters_t *dot1dgarp_parms = NULL;

/* Path to this MIB WRT the global MIB tree */
oid dot1dGarp_variables_oid[] = { 1,3,6,1,2,1,17,6,1,3 };

/* Variables that this MIB supports */
struct variable4 dot1dGarp_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define DOT1DPORTGARPJOINTIME       1
    {DOT1DPORTGARPJOINTIME,  ASN_INTEGER,  RWRITE,  var_dot1dPortGarpTable, 3,  { 1, 1, 1 }},
#define DOT1DPORTGARPLEAVETIME      2
    {DOT1DPORTGARPLEAVETIME,  ASN_INTEGER,  RWRITE,  var_dot1dPortGarpTable, 3,  { 1, 1, 2 }},
#define DOT1DPORTGARPLEAVEALLTIME       3
    {DOT1DPORTGARPLEAVEALLTIME,  ASN_INTEGER,  RWRITE,  var_dot1dPortGarpTable, 3,  { 1, 1, 3 }},
};

/* Initializes the dot1dGarp module */
void
init_dot1dGarp(void)
{
    AGENT_ENTER;

    /* BRCM SNMP init */
    dot1dgarp_parms = (bridge_parameters_t *) \
                sal_alloc(sizeof(bridge_parameters_t), MIBII_DOT1DGARP_MIB);

    if (dot1dgarp_parms) {
        dot1dgarp_parms->group = BRIDGE_DOT1DGARP;

        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(MIBII_DOT1DGARP_MIB, dot1dGarp_variables, variable4,
                   dot1dGarp_variables_oid);
    }

    AGENT_EXIT;
}

/*
 * var_dot1dGarp(): Global values access table
 */
unsigned char *
var_dot1dGarp(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{

    AGENT_ENTER;
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                                    == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
        default:
          ERROR_MSG("--");
    }
    
    AGENT_EXIT;
    return NULL;
}

/*
 * Validation function
 */
static int
header_dot1dGarpEntry(struct variable *vp,
           oid *name,
           size_t *length,
           int exact,
           size_t *var_len,
           WriteMethod **write_method)
{
#define DOT1DGARPENTRY_LENGTH   13
    AGENT_ENTER;
    oid newname[MAX_OID_LEN];
    int i, result;
    int port_number;
    int found = FALSE;

    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
    for(i = 1; i <= interface_count; i++) {
        newname[DOT1DGARPENTRY_LENGTH] = (oid)i;
        result = snmp_oid_compare(
                name, *length, newname, (int)vp->namelen + 1);
        if ((exact && (result == 0)) || (!exact && (result < 0))) {
            AGENT_DBG(("Found port %d", i));
            found = TRUE;
            port_number = i;
            break;
        }
    }

    /* TODO: AGG ports? */

    if ( found == FALSE ) {
        DEBUGMSGTL(("bridge/dot1dGarp", "... index out of range\n"));
        AGENT_EXIT;
        return MATCH_FAILED;
    }


    memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    *length = vp->namelen + 1;
    *var_len = sizeof(long);

    AGENT_EXIT;
    return port_number;
}

/*
 * var_dot1dPortGarpTable():
 *  This function contains Multi-Index tables */
unsigned char *
var_dot1dPortGarpTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    static long long_ret;
    AGENT_ENTER;

    /* Validate table */
    snmp_mib_setInterfaceCount();
    if ((dot1dgarp_parms->index = 
                    header_dot1dGarpEntry(vp,name,length,exact,var_len,
                        write_method)) == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    /* Start assignments */
    switch(vp->magic) {
        case DOT1DPORTGARPJOINTIME:
            *write_method = write_dot1dPortGarpJoinTime;
            dot1dgarp_parms->group_obj = DOT1D_PORTGARPJOINTIME;
            break;

        case DOT1DPORTGARPLEAVETIME:
            *write_method = write_dot1dPortGarpLeaveTime;
            dot1dgarp_parms->group_obj = DOT1D_PORTGARPLEAVETIME;
            break;

        case DOT1DPORTGARPLEAVEALLTIME:
            *write_method = write_dot1dPortGarpLeaveAllTime;
            dot1dgarp_parms->group_obj = DOT1D_PORTGARPLEAVEALLTIME;
            break;

        default:
            ERROR_MSG("Undefined MIB reference");
            AGENT_EXIT;
            return NULL;
    }

    /* Callback */
    dot1dgarp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1dgarp_parms);
    
    /* Give the value back */
    long_ret = (long)dot1dgarp_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}

int
write_dot1dPortGarpJoinTime(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    AGENT_ENTER;
    int value = 0;

    if (var_val) {
        value = * (int *) var_val;
    }

    dot1dgarp_parms->group_obj = DOT1D_PORTGARPJOINTIME;
    dot1dgarp_parms->index = (int) name[ DOT1DGARPENTRY_LENGTH ];
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to dot1dGarp not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }

          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to dot1dGarp: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }

          if (value < 1 || value > 20) {
              fprintf(stderr,"write to dot1dGarp: bad value\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
            dot1dgarp_parms->value = value;
            dot1dgarp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                                SNMPD_CALLBACK_BRIDGE,
                                dot1dgarp_parms);
            if (dot1dgarp_parms->rv == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
          break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


int
write_dot1dPortGarpLeaveTime(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    AGENT_ENTER;
    int value = 0;

    if (var_val) {
        value = * (int *) var_val;
    }

    dot1dgarp_parms->group_obj = DOT1D_PORTGARPLEAVETIME;
    dot1dgarp_parms->index = (int) name[ DOT1DGARPENTRY_LENGTH ];
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to dot1dGarp not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to dot1dGarp: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          if (value < 1 || value > 60) {
              fprintf(stderr,"write to dot1dGarp: bad value\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
            dot1dgarp_parms->value = value;
            dot1dgarp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                                SNMPD_CALLBACK_BRIDGE,
                                dot1dgarp_parms);
            if (dot1dgarp_parms->rv == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
          break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


int
write_dot1dPortGarpLeaveAllTime(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    AGENT_ENTER;
    int value = 0;

    if (var_val) {
        value = * (int *) var_val;
    }

    dot1dgarp_parms->group_obj = DOT1D_PORTGARPLEAVEALLTIME;
    dot1dgarp_parms->index = (int) name[ DOT1DGARPENTRY_LENGTH ];
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to dot1dGarp not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to dot1dGarp: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          if (value < 1 || value > 1000) {
              fprintf(stderr,"write to dot1dGarp: bad value\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
            dot1dgarp_parms->value = value;
            dot1dgarp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                                SNMPD_CALLBACK_BRIDGE,
                                dot1dgarp_parms);
            if (dot1dgarp_parms->rv == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
          break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}
