/*
 * $Id: dot1dGmrp.c,v 1.1 2006/11/23 14:48:41 jeffc Exp $
 */

/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include "mibgroup/mibII/dot1dGmrp.h"
#include <ucd-snmp/mibgroup/mibII/bridge_mib.h>

/* Argument to callback */
static bridge_parameters_t *dot1dgmrp_parms = NULL;

/* Path to this MIB WRT the global MIB tree */
oid dot1dGmrp_variables_oid[] = { 1,3,6,1,2,1,17,6,1,4 };

/* Variables that this MIB supports */
struct variable4 dot1dGmrp_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

#define DOT1DPORTGMRPSTATUS     1
    {DOT1DPORTGMRPSTATUS,  ASN_INTEGER,  RWRITE,  var_dot1dPortGmrpTable, 3,  { 1, 1, 1 }},
#define DOT1DPORTGMRPFAILEDREGISTRATIONS        2
    {DOT1DPORTGMRPFAILEDREGISTRATIONS,  ASN_COUNTER,  RONLY,   var_dot1dPortGmrpTable, 3,  { 1, 1, 2 }},
#define DOT1DPORTGMRPLASTPDUORIGIN      3
    {DOT1DPORTGMRPLASTPDUORIGIN,  ASN_OCTET_STR,  RONLY,   var_dot1dPortGmrpTable, 3,  { 1, 1, 3 }},
};

/** Initializes the dot1dGmrp module */
void
init_dot1dGmrp(void)
{
    AGENT_ENTER;

    dot1dgmrp_parms = (bridge_parameters_t *) \
                sal_alloc(sizeof(bridge_parameters_t), MIBII_DOT1DGMRP_MIB);

    if (dot1dgmrp_parms) {
        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(MIBII_DOT1DGMRP_MIB, dot1dGmrp_variables, variable4,
                   dot1dGmrp_variables_oid);

        dot1dgmrp_parms->group = BRIDGE_DOT1DGMRP; 
    }

    AGENT_EXIT;
}

/*
 * var_dot1dGmrp(): GMRP top level global configs.
 */
unsigned char *
var_dot1dGmrp(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    AGENT_ENTER;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ) {
        return NULL;
    }

    AGENT_EXIT;
    return NULL;
}

/*
 * Validate GmrpEntry request
 */
static int
header_dot1dGmrpEntry(struct variable *vp,
           oid *name,
           size_t *length,
           int exact,
           size_t *var_len,
           WriteMethod **write_method)
{
#define DOT1DGMRPENTRY_LENGTH   13
    AGENT_ENTER;
    oid newname[MAX_OID_LEN];
    int i, result;
    int port_number;
    int found = FALSE;

    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
    for(i = 1; i <= interface_count; i++ ) {
        newname[DOT1DGMRPENTRY_LENGTH] = (oid)i;
        result = snmp_oid_compare(
                name, *length, newname, (int)vp->namelen + 1);
        if ((exact && (result == 0)) || (!exact && (result < 0))) {
            found = TRUE;
            port_number = i;
            break;
        }
    }

    /* TODO: AGG port info? */

    if ( found == FALSE ) {
        DEBUGMSGTL(("dot1dGmrp", "... index out of range\n"));
        AGENT_EXIT;
        return MATCH_FAILED;
    }

    memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    *length = vp->namelen + 1;
    *write_method = 0;
    *var_len = sizeof(long);    /* default to 'long' results */

    AGENT_EXIT;
    return port_number;
}


/*
 * var_dot1dPortGmrpTable(): Table entry configurations.
 */
unsigned char *
var_dot1dPortGmrpTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    AGENT_ENTER;
    static long long_ret;
    static unsigned char string[SPRINT_MAX_LEN];

    /* Validate table */
    snmp_mib_setInterfaceCount();
    if ((dot1dgmrp_parms->index = header_dot1dGmrpEntry(vp,name,length,exact,
                                    var_len,write_method)) == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
        case DOT1DPORTGMRPSTATUS:
            *write_method = write_dot1dPortGmrpStatus;
            dot1dgmrp_parms->group_obj = DOT1D_PORTGMRPSTATUS;
            break;

        case DOT1DPORTGMRPFAILEDREGISTRATIONS:
            dot1dgmrp_parms->group_obj = DOT1D_PORTGMRPFAILEDREGISTRATIONS;
            break;

        case DOT1DPORTGMRPLASTPDUORIGIN:
            dot1dgmrp_parms->group_obj = DOT1D_PORTGMRPLASTPDUORIGIN;
            break;

        default:
            ERROR_MSG("Undefined MIB reference");
            AGENT_EXIT;
            return NULL;
    }

    /* Callback */
    dot1dgmrp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1dgmrp_parms);

    /* Give the value back */
    switch(vp->magic) {
        case DOT1DPORTGMRPLASTPDUORIGIN:
            memcpy (string, dot1dgmrp_parms->buf, 6);
            string [6] = 0;
            AGENT_EXIT;
            return (u_char *) string;
    }

    long_ret = (long)dot1dgmrp_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}


int
write_dot1dPortGmrpStatus(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    int value = 0;

    if (var_val) {
        value = * (int *) var_val;
    }

    dot1dgmrp_parms->group_obj = DOT1D_PORTGMRPSTATUS;
    dot1dgmrp_parms->index = (int) name[DOT1DGMRPENTRY_LENGTH];
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to dot1dGmrp not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to dot1dGmrp: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          if (value != 1 && value  != 2) {
              fprintf(stderr,"write to dot1dGmrp: bad value\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
            dot1dgmrp_parms->value = value;
            dot1dgmrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                                SNMPD_CALLBACK_BRIDGE,
                                dot1dgmrp_parms);
            if (dot1dgmrp_parms->rv == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
          break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}
