/*
 * $Id: dot1qBase.c,v 1.1 2006/11/23 14:48:41 jeffc Exp $
 */

/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include "mibgroup/mibII/dot1qBase.h"
#include <ucd-snmp/mibgroup/mibII/bridge_mib.h>

/* parameter to the callback function */
static bridge_parameters_t *dot1qbase_parms = NULL;

/* Path to this MIB WRT the global MIB tree */
oid dot1qBase_variables_oid[] = { 1,3,6,1,2,1,17,7,1,1 };

/* variables that this MIB supports */
struct variable4 dot1qBase_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define DOT1QVLANVERSIONNUMBER		1
    {DOT1QVLANVERSIONNUMBER,  ASN_INTEGER,  RONLY ,  var_dot1qBase, 1,  { 1 }},
#define DOT1QMAXVLANID		2
    {DOT1QMAXVLANID,  ASN_INTEGER,  RONLY ,  var_dot1qBase, 1,  { 2 }},
#define DOT1QMAXSUPPORTEDVLANS		3
    {DOT1QMAXSUPPORTEDVLANS,  ASN_UNSIGNED,  RONLY ,  var_dot1qBase, 1,  { 3 }},
#define DOT1QNUMVLANS		4
    {DOT1QNUMVLANS,  ASN_UNSIGNED,  RONLY ,  var_dot1qBase, 1,  { 4 }},
#define DOT1QGVRPSTATUS		5
    {DOT1QGVRPSTATUS,  ASN_INTEGER,  RWRITE,  var_dot1qBase, 1,  { 5 }},
};

/** Initializes the dot1qBase module */
void
init_dot1qBase(void)
{
    /* Allocate memory */
    dot1qbase_parms = (bridge_parameters_t *) \
                sal_alloc(sizeof(bridge_parameters_t), MIBII_DOT1QBASE_MIB);

    if (dot1qbase_parms != NULL) {
        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(MIBII_DOT1QBASE_MIB, dot1qBase_variables, variable4,
                     dot1qBase_variables_oid);
        
        dot1qbase_parms->group = BRIDGE_DOT1QBASE; 
    }
}

/*
 * var_dot1qBase():
 */
unsigned char *
var_dot1qBase(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static u_long ulong_ret;

    /* Proceed only if 802.1Q VLAN, not if port vlan */
    if (snmp_mib_getVlanMode() != MIB_VLAN_MODE_TAGGED) {
        AGENT_EXIT;
        return NULL;
    }

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    /* Start the MIB assignments */
    switch(vp->magic) {
        case DOT1QVLANVERSIONNUMBER:
            dot1qbase_parms->group_obj = DOT1Q_VLANVERSIONNUMBER;
            break;

        case DOT1QMAXVLANID:
            dot1qbase_parms->group_obj = DOT1Q_MAXVLANID;
            break;

        case DOT1QMAXSUPPORTEDVLANS:
            dot1qbase_parms->group_obj = DOT1Q_MAXSUPPORTEDVLANS;
            break;

        case DOT1QNUMVLANS:
            dot1qbase_parms->group_obj = DOT1Q_NUMVLANS;
            break;

        case DOT1QGVRPSTATUS:
            *write_method = write_dot1qGvrpStatus;
            dot1qbase_parms->group_obj = DOT1Q_GVRPSTATUS;
            break;

        default:
            ERROR_MSG("Undefined qBase MIb reference");
    }

    dot1qbase_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1qbase_parms);
	
	/* Give the value back */
	ulong_ret = (unsigned long)dot1qbase_parms->value;
    return (unsigned char *)&ulong_ret;
}

int
write_dot1qGvrpStatus(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    int value = 0;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to dot1qBase not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to dot1qBase: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          value = (* (int *) var_val);
          if (value != 1 && value != 2) {
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
          /* For now, not writable */
          return SNMP_ERR_NOTWRITABLE;
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          dot1qbase_parms->value = (* (int *) var_val);
          dot1qbase_parms->group_obj = DOT1Q_GVRPSTATUS;
          dot1qbase_parms->callback_rw = CALLBACK_ACT_WRITE;
          snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                  SNMPD_CALLBACK_BRIDGE,
                                  dot1qbase_parms);
          if (dot1qbase_parms->rv == MIB_STATUS_ERROR) {
              AGENT_EXIT;
              return SNMP_ERR_COMMITFAILED;
          }
          break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

