/*
 * $Id: dot1qStatic.c,v 1.1 2006/11/23 14:48:41 jeffc Exp $
 */

/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include "mibgroup/mibII/dot1qStatic.h"
#include <ucd-snmp/mibgroup/mibII/bridge_mib.h>

/* Argument that will be passed to the callback function */
static bridge_parameters_t *dot1qstatic_parms = NULL;

/* Path to this MIB WRT the global MIB tree */
oid dot1qStatic_variables_oid[] = { 1,3,6,1,2,1,17,7,1,3 };

/* Index of the entry for which we got a hit */
static int qstatic_entry_num;

/* Multicast entry for which we got a hit */
static int qmstatic_entry_num;

/* We just save all info that we get during recursion */
static qbridge_static_t stat_mcparam;

/* Variables that this MIB supports*/
struct variable4 dot1qStatic_variables[] = {
#define DOT1QSTATICUNICASTADDRESS       1
    {DOT1QSTATICUNICASTADDRESS,  ASN_OCTET_STR,  RONLY,   var_dot1qStaticUnicastTable, 3,  { 1, 1, 1 }},
#define DOT1QSTATICUNICASTRECEIVEPORT       2
    {DOT1QSTATICUNICASTRECEIVEPORT,  ASN_INTEGER,  RONLY,   var_dot1qStaticUnicastTable, 3,  { 1, 1, 2 }},
#define DOT1QSTATICUNICASTALLOWEDTOGOTO     3
    {DOT1QSTATICUNICASTALLOWEDTOGOTO,  ASN_OCTET_STR,  RWRITE,  var_dot1qStaticUnicastTable, 3,  { 1, 1, 3 }},
#define DOT1QSTATICUNICASTSTATUS        4
    {DOT1QSTATICUNICASTSTATUS,  ASN_INTEGER,  RWRITE,  var_dot1qStaticUnicastTable, 3,  { 1, 1, 4 }},
#define DOT1QSTATICMULTICASTADDRESS     5
    {DOT1QSTATICMULTICASTADDRESS,  ASN_OCTET_STR,  RONLY,   var_dot1qStaticMulticastTable, 3,  { 2, 1, 1 }},
#define DOT1QSTATICMULTICASTRECEIVEPORT     6
    {DOT1QSTATICMULTICASTRECEIVEPORT,  ASN_INTEGER,  RONLY,   var_dot1qStaticMulticastTable, 3,  { 2, 1, 2 }},
#define DOT1QSTATICMULTICASTSTATICEGRESSPORTS       7
    {DOT1QSTATICMULTICASTSTATICEGRESSPORTS,  ASN_OCTET_STR,  RWRITE,  var_dot1qStaticMulticastTable, 3,  { 2, 1, 3 }},
#define DOT1QSTATICMULTICASTFORBIDDENEGRESSPORTS        8
    {DOT1QSTATICMULTICASTFORBIDDENEGRESSPORTS,  ASN_OCTET_STR,  RWRITE,  var_dot1qStaticMulticastTable, 3,  { 2, 1, 4 }},
#define DOT1QSTATICMULTICASTSTATUS      9
    {DOT1QSTATICMULTICASTSTATUS,  ASN_INTEGER,  RWRITE,  var_dot1qStaticMulticastTable, 3,  { 2, 1, 5 }},
};

/** Initializes the dot1qStatic module */
void
init_dot1qStatic(void)
{
    AGENT_ENTER;

    /* Allocate memory */
    dot1qstatic_parms = (bridge_parameters_t *) \
                sal_alloc(sizeof(bridge_parameters_t), MIBII_DOT1QSTATIC_MIB);

    if (dot1qstatic_parms != NULL) { 
        dot1qstatic_parms->group = BRIDGE_DOT1QSTATIC;

        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(MIBII_DOT1QSTATIC_MIB, dot1qStatic_variables, variable4,
               dot1qStatic_variables_oid);
    }

    AGENT_EXIT;
}

/*
 * var_dot1qStatic():
 */
unsigned char *
var_dot1qStatic(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    AGENT_ENTER;
    
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    AGENT_EXIT;
    return NULL;
}

/*
 * Validate the MAC Address
 */
static int
header_dot1qStaticEntry(struct variable *vp,
           oid *name,
           size_t *length,
           int exact,
           size_t *var_len,
           WriteMethod **write_method)
{
#define DOT1QSTATICENTRY_NAME_LENGTH	13
#define DOT1QSTATIC_TOTAL_OID_LENGTH    21
#define DOT1QSTATIC_FDB_ID               1
    AGENT_ENTER;
    oid newname[MAX_OID_LEN];
    oid lowest[MAX_OID_LEN];
    int i, result, found = FALSE;
    int num_mac_addr;
    unsigned char mac[MAC_ADDRESS_LEN];
    int j = 0;

    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
    *write_method = 0; /* No write */
    lowest[0] = 99999; /* For first time walk */

    /* Set the Fdb Id to 1 */
    newname[DOT1QSTATICENTRY_NAME_LENGTH] = DOT1QSTATIC_FDB_ID;

    /* Set the entry number to an invalid index at the start */
    qstatic_entry_num = -1;

    /* Get count of number of L2 entries */
    dot1qstatic_parms->group =  BRIDGE_DOT1DSTATIC;

    /* dump l2 entries first */
    dot1qstatic_parms->group_obj = DOT1D_STATICADDRESSDUMP;
    dot1qstatic_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1qstatic_parms);

    dot1qstatic_parms->group_obj = DOT1D_STATICADDRESSCOUNT;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1qstatic_parms);
    num_mac_addr = dot1qstatic_parms->value;
    AGENT_DBG(("Num static address %d", num_mac_addr));

    /* Now, get the entry by index and validate request */
    for (i = 0; i < num_mac_addr; i++) {
        /* get the mac address by index */
        dot1qstatic_parms->index = i;
        dot1qstatic_parms->group_obj = DOT1D_STATICADDRESS;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                            SNMPD_CALLBACK_BRIDGE,
                            dot1qstatic_parms);
        memcpy (mac, dot1qstatic_parms->buf, MAC_ADDRESS_LEN);

        /* Not interested in multicast entries */
        if ( (mac[0] & 0x1)) {
            continue;
        }

        for (j = 0; j < MAC_ADDRESS_LEN; j++)  { /* 6 octets for MAC Address */
            newname[DOT1QSTATICENTRY_NAME_LENGTH + j + 1] = mac[j];
        }

        /* get the port associated with the mac address */
        dot1qstatic_parms->group_obj = DOT1D_STATICRECEIVEPORT;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                            SNMPD_CALLBACK_BRIDGE,
                            dot1qstatic_parms);

        /* TODO: Check if port is zero based or 1 based */
        newname[DOT1QSTATICENTRY_NAME_LENGTH+MAC_ADDRESS_LEN+1] = 
                                        (oid)dot1qstatic_parms->value;

        /* if it's a SET/GET request, exact is true. So return only if there
         * is an exact match or snmp_oid_compare returns 0
         */
        if (exact) {
            if (snmp_oid_compare(name, *length, newname, 
                        DOT1QSTATIC_TOTAL_OID_LENGTH) == 0) {
                memcpy(lowest, newname, 
                                DOT1QSTATIC_TOTAL_OID_LENGTH * sizeof(oid));
                found = TRUE;
                qstatic_entry_num = i;
                break;
            }
        } else {
            if (snmp_oid_compare(newname, DOT1QSTATIC_TOTAL_OID_LENGTH, 
                        name, *length) > 0 &&
                snmp_oid_compare(newname, DOT1QSTATIC_TOTAL_OID_LENGTH, lowest, 
                    DOT1QSTATIC_TOTAL_OID_LENGTH) < 0) {
                memcpy((char *) lowest, (char *) newname,
                       DOT1QSTATIC_TOTAL_OID_LENGTH * sizeof(oid));
                found = TRUE;
                qstatic_entry_num = i;
            }
        }
    }
    dot1qstatic_parms->group =  BRIDGE_DOT1QSTATIC;
    
    if (!found) {
        AGENT_DBG (("Invalid index"));
        return MATCH_FAILED;
    }

    memcpy( (char *)name,(char *)lowest, DOT1QSTATIC_TOTAL_OID_LENGTH * 
                                                               sizeof(oid));
    *length = DOT1QSTATIC_TOTAL_OID_LENGTH;
    *var_len = sizeof(long);

    AGENT_EXIT;
    return MATCH_SUCCEEDED;
}

/*
 * var_dot1qStaticUnicastTable():
 *  This function contains Multi-Index tables
 *   The workings of this are basically the same as for var_dot1qStatic above.
 */
unsigned char *
var_dot1qStaticUnicastTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    AGENT_ENTER;
    static long long_ret;
    static unsigned char string[SPRINT_MAX_LEN];

    if (snmp_mib_getVlanMode() != MIB_VLAN_MODE_TAGGED) {
        return NULL;
    }

    /* Validate */
    snmp_mib_setInterfaceCount();
    if (header_dot1qStaticEntry(vp,name,length,exact,var_len,write_method)
                                             == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    /* Assign the values now */
    dot1qstatic_parms->index = qstatic_entry_num;
    switch(vp->magic) {
        case DOT1QSTATICUNICASTADDRESS:
            dot1qstatic_parms->group_obj = DOT1Q_STATICUNICASTADDRESS;
            break;

        case DOT1QSTATICUNICASTRECEIVEPORT:
            dot1qstatic_parms->group_obj = DOT1Q_STATICUNICASTRECEIVEPORT;
            break;
            
        case DOT1QSTATICUNICASTALLOWEDTOGOTO:
            *write_method = write_dot1qStaticUnicastAllowedToGoTo;
            dot1qstatic_parms->group_obj = DOT1Q_STATICUNICASTALLOWEDTOGOTO;
            break;
            
        case DOT1QSTATICUNICASTSTATUS:
            *write_method = write_dot1qStaticUnicastStatus;
            dot1qstatic_parms->group_obj = DOT1Q_STATICUNICASTSTATUS;
            break;

        default:
            AGENT_EXIT;
            return NULL;
    }

    /* Callback */
    dot1qstatic_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1qstatic_parms);

    /* Give the value back */
    switch(vp->magic) {
        case DOT1QSTATICUNICASTALLOWEDTOGOTO:
            *var_len = (interface_count/INTERFACE_BYTE_COUNT);
            memcpy (string, dot1qstatic_parms->buf, *var_len);
            return (unsigned char *) string;

        case DOT1QSTATICUNICASTADDRESS:
             *var_len = MAC_ADDRESS_LEN;
             memcpy(string, dot1qstatic_parms->buf, *var_len);
             return (u_char *) string;
    }

    long_ret = (long)dot1qstatic_parms->value;
    return (unsigned char *)&long_ret;
}

static int
header_dot1qStaticMCEntry(struct variable *vp,
           oid *name,
           size_t *length,
           int exact,
           size_t *var_len,
           WriteMethod **write_method)
{
#define DOT1Q_STATIC_MC_NAME_LENGTH     13
#define DOT1Q_STATIC_MC_TOTAL_LENGTH    21

    oid newname[MAX_OID_LEN];
    oid lowest[MAX_OID_LEN];
    int i, result, found = FALSE;
    qbridge_static_t mcparam, temp_mcparam;
    int num_mac_addr;
    int j = 0;

    /* Let us set the write method here */
    *write_method = 0;

    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
    memset (&mcparam, 0, sizeof(qbridge_static_t));
    memset (&temp_mcparam, 0, sizeof(qbridge_static_t));
    lowest[0] = 9999;

    /* Get count of number of static MC addresses */
    dot1qstatic_parms->group_obj = DOT1Q_STATICMULTICASTNUMADDR;
    dot1qstatic_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1qstatic_parms);
    num_mac_addr = dot1qstatic_parms->value;

    for (i = 0; i < num_mac_addr; i++) {
        /* Now, get all data about the address */
        dot1qstatic_parms->index = i;
        dot1qstatic_parms->group_obj = DOT1Q_STATICMULTICASTGETADDR;
        dot1qstatic_parms->callback_rw = CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                            SNMPD_CALLBACK_BRIDGE,
                            dot1qstatic_parms);
        memcpy (&mcparam, &(dot1qstatic_parms->group_param.dot1qstatic_param),
                                                    sizeof(qbridge_static_t));

        /* If no port is configured for this mcast address, return */
        if (snmp_get_bit_position_set
                    (mcparam.ports, 0,
                    sizeof(mcparam.ports)*INTERFACE_BYTE_COUNT) == -1) {
            continue;
        }

        /* Now, we set the VLAN ID for this MIB */
        newname[DOT1Q_STATIC_MC_NAME_LENGTH] = mcparam.vid;

        /* Copy the multicast mac */
        for (j = 0; j < MAC_ADDRESS_LEN; j++) {
            newname[DOT1Q_STATIC_MC_NAME_LENGTH + j + 1] = mcparam.mac[j];
        }

        /* Set receive port to zero */
        newname[DOT1Q_STATIC_MC_NAME_LENGTH+MAC_ADDRESS_LEN+1] = 0;

        /* if it's a SET/GET request, exact is true. So return only if there
         * is an exact match or snmp_oid_compare returns 0
         */
        if (exact) {
            if (snmp_oid_compare(name, *length, newname, 
                        DOT1Q_STATIC_MC_TOTAL_LENGTH) == 0)
            {
                memcpy(lowest, newname, 
                        DOT1Q_STATIC_MC_TOTAL_LENGTH * sizeof(oid));
                found = TRUE;
                qmstatic_entry_num = i;
                memcpy (&temp_mcparam, &mcparam, sizeof(qbridge_static_t));
                break;
            }
        } else {
            if (snmp_oid_compare(newname, DOT1Q_STATIC_MC_TOTAL_LENGTH, 
                        name, *length) > 0 &&
                snmp_oid_compare(newname, DOT1Q_STATIC_MC_TOTAL_LENGTH, lowest, 
                    DOT1Q_STATIC_MC_TOTAL_LENGTH) < 0) {
                memcpy((char *) lowest, (char *) newname,
                       DOT1Q_STATIC_MC_TOTAL_LENGTH * sizeof(oid));
                found = TRUE;
                qmstatic_entry_num = i;
                memcpy (&temp_mcparam, &mcparam, sizeof(qbridge_static_t));
            }
        }
    }

    if (!found) {
        AGENT_EXIT;
        return MATCH_FAILED;
    }
    
    memcpy((char *)name,(char *)lowest, 
                       DOT1Q_STATIC_MC_TOTAL_LENGTH * sizeof(oid));

    /* restore our data back */
    memcpy( &stat_mcparam, &temp_mcparam, sizeof(qbridge_static_t));

    *length = DOT1Q_STATIC_MC_TOTAL_LENGTH;
    *var_len = sizeof(long);
    
    AGENT_EXIT;
    return MATCH_SUCCEEDED;
}

/*
 * var_dot1qStaticMulticastTable():
 *  This function contains Multi-Index tables
 *   The workings of this are basically the same as for var_dot1qStatic above.
 */
unsigned char *
var_dot1qStaticMulticastTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
#define DOT1Q_MCAST_STATUS 3 /* Permanent, as no GMRP */
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];

    if (snmp_mib_getVlanMode() != MIB_VLAN_MODE_TAGGED) {
        return NULL;
    }

    /* Validate the address */
    snmp_mib_setInterfaceCount();
    if (header_dot1qStaticMCEntry(vp,name,length,exact,var_len,write_method) 
                                                == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    /*  We will not have any callbacks here,as we already have the values */
    switch(vp->magic) {
        case DOT1QSTATICMULTICASTADDRESS:
            *var_len = MAC_ADDRESS_LEN;
            memcpy(string, stat_mcparam.mac, *var_len);
            return (u_char *) string;
            
        case DOT1QSTATICMULTICASTRECEIVEPORT:
            /* Receive ports are always 0 */
            ulong_ret = 0; 
            return (u_char *) & ulong_ret;

        case DOT1QSTATICMULTICASTSTATICEGRESSPORTS:
            *write_method = write_dot1qStaticMulticastStaticEgressPorts;
            *var_len = interface_count/INTERFACE_BYTE_COUNT;
            memcpy(string, stat_mcparam.ports, *var_len);
            return (u_char *)string;

        case DOT1QSTATICMULTICASTFORBIDDENEGRESSPORTS:
            *write_method = write_dot1qStaticMulticastForbiddenEgressPorts;
            *var_len = interface_count/INTERFACE_BYTE_COUNT;
            memset(string, 0, *var_len);
            return (u_char *) string;

        case DOT1QSTATICMULTICASTSTATUS:
            *write_method = write_dot1qStaticMulticastStatus;
            ulong_ret = DOT1Q_MCAST_STATUS;
            return (u_char *) & ulong_ret;

        default:
            AGENT_EXIT;
            return NULL;
    }

    AGENT_EXIT;
    return NULL;
}


int
write_dot1qStaticMulticastStaticEgressPorts(int action,
                                            u_char * var_val,
                                            u_char var_val_type,
                                            size_t var_val_len,
                                            u_char * statP,
                                            oid * name, size_t name_len)
{
    char            value;

    switch (action) {
    case RESERVE1:
        if (var_val_type != ASN_OCTET_STR) {
            fprintf(stderr, "write to dot1qStatic not ASN_OCTET_STR\n");
            return SNMP_ERR_WRONGTYPE;
        }

        if (var_val_len != 
                (sizeof(char)*(interface_count/INTERFACE_BYTE_COUNT))) {
            fprintf(stderr, "write to dot1qStatic: bad length\n");
            return SNMP_ERR_WRONGLENGTH;
        }
        break;

    case RESERVE2:
        break;

    case FREE:
        break;

    case ACTION:
        break;

    case UNDO:
        break;

    case COMMIT:
        /* We have all the info that we need. Let us do the update */
        dot1qstatic_parms->index = qmstatic_entry_num;
        memcpy(dot1qstatic_parms->buf, var_val, 
                                    (interface_count/INTERFACE_BYTE_COUNT));
        dot1qstatic_parms->group_obj = DOT1Q_STATICMULTICASTSTATICEGRESSPORTS;
        dot1qstatic_parms->callback_rw = CALLBACK_ACT_WRITE;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                            SNMPD_CALLBACK_BRIDGE,
                            dot1qstatic_parms);
        if (dot1qstatic_parms->rv == MIB_STATUS_ERROR) {
            AGENT_EXIT;
            return SNMP_ERR_COMMITFAILED;
        }
        break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


int
write_dot1qStaticMulticastStatus(int action,
                                 u_char * var_val,
                                 u_char var_val_type,
                                 size_t var_val_len,
                                 u_char * statP,
                                 oid * name, size_t name_len)
{
    long            value = 0;
    
    if (var_val)
        value = *(long *) var_val;

    switch (action) {
    case RESERVE1:
        if (var_val_type != ASN_INTEGER) {
            fprintf(stderr, "write to dot1qStatic not ASN_INTEGER\n");
            return SNMP_ERR_WRONGTYPE;
        }

        if (var_val_len > sizeof(long)) {
            fprintf(stderr, "write to dot1qStatic: bad length\n");
            return SNMP_ERR_WRONGLENGTH;
        }

        if (value < C_STATIC_STATUS_OTHER || 
                            value > C_STATIC_STATUS_DELONTMOUT) {
            fprintf(stderr, "write to dot1qStatic: bad value\n");
            return SNMP_ERR_WRONGVALUE;
        }
        break;

    case RESERVE2:
        break;

    case FREE:
        /*
         * Release any resources that have been allocated 
         */
        break;

    case ACTION:
        break;

    case UNDO:
        break;

    case COMMIT:
        /* We have all the info that we need. Let us do the update */
        dot1qstatic_parms->index = qmstatic_entry_num;
        dot1qstatic_parms->value = value;
        dot1qstatic_parms->group_obj = DOT1Q_STATICMULTICASTSTATUS;
        dot1qstatic_parms->callback_rw = CALLBACK_ACT_WRITE;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                            SNMPD_CALLBACK_BRIDGE,
                            dot1qstatic_parms);
        if (dot1qstatic_parms->rv == MIB_STATUS_ERROR) {
            AGENT_EXIT;
            return SNMP_ERR_COMMITFAILED;
        }
        break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

/*
 * Forbidden Egress Ports - For a multicast entry, not sure if this
 * support exists 
 */
int
write_dot1qStaticMulticastForbiddenEgressPorts(int action,
                                               u_char * var_val,
                                               u_char var_val_type,
                                               size_t var_val_len,
                                               u_char * statP,
                                               oid * name, size_t name_len)
{
    int             size;

    switch (action) {
    case RESERVE1:
        if (var_val_type != ASN_OCTET_STR) {
            fprintf(stderr, "write to dot1qStatic not ASN_OCTET_STR\n");
            return SNMP_ERR_WRONGTYPE;
        }

        if (var_val_len != 
                (sizeof(char)*(interface_count/INTERFACE_BYTE_COUNT))) {
            fprintf(stderr, "write to dot1qStatic: bad length\n");
            return SNMP_ERR_WRONGLENGTH;
        }
        break;

    case RESERVE2:
        size = var_val_len;
        break;

    case FREE:
        /*
         * Release any resources that have been allocated 
         */
        break;

    case ACTION:
        /*
         * The variable has been stored in 'value' for you to use,
         * and you have just been asked to do something with it.
         * Note that anything done here must be reversable in the UNDO case
         */
        break;

    case UNDO:
        /*
         * Back out any changes made in the ACTION case 
         */
        break;

    case COMMIT:
        /* We have all the info that we need. Let us do the update */
        memcpy(dot1qstatic_parms->buf, var_val, 
                                    (interface_count/INTERFACE_BYTE_COUNT));
        dot1qstatic_parms->group_obj =
                                    DOT1Q_STATICMULTICASTFORBIDDENEGRESSPORTS;
        dot1qstatic_parms->callback_rw = CALLBACK_ACT_WRITE;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                            SNMPD_CALLBACK_BRIDGE,
                            dot1qstatic_parms);
        if (dot1qstatic_parms->rv == MIB_STATUS_ERROR) {
            AGENT_EXIT;
            return SNMP_ERR_COMMITFAILED;
        }
        break;
    }
    return SNMP_ERR_NOERROR;
}

/*
 * Ports that an unicast MAC is allowed to go to. This is implemented by
 * setting the block ports for an unicast MAC address
 */
int
write_dot1qStaticUnicastAllowedToGoTo(int action,
                                      u_char * var_val,
                                      u_char var_val_type,
                                      size_t var_val_len,
                                      u_char * statP,
                                      oid * name, size_t name_len)
{
    char            value;

    switch (action) {
    case RESERVE1:
        if (var_val_type != ASN_OCTET_STR) {
            fprintf(stderr, "write to dot1qStatic not ASN_OCTET_STR\n");
            return SNMP_ERR_WRONGTYPE;
        }
        if (var_val_len != 
                    sizeof(char) * (interface_count/INTERFACE_BYTE_COUNT)) {
            fprintf(stderr, "write to dot1qStatic: bad length\n");
            return SNMP_ERR_WRONGLENGTH;
        }
        break;

    case RESERVE2:
        /* For now this is not writable */
        return SNMP_ERR_NOTWRITABLE;
        break;

    case FREE:
        break;

    case ACTION:
        break;

    case UNDO:
        break;

    case COMMIT:
        dot1qstatic_parms->index = qstatic_entry_num;
        memcpy(dot1qstatic_parms->buf, var_val, 
                            (interface_count/INTERFACE_BYTE_COUNT));
        dot1qstatic_parms->group_obj =
                                    DOT1Q_STATICUNICASTALLOWEDTOGOTO;
        dot1qstatic_parms->callback_rw = CALLBACK_ACT_WRITE;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                            SNMPD_CALLBACK_BRIDGE,
                            dot1qstatic_parms);
        if (dot1qstatic_parms->rv == MIB_STATUS_ERROR) {
            AGENT_EXIT;
            return SNMP_ERR_COMMITFAILED;
        }
        break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

int
write_dot1qStaticUnicastStatus(int action,
                               u_char * var_val,
                               u_char var_val_type,
                               size_t var_val_len,
                               u_char * statP, oid * name, size_t name_len)
{
    long            value;

    switch (action) {
    case RESERVE1:
        if (var_val_type != ASN_INTEGER) {
            fprintf(stderr, "write to dot1qStatic not ASN_INTEGER\n");
            AGENT_EXIT;
            return SNMP_ERR_WRONGTYPE;
        }
        if (var_val_len > sizeof(long)) {
            fprintf(stderr, "write to dot1qStatic: bad length\n");
            AGENT_EXIT;
            return SNMP_ERR_WRONGLENGTH;
        }

        value = *(long *) var_val;
        if (value < 1 || value > 5) {
            fprintf(stderr, "write to dot1qStatic: bad value\n");
            AGENT_EXIT;
            return SNMP_ERR_WRONGVALUE;
        }
        break;

    case RESERVE2:
        break;

    case FREE:
        break;

    case ACTION:
        break;

    case UNDO:
        break;

    case COMMIT:
        dot1qstatic_parms->index = qstatic_entry_num;
        dot1qstatic_parms->value = * (int *) var_val;
        dot1qstatic_parms->group_obj = DOT1Q_STATICUNICASTSTATUS;
        dot1qstatic_parms->callback_rw = CALLBACK_ACT_WRITE;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                            SNMPD_CALLBACK_BRIDGE,
                            dot1qstatic_parms);
        if (dot1qstatic_parms->rv == MIB_STATUS_ERROR) {
            AGENT_EXIT;
            return SNMP_ERR_COMMITFAILED;
        }
        break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

