/*
 * $Id: dot1qTp.c,v 1.1 2006/11/23 14:48:41 jeffc Exp $
 */

/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include "mibgroup/mibII/dot1qTp.h"
#include <ucd-snmp/mibgroup/mibII/bridge_mib.h>

/* Callback function to be passed to backend */
static bridge_parameters_t *dot1qtp_parms = NULL;

/* Path to this MIB WRT the global MIB tree */
oid dot1qTp_variables_oid[] = { 1,3,6,1,2,1,17,7,1,2 };

/* Index of exact match entry */
static int qTp_entry_num;

/* Num port count */
extern int interface_count;

/* We just save all info that we get during recursion */
static qbridge_static_t stat_mcparam;

unsigned char mac[MAC_ADDRESS_LEN];

/* Variables that this MIB supports */
struct variable4 dot1qTp_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define DOT1QFDBID      1
    {DOT1QFDBID,  ASN_UNSIGNED,  RONLY,   var_dot1qFdbTable, 3,  { 1, 1, 1 }},
#define DOT1QFDBDYNAMICCOUNT        2
    {DOT1QFDBDYNAMICCOUNT,  ASN_COUNTER,  RONLY,   var_dot1qFdbTable, 3,  { 1, 1, 2 }},
#define DOT1QTPFDBADDRESS       3
    {DOT1QTPFDBADDRESS,  ASN_OCTET_STR,  RONLY,   var_dot1qTpFdbTable, 3,  { 2, 1, 1 }},
#define DOT1QTPFDBPORT      4
    {DOT1QTPFDBPORT,  ASN_INTEGER,  RONLY,   var_dot1qTpFdbTable, 3,  { 2, 1, 2 }},
#define DOT1QTPFDBSTATUS        5
    {DOT1QTPFDBSTATUS,  ASN_INTEGER,  RONLY,   var_dot1qTpFdbTable, 3,  { 2, 1, 3 }},
#define DOT1QTPGROUPADDRESS     6
    {DOT1QTPGROUPADDRESS,  ASN_OCTET_STR,  RONLY,   var_dot1qTpGroupTable, 3,  { 3, 1, 1 }},
#define DOT1QTPGROUPEGRESSPORTS     7
    {DOT1QTPGROUPEGRESSPORTS,  ASN_OCTET_STR,  RONLY,   var_dot1qTpGroupTable, 3,  { 3, 1, 2 }},
#define DOT1QTPGROUPLEARNT      8
    {DOT1QTPGROUPLEARNT,  ASN_OCTET_STR,  RONLY,   var_dot1qTpGroupTable, 3,  { 3, 1, 3 }},
#define DOT1QFORWARDALLPORTS        9
    {DOT1QFORWARDALLPORTS,  ASN_OCTET_STR,  RONLY,   var_dot1qForwardAllTable, 3,  { 4, 1, 1 }},
#define DOT1QFORWARDALLSTATICPORTS      10
    {DOT1QFORWARDALLSTATICPORTS,  ASN_OCTET_STR,  RWRITE,  var_dot1qForwardAllTable, 3,  { 4, 1, 2 }},
#define DOT1QFORWARDALLFORBIDDENPORTS       11
    {DOT1QFORWARDALLFORBIDDENPORTS,  ASN_OCTET_STR,  RWRITE,  var_dot1qForwardAllTable, 3,  { 4, 1, 3 }},
#define DOT1QFORWARDUNREGISTEREDPORTS       12
    {DOT1QFORWARDUNREGISTEREDPORTS,  ASN_OCTET_STR,  RONLY,   var_dot1qForwardUnregisteredTable, 3,  { 5, 1, 1 }},
#define DOT1QFORWARDUNREGISTEREDSTATICPORTS     13
    {DOT1QFORWARDUNREGISTEREDSTATICPORTS,  ASN_OCTET_STR,  RWRITE,  var_dot1qForwardUnregisteredTable, 3,  { 5, 1, 2 }},
#define DOT1QFORWARDUNREGISTEREDFORBIDDENPORTS      14
    {DOT1QFORWARDUNREGISTEREDFORBIDDENPORTS,  ASN_OCTET_STR,  RWRITE,  var_dot1qForwardUnregisteredTable, 3,  { 5, 1, 3 }},
};

/** Initializes the dot1qTp module */
void
init_dot1qTp(void)
{
    /* Allocation for callback parameter */
    dot1qtp_parms = (bridge_parameters_t *) \
                sal_alloc(sizeof(bridge_parameters_t), "dot1qtp_parms");

    if (dot1qtp_parms != NULL) {
        dot1qtp_parms->group  = BRIDGE_DOT1QTP;
        
        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB("dot1qTp", dot1qTp_variables, variable4,
                   dot1qTp_variables_oid);
    }
}

/*
 * var_dot1qTp():
 */
unsigned char *
var_dot1qTp(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* Proceed only if VLAN mode is 802.1Q */
    if (snmp_mib_getVlanMode() != MIB_VLAN_MODE_TAGGED) {
        AGENT_EXIT;
        return NULL;
    }


    /* Validate */
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ) {
        return NULL;
    }

    return NULL;
}

/* 
 * Filter database table implementation
 */
unsigned char *
var_dot1qFdbTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    static u_long   ulong_ret;

    /* Proceed only if VLAN mode is 802.1Q */
    if (snmp_mib_getVlanMode() != MIB_VLAN_MODE_TAGGED) {
        AGENT_EXIT;
        return NULL;
    }

    /* Its just one single table here, teh global table */
    if (header_simple_table
        (vp, name, length, exact, var_len, write_method, 1)
        == MATCH_FAILED) {
        return NULL;
    }

    /* Start the MIB assignments */
    switch(vp->magic) {
        case DOT1QFDBID:
            /* From backedn, we return 1 */
            dot1qtp_parms->group_obj = DOT1Q_FDBID;
            break;
        case DOT1QFDBDYNAMICCOUNT:
            /* This is count of L2 and L2MC table */
            dot1qtp_parms->group_obj = DOT1Q_FDBDYNAMICCOUNT;
            break;
        default:
            ERROR_MSG("");
    }

    /* Callback */
    dot1qtp_parms->callback_rw = CALLBACK_ACT_READ;
    dot1qtp_parms->group = BRIDGE_DOT1QTP; 
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1qtp_parms);
    
    /* Give the value back */
    ulong_ret = (unsigned long)dot1qtp_parms->value;
    return (unsigned char *) &ulong_ret;
}

static int
header_dot1qTpFdbTable(struct variable *vp,
           oid *name,
           size_t *length,
           int exact,
           size_t *var_len,
           WriteMethod **write_method)
{
#define DOT1QTPFDB_NAME_LENGTH  13
#define DOT1QTPFDB_TOTAL_OID_LENGTH 20

    oid newname[MAX_OID_LEN];
    oid lowest[MAX_OID_LEN];
    int i, result, found = FALSE;
    int num_mac_addr;
    int j = 0;

    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    /* Set the entry number to an invalid index at the start */
    qTp_entry_num = -1;
    lowest[0] = 9999;

    /* Let us set the write method */
     *write_method = 0;

    /* Get count of number of L2 entries */
    dot1qtp_parms->group = BRIDGE_DOT1DSTATIC;

    /* dump l2 entries first */
    dot1qtp_parms->group_obj = DOT1D_STATICADDRESSDUMP;
    dot1qtp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1qtp_parms);

    dot1qtp_parms->group_obj = DOT1D_STATICADDRESSCOUNT;
    dot1qtp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1qtp_parms);
    num_mac_addr = dot1qtp_parms->value;

    /* We have only 1 FDB */
    newname[DOT1QTPFDB_NAME_LENGTH] = 1;

    /* Now, get the entry by index and validate request */
    for (i = 0; i < num_mac_addr; i++) {
        /* get the mac address by index */
        dot1qtp_parms->index = i;
        dot1qtp_parms->group_obj = DOT1D_STATICADDRESS;
        dot1qtp_parms->callback_rw = CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                            SNMPD_CALLBACK_BRIDGE,
                            dot1qtp_parms);
        memcpy (mac, dot1qtp_parms->buf, MAC_ADDRESS_LEN);

        /* Leave out L2MC entries */
        if ((mac[0] & 0x1)){
            continue;
        }
        
        for (j = 0; j < MAC_ADDRESS_LEN; j++)  { /* 6 octets for MAC Address */
            newname[DOT1QTPFDB_NAME_LENGTH+j+1] = mac[j];
        }
        
        /* if it's a SET/GET request, exact is true. So return only if there
         * is an exact match or snmp_oid_compare returns 0
         */
        if (exact) {
            if (snmp_oid_compare(name, *length, newname, 
                        DOT1QTPFDB_TOTAL_OID_LENGTH) == 0) {
                memcpy(lowest, newname, 
                                DOT1QTPFDB_TOTAL_OID_LENGTH * sizeof(oid));
                found = TRUE;
                qTp_entry_num = i;
               // memcpy(dot1qtp_parms->buf, mac, 6);
                break;
            }
        } else {
            if (snmp_oid_compare(newname, DOT1QTPFDB_TOTAL_OID_LENGTH, 
                        name, *length) > 0 &&
                snmp_oid_compare(newname, DOT1QTPFDB_TOTAL_OID_LENGTH, lowest, 
                    DOT1QTPFDB_TOTAL_OID_LENGTH) < 0) {
                /* Determine if get or get-next */
                memcpy((char *) lowest, (char *) newname,
                       DOT1QTPFDB_TOTAL_OID_LENGTH * sizeof(oid));
                found = TRUE;
                qTp_entry_num = i;
               // memcpy(dot1qtp_parms->buf, mac, 6);
            }
        }
    }

    if (!found) {
        DEBUGMSGTL(("dot1qTp", "Invalid index\n"));
        return MATCH_FAILED;
    }

    memcpy( (char *)name,(char *)lowest, 
                    DOT1QTPFDB_TOTAL_OID_LENGTH * sizeof(oid));
    *length = DOT1QTPFDB_TOTAL_OID_LENGTH;
    *var_len = sizeof(long);

    return MATCH_SUCCEEDED;
}


/*
 * var_dot1qTpFdbTable():
 *  This function contains Multi-Index tables
 *   The workings of this are basically the same as for var_dot1qTp above.
 */
unsigned char *
var_dot1qTpFdbTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    static long long_ret;
    static unsigned char string[SPRINT_MAX_LEN];

    /* Proceed only if VLAN mode is 802.1Q */
    if (snmp_mib_getVlanMode() != MIB_VLAN_MODE_TAGGED) {
        AGENT_EXIT;
        return NULL;
    }

    /*
     * Simple table validation
     */
    if (header_dot1qTpFdbTable(vp,name,length,exact,var_len,write_method)
                                                == MATCH_FAILED ) {
        return NULL;
    }

  /* 
   * Start the MIB assignments
   */
    dot1qtp_parms->index = qTp_entry_num;
    switch (vp->magic) {
        case DOT1QTPFDBADDRESS:
            dot1qtp_parms->group_obj = DOT1Q_TPFDBADDRESS;
            //*var_len = 6;
            //memcpy(string, dot1qtp_parms->buf, *var_len);
            //return (unsigned char*)string;
            break;
        case DOT1QTPFDBPORT:
            dot1qtp_parms->value = -1; /* Setting default value */
            dot1qtp_parms->group_obj = DOT1Q_TPFDBPORT;
            break;
        case DOT1QTPFDBSTATUS:
            dot1qtp_parms->group_obj = DOT1Q_TPFDBSTATUS;
            break;
        default:
            ERROR_MSG("Undefined MIB reference");
    }

    /* Callback */
    dot1qtp_parms->group = BRIDGE_DOT1QTP;
    dot1qtp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1qtp_parms);
    
    /* Give the value back */
    switch (vp->magic) {
    case DOT1QTPFDBADDRESS:
         *var_len = MAC_ADDRESS_LEN;
         memcpy(string, dot1qtp_parms->buf, *var_len);
         return (unsigned char *) string;
    }

    long_ret = (long)dot1qtp_parms->value;
    return (unsigned char *)&long_ret;
}

static int
header_dot1qTpGroupEntry(struct variable *vp,
           oid *name,
           size_t *length,
           int exact,
           size_t *var_len,
           WriteMethod **write_method)
{
#define DOT1QTP_GRP_NAME_LENGTH         13
#define DOT1QTP_GRP_TOTAL_OID_LENGTH    20

    oid newname[MAX_OID_LEN];
    oid lowest[MAX_OID_LEN];
    int i, result, found = FALSE;
    qbridge_static_t mcparam, temp_mcparam;
    int num_mac_addr;
    int j = 0;

    /* Let us set the write method here */
    *write_method = 0;

    lowest[0] = 9999;

    DEBUGMSGTL(("dot1qTp", "header_dot1qTpMCEntry: "));
    
    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
    memset (&mcparam, 0, sizeof(qbridge_static_t));
    memset (&temp_mcparam, 0, sizeof(qbridge_static_t));

    /* Get count of number of static MC addresses */
    dot1qtp_parms->group = BRIDGE_DOT1QSTATIC;
    dot1qtp_parms->group_obj = DOT1Q_STATICMULTICASTNUMADDR;
    dot1qtp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1qtp_parms);
    num_mac_addr = dot1qtp_parms->value;

    for (i = 0; i < num_mac_addr; i++) {
        /* Now, get all data about the address */
        dot1qtp_parms->group_obj = DOT1Q_STATICMULTICASTGETADDR;
        dot1qtp_parms->callback_rw = CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                            SNMPD_CALLBACK_BRIDGE,
                            dot1qtp_parms);
        memcpy (&mcparam, &(dot1qtp_parms->group_param.dot1qstatic_param),
                                                    sizeof(qbridge_static_t));
#if 0
        /* If no port is configured for this mcast address, return */
        if (snmp_get_bit_position_set(mcparam.ports, 0,sizeof(mcparam.ports)*8) 
                                                                     == -1) {
            continue;
        }
#endif
        /* Now, we set the VLAN ID for this MIB */
        newname[DOT1QTP_GRP_NAME_LENGTH] = mcparam.vid;

        /* Copy the multicast mac */
        for (j = 0; j < MAC_ADDRESS_LEN; j++) {
            newname[DOT1QTP_GRP_NAME_LENGTH + j + 1] = mcparam.mac[j];
        }

        /* if it's a SET/GET request, exact is true. So return only if there
         * is an exact match or snmp_oid_compare returns 0
         */
        if (exact) {
            if (snmp_oid_compare(name, *length, newname, 
                        DOT1QTP_GRP_TOTAL_OID_LENGTH) == 0)
            {
                memcpy(lowest, newname, 
                        DOT1QTP_GRP_TOTAL_OID_LENGTH * sizeof(oid));
                found = TRUE;
                memcpy (&temp_mcparam, &mcparam, sizeof(qbridge_static_t));
                dot1qtp_parms->index = i;
                break;
            }
        } else {
            if (snmp_oid_compare(newname, DOT1QTP_GRP_TOTAL_OID_LENGTH, 
                        name, *length) > 0 &&
                snmp_oid_compare(newname, DOT1QTP_GRP_TOTAL_OID_LENGTH, lowest, 
                    DOT1QTP_GRP_TOTAL_OID_LENGTH) < 0) {
                /*
                 * if new one is greater than input and closer to input than
                 * previous lowest, save this one as the "next" one.
                 */
                memcpy((char *) lowest, (char *) newname,
                       DOT1QTP_GRP_TOTAL_OID_LENGTH * sizeof(oid));
                found = TRUE;
                memcpy (&temp_mcparam, &mcparam, sizeof(qbridge_static_t));
                dot1qtp_parms->index = i;
            }
        }
    }

    dot1qtp_parms->group = BRIDGE_DOT1QTP;
    if (!found) {
        DEBUGMSGTL(("dot1qTp", "... index out of range\n"));
        return MATCH_FAILED;
    }
    
    memcpy( (char *)name,(char *)lowest, 
            DOT1QTP_GRP_TOTAL_OID_LENGTH * sizeof(oid));

    /* restore our data back */
    memcpy( &stat_mcparam, &temp_mcparam, sizeof(qbridge_static_t));

    *write_method = 0;
    *length = DOT1QTP_GRP_TOTAL_OID_LENGTH;
    *var_len = sizeof(long);

    return MATCH_SUCCEEDED;
}

/*
 * var_dot1qTpGroupTable():
 */
unsigned char *
var_dot1qTpGroupTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    static unsigned char string[SPRINT_MAX_LEN];

    /* Proceed only if VLAN mode is 802.1Q */
    if (snmp_mib_getVlanMode() != MIB_VLAN_MODE_TAGGED) {
        AGENT_EXIT;
        return NULL;
    }

    /*
     * Simple table validation
     */
    if (header_dot1qTpGroupEntry(vp,name,length,exact,var_len,write_method)
                                                == MATCH_FAILED ) {
        return NULL;
    }

    snmp_mib_setInterfaceCount();

    /* Start assigning the values */
    switch(vp->magic) {
        case DOT1QTPGROUPADDRESS:
            memcpy(string, stat_mcparam.mac, MAC_ADDRESS_LEN);
            *var_len = MAC_ADDRESS_LEN;
            return (unsigned char *) string;

        case DOT1QTPGROUPEGRESSPORTS:
#if 0
            memcpy(string, stat_mcparam.ports, (interface_count/8));
#endif
            *var_len = interface_count/INTERFACE_BYTE_COUNT;
            return (unsigned char *) string;

        case DOT1QTPGROUPLEARNT:
            /* We have no differentiation now between learnt & static ports? */
#if 0
            memcpy(string, stat_mcparam.ports, (interface_count/8));
#endif
            *var_len = interface_count/INTERFACE_BYTE_COUNT;
            return (unsigned char *) string;

        default:
            ERROR_MSG("Undefined MIB reference");
    }

    return NULL;
}

static int
header_dot1qForwardTable(struct variable *vp,
           oid *name,
           size_t *length,
           int exact,
           size_t *var_len,
           WriteMethod **write_method)
{
#define DOT1QFWD_NAME_LENGTH    13

    oid newname[MAX_OID_LEN];
    oid lowest[MAX_OID_LEN];
    int vlan_count;
    int i, result, found = FALSE;
    int vlan_id = 0; 

    /* Get count of number of VLANs on the switch */
    dot1qtp_parms->group = BRIDGE_DOT1QVLAN;
    dot1qtp_parms->group_obj = DOT1Q_VLANCOUNT;
    dot1qtp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_BRIDGE,
                        dot1qtp_parms);
    vlan_count = dot1qtp_parms->value;
    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
    lowest[0] = 99999; /* for first time comparison */

    /* Get VLAN ID by index and validate */
    for (i = 0; i < vlan_count; i++ ) {
        dot1qtp_parms->index = i;
        dot1qtp_parms->group_obj = DOT1Q_VLANID;
        dot1qtp_parms->callback_rw = CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                            SNMPD_CALLBACK_BRIDGE,
                            dot1qtp_parms);
        vlan_id = dot1qtp_parms->value;

        newname[DOT1QFWD_NAME_LENGTH] = (oid) vlan_id ;
        if (exact) {
            if (snmp_oid_compare(name, *length, newname, 
                        DOT1QFWD_NAME_LENGTH) == 0) {
                memcpy(lowest, newname, 
                                DOT1QFWD_NAME_LENGTH * sizeof(oid));
                found = TRUE;
                dot1qtp_parms->index = i;
                break;
            }
        } else {
            if ((snmp_oid_compare(newname, DOT1QFWD_NAME_LENGTH, 
                   name, *length) > 0) && (found == FALSE || 
                       snmp_oid_compare(newname, 
                       DOT1QFWD_NAME_LENGTH, lowest, 
                           DOT1QFWD_NAME_LENGTH) < 0))  {
                memcpy((char *) lowest, (char *) newname,
                       DOT1QFWD_NAME_LENGTH * sizeof(oid));
                dot1qtp_parms->index = i;
                found = TRUE;
            }
        }
    }

    dot1qtp_parms->group = BRIDGE_DOT1QTP;
    if (!found) {
        AGENT_EXIT;
        return MATCH_FAILED;
    }
    
    memcpy( (char *)name,(char *)lowest,((int) vp->namelen +1) * sizeof(oid));

    *length = vp->namelen + 1;
    *write_method = 0;
    *var_len = sizeof(long);

    return MATCH_SUCCEEDED;
}


/*
 * var_dot1qForwardAllTable():
 */
unsigned char *
var_dot1qForwardAllTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    static unsigned char string[SPRINT_MAX_LEN];

    /* Proceed only if VLAN mode is 802.1Q */
    if (snmp_mib_getVlanMode() != MIB_VLAN_MODE_TAGGED) {
        AGENT_EXIT;
        return NULL;
    }

    /*
     * Simple table validation
     */
    snmp_mib_setInterfaceCount();
    if (header_dot1qForwardTable(vp,name,length,exact,var_len,write_method)
                                                == MATCH_FAILED ) {
        return NULL;
    }


    /* 
     * Start the MIB assignments
     */
    switch(vp->magic) {
        case DOT1QFORWARDALLPORTS:
            dot1qtp_parms->group_obj = DOT1Q_FORWARDALLPORTS;
            break;

        case DOT1QFORWARDALLSTATICPORTS:
            *write_method = write_dot1qForwardAllStaticPorts;
            dot1qtp_parms->group_obj = DOT1Q_FORWARDALLSTATICPORTS;
            break;

        case DOT1QFORWARDALLFORBIDDENPORTS:
            *write_method = write_dot1qForwardAllForbiddenPorts;
            dot1qtp_parms->group_obj = DOT1Q_FORWARDALLFORBIDDENPORTS;
            break;

        default:
            AGENT_EXIT;
            return NULL;
    }

    /* Callback */
    dot1qtp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1qtp_parms);
    
    /* Give the value back */
    *var_len = interface_count/INTERFACE_BYTE_COUNT;
    memcpy(string, dot1qtp_parms->buf, *var_len);
    return (unsigned char *) string;
}

/*
 * var_dot1qForwardUnregisteredTable():
 */
unsigned char *
var_dot1qForwardUnregisteredTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    static unsigned char string[SPRINT_MAX_LEN];

    /* Proceed only if VLAN mode is 802.1Q */
    if (snmp_mib_getVlanMode() != MIB_VLAN_MODE_TAGGED) {
        AGENT_EXIT;
        return NULL;
    }

    /* Validate using custom function */
    if (header_dot1qForwardTable(vp,name,length,exact,var_len,write_method)
                                                == MATCH_FAILED ) {
        return NULL;
    }

    snmp_mib_setInterfaceCount();
    dot1qtp_parms->group  = BRIDGE_DOT1QTP;

    /* Now, start the assignments */
    switch(vp->magic) {
        case DOT1QFORWARDUNREGISTEREDPORTS:
            dot1qtp_parms->group_obj = DOT1Q_FORWARDUNREGISTEREDPORTS;
            break;
        case DOT1QFORWARDUNREGISTEREDSTATICPORTS:
            *write_method = write_dot1qForwardUnregisteredStaticPorts;
            dot1qtp_parms->group_obj = DOT1Q_FORWARDUNREGISTEREDSTATICPORTS;
            break;
        case DOT1QFORWARDUNREGISTEREDFORBIDDENPORTS:
            *write_method = write_dot1qForwardUnregisteredForbiddenPorts;
            dot1qtp_parms->group_obj = DOT1Q_FORWARDUNREGISTEREDFORBIDDENPORTS;
            break;
        default:
            ERROR_MSG("Undefined MIB Reference");
    }

    /* Callback */
    dot1qtp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_BRIDGE,
                        dot1qtp_parms);
    
    /* Give the value back */
    memcpy(string, dot1qtp_parms->buf, interface_count/INTERFACE_BYTE_COUNT);
    *var_len = interface_count/INTERFACE_BYTE_COUNT;
    return (unsigned char *)string;
}

int
write_dot1qForwardAllStaticPorts(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    char value;
    int size;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to dot1qTp not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to dot1qTp: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          size  = var_val_len;
          value = * (char *) var_val;

          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          break;
    }
    return SNMP_ERR_NOERROR;
}

int
write_dot1qForwardAllForbiddenPorts(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    char value;
    int size;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to dot1qTp not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to dot1qTp: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          size  = var_val_len;
          value = * (char *) var_val;

          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          break;
    }
    return SNMP_ERR_NOERROR;
}

int
write_dot1qForwardUnregisteredStaticPorts(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    char value;
    int size;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to dot1qTp not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to dot1qTp: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          size  = var_val_len;
          value = * (char *) var_val;

          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          break;
    }
    return SNMP_ERR_NOERROR;
}

int
write_dot1qForwardUnregisteredForbiddenPorts(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    char value;
    int size;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to dot1qTp not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to dot1qTp: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          size  = var_val_len;
          value = * (char *) var_val;

          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          break;
    }
    return SNMP_ERR_NOERROR;
}
