/*
 * $Id: rmonlite_mib.c,v 1.1 2006/11/23 14:48:41 jeffc Exp $
 *
 * $Copyright: $
 *
 */

/*
 *  RMON-Lite MIB group implementation - rmonlite_mib.c
 *
 */

/* This should always be included first before anything else */
#include <config.h>


/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"
#include "mibgroup/mibII/dot3.h"
#include "mibgroup/mibII/ifr_helper.h"
#include "mibgroup/mibII/rmonlite_mib.h"

#include <net/if_types.h>
#include <net/if.h>
#ifdef CYGPKG_NET_FREEBSD_STACK
#include <net/if_var.h>
#endif

#include <pkgconf/system.h>

#include <cyg/io/eth/eth_drv_stats.h>
#include <sys/sockio.h>

/* RMON-Lite probe interfaces header file*/
#include "rmonlite_probe.h"

/*********************
 *
 *  Initialisation & common implementation functions
 *
 *********************/
 
/* 
 * rmon_variables_oid:
 *   this is the top level oid that we want to register under.  This
 *   is essentially a prefix, with the suffix appearing in the
 *   variable below.
 */
oid rmon_variables_oid[] = { 1,3,6,1,2,1,16 };

/* Ethernet Statistics Group */
#define RMONETHERSTATSINDEX                    1
#define RMONETHERSTATSDATASOURCE               2
#define RMONETHERSTATSDROPEVENTS               3
#define RMONETHERSTATSOCTETS                   4
#define RMONETHERSTATSPKTS                     5
#define RMONETHERSTATSBROADCASTPKTS            6
#define RMONETHERSTATSMULTICASTPKTS            7
#define RMONETHERSTATSCRCALIGNERRORS           8
#define RMONETHERSTATSUNDERSIZEPKTS            9
#define RMONETHERSTATSOVERSIZEPKTS             10
#define RMONETHERSTATSFRAGMENTS                11
#define RMONETHERSTATSJABBERS                  12
#define RMONETHERSTATSCOLLISIONS               13
#define RMONETHERSTATSPKTS64OCTETS             14
#define RMONETHERSTATSPKTS65TO127OCTETS        15
#define RMONETHERSTATSPKTS128TO255OCTETS       16
#define RMONETHERSTATSPKTS256TO511OCTETS       17
#define RMONETHERSTATSPKTS512TO1023OCTETS      18
#define RMONETHERSTATSPKTS1024TO1518OCTETS     19
#define RMONETHERSTATSOWNER                    20
#define RMONETHERSTATSSTATUS                   21

/* History Control Group */
#define RMONHISTORYCONTROLINDEX                22
#define RMONHISTORYCONTROLDATASOURCE           23
#define RMONHISTORYCONTROLBUCKETSREQUESTED     24
#define RMONHISTORYCONTROLBUCKETSGRANTED       25
#define RMONHISTORYCONTROLINTERVAL             26
#define RMONHISTORYCONTROLOWNER                27
#define RMONHISTORYCONTROLSTATUS               28

/* Ethernet History Group */
#define RMONETHERHISTORYINDEX                  29
#define RMONETHERHISTORYSAMPLEINDEX            30
#define RMONETHERHISTORYINTERVALSTART          31
#define RMONETHERHISTORYDROPEVENTS             32
#define RMONETHERHISTORYOCTETS                 33
#define RMONETHERHISTORYPKTS                   34
#define RMONETHERHISTORYBROADCASTPKTS          35
#define RMONETHERHISTORYMULTICASTPKTS          36
#define RMONETHERHISTORYCRCALIGNERRORS         37
#define RMONETHERHISTORYUNDERSIZEPKTS          38
#define RMONETHERHISTORYOVERSIZEPKTS           39
#define RMONETHERHISTORYFRAGMENTS              40
#define RMONETHERHISTORYJABBERS                41
#define RMONETHERHISTORYCOLLISIONS             42
#define RMONETHERHISTORYUTILIZATION            43

/* Alarm Group */
#define RMONALARMINDEX                         44
#define RMONALARMINTERVAL                      45
#define RMONALARMVARIABLE                      46
#define RMONALARMSAMPLETYPE                    47
#define RMONALARMVALUE                         48
#define RMONALARMSTARTUPALARM                  49
#define RMONALARMRISINGTHRESHOLD               50
#define RMONALARMFALLINGTHRESHOLD              51
#define RMONALARMRISINGEVENTINDEX              52
#define RMONALARMFALLINGEVENTINDEX             53
#define RMONALARMOWNER                         54
#define RMONALARMSTATUS                        55

/* Event Group */
#define RMONEVENTINDEX                         56
#define RMONEVENTDESCRIPTION                   57
#define RMONEVENTTYPE                          58
#define RMONEVENTCOMMUNITY                     59
#define RMONEVENTLASTTIMESENT                  60
#define RMONEVENTOWNER                         61
#define RMONEVENTSTATUS                        62

/* Event Log Table */
#define RMONLOGEVENTINDEX                      63
#define RMONLOGINDEX                           64
#define RMONLOGTIME                            65
#define RMONLOGDESCRIPTION                     66



/* define the structure we're going to ask the agent to register our
   information at */
   
 
struct variable5 rmonlite_variables[] = {
/*    magic number        , variable type , ro/rw , callback fn  , 
                                L, oidsuffix */
                                
                                
    /* Statistics Ethernet Stats Group */
    { RMONETHERSTATSINDEX               , ASN_INTEGER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,1 } },
    { RMONETHERSTATSDATASOURCE          , ASN_OBJECT_ID , RWRITE, 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,2 } },
    { RMONETHERSTATSDROPEVENTS          , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,3 } },
    { RMONETHERSTATSOCTETS              , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,4 } },
    { RMONETHERSTATSPKTS                , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,5 } },
    { RMONETHERSTATSBROADCASTPKTS       , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,6 } },
    { RMONETHERSTATSMULTICASTPKTS       , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,7 } },
    { RMONETHERSTATSCRCALIGNERRORS      , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,8 } },
    { RMONETHERSTATSUNDERSIZEPKTS       , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,9 } },
    { RMONETHERSTATSOVERSIZEPKTS        , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,10 } },
    { RMONETHERSTATSFRAGMENTS           , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,11 } },
    { RMONETHERSTATSJABBERS             , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,12 } },
    { RMONETHERSTATSCOLLISIONS          , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,13 } },
    { RMONETHERSTATSPKTS64OCTETS        , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,14 } },
    { RMONETHERSTATSPKTS65TO127OCTETS   , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,15 } },
    { RMONETHERSTATSPKTS128TO255OCTETS  , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,16 } },
    { RMONETHERSTATSPKTS256TO511OCTETS  , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,17 } },
    { RMONETHERSTATSPKTS512TO1023OCTETS , ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,18 } },
    { RMONETHERSTATSPKTS1024TO1518OCTETS, ASN_COUNTER   , RONLY , 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,19 } },
    { RMONETHERSTATSOWNER               , ASN_OCTET_STR , RWRITE, 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,20 } },
    { RMONETHERSTATSSTATUS              , ASN_INTEGER   , RWRITE, 
                                var_rmonStatEtStatsTable, 4, { 1,1,1,21 } },
    /* 21 */
    
    /* History Control Group */
    { RMONHISTORYCONTROLINDEX           , ASN_INTEGER   , RONLY , 
                                var_rmonHistCtrlTable, 4, { 2,1,1,1 } },
    { RMONHISTORYCONTROLDATASOURCE      , ASN_OBJECT_ID , RWRITE, 
                                var_rmonHistCtrlTable, 4, { 2,1,1,2 } },
    { RMONHISTORYCONTROLBUCKETSREQUESTED, ASN_INTEGER   , RWRITE, 
                                var_rmonHistCtrlTable, 4, { 2,1,1,3 } },
    { RMONHISTORYCONTROLBUCKETSGRANTED  , ASN_INTEGER   , RONLY , 
                                var_rmonHistCtrlTable, 4, { 2,1,1,4 } },
    { RMONHISTORYCONTROLINTERVAL        , ASN_INTEGER   , RWRITE, 
                                var_rmonHistCtrlTable, 4, { 2,1,1,5 } },
    { RMONHISTORYCONTROLOWNER           , ASN_OCTET_STR , RWRITE, 
                                var_rmonHistCtrlTable, 4, { 2,1,1,6 } },
    { RMONHISTORYCONTROLSTATUS          , ASN_INTEGER   , RWRITE, 
                                var_rmonHistCtrlTable, 4, { 2,1,1,7 } },
    /* 28 */
    
    /* History Ethernet Group */
    { RMONETHERHISTORYINDEX             , ASN_INTEGER   , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,1 } },
    { RMONETHERHISTORYSAMPLEINDEX       , ASN_INTEGER   , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,2 } },
    { RMONETHERHISTORYINTERVALSTART     , ASN_TIMETICKS , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,3 } },
    { RMONETHERHISTORYDROPEVENTS        , ASN_COUNTER   , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,4 } },
    { RMONETHERHISTORYOCTETS            , ASN_COUNTER   , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,5 } },
    { RMONETHERHISTORYPKTS              , ASN_COUNTER   , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,6 } },
    { RMONETHERHISTORYBROADCASTPKTS     , ASN_COUNTER   , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,7 } },
    { RMONETHERHISTORYMULTICASTPKTS     , ASN_COUNTER   , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,8 } },
    { RMONETHERHISTORYCRCALIGNERRORS    , ASN_COUNTER   , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,9 } },
    { RMONETHERHISTORYUNDERSIZEPKTS     , ASN_COUNTER   , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,10 } },
    { RMONETHERHISTORYOVERSIZEPKTS      , ASN_COUNTER   , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,11 } },
    { RMONETHERHISTORYFRAGMENTS         , ASN_COUNTER   , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,12 } },
    { RMONETHERHISTORYJABBERS           , ASN_COUNTER   , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,13 } },
    { RMONETHERHISTORYCOLLISIONS        , ASN_COUNTER   , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,14 } },
    { RMONETHERHISTORYUTILIZATION       , ASN_INTEGER   , RONLY , 
                                var_rmonHistEtStatTable, 4, { 2,2,1,15 } },
    /* 43 */
    
    /* Alarm Group */
    { RMONALARMINDEX                    , ASN_INTEGER   , RONLY , 
                                var_rmonAlarmTable, 4, { 3,1,1,1 } },
    { RMONALARMINTERVAL                 , ASN_INTEGER   , RWRITE, 
                                var_rmonAlarmTable, 4, { 3,1,1,2 } },
    { RMONALARMVARIABLE                 , ASN_OBJECT_ID , RWRITE, 
                                var_rmonAlarmTable, 4, { 3,1,1,3 } },
    { RMONALARMSAMPLETYPE               , ASN_INTEGER   , RWRITE, 
                                var_rmonAlarmTable, 4, { 3,1,1,4 } },
    { RMONALARMVALUE                    , ASN_INTEGER   , RONLY , 
                                var_rmonAlarmTable, 4, { 3,1,1,5 } },
    { RMONALARMSTARTUPALARM             , ASN_INTEGER   , RWRITE, 
                                var_rmonAlarmTable, 4, { 3,1,1,6 } },
    { RMONALARMRISINGTHRESHOLD          , ASN_INTEGER   , RWRITE, 
                                var_rmonAlarmTable, 4, { 3,1,1,7 } },
    { RMONALARMFALLINGTHRESHOLD         , ASN_INTEGER   , RWRITE, 
                                var_rmonAlarmTable, 4, { 3,1,1,8 } },
    { RMONALARMRISINGEVENTINDEX         , ASN_INTEGER   , RWRITE, 
                                var_rmonAlarmTable, 4, { 3,1,1,9 } },
    { RMONALARMFALLINGEVENTINDEX        , ASN_INTEGER   , RWRITE, 
                                var_rmonAlarmTable, 4, { 3,1,1,10 } },
    { RMONALARMOWNER                    , ASN_OCTET_STR , RWRITE, 
                                var_rmonAlarmTable, 4, { 3,1,1,11 } },
    { RMONALARMSTATUS                   , ASN_INTEGER   , RWRITE, 
                                var_rmonAlarmTable, 4, { 3,1,1,12 } },
    /* 55 */                 
    
    /* Event Group */
    { RMONEVENTINDEX                    , ASN_INTEGER   , RONLY , 
                                var_rmonEventTable, 4, { 9,1,1,1 } },
    { RMONEVENTDESCRIPTION              , ASN_OCTET_STR , RWRITE, 
                                var_rmonEventTable, 4, { 9,1,1,2 } },
    { RMONEVENTTYPE                     , ASN_INTEGER   , RWRITE, 
                                var_rmonEventTable, 4, { 9,1,1,3 } },
    { RMONEVENTCOMMUNITY                , ASN_OCTET_STR , RWRITE, 
                                var_rmonEventTable, 4, { 9,1,1,4 } },
    { RMONEVENTLASTTIMESENT             , ASN_TIMETICKS , RONLY , 
                                var_rmonEventTable, 4, { 9,1,1,5 } },
    { RMONEVENTOWNER                    , ASN_OCTET_STR , RWRITE, 
                                var_rmonEventTable, 4, { 9,1,1,6 } },
    { RMONEVENTSTATUS                   , ASN_INTEGER   , RWRITE, 
                                var_rmonEventTable, 4, { 9,1,1,7 } },
    /* 62 */                    

    /* Event Group : Log Table */
    { RMONLOGEVENTINDEX                , ASN_INTEGER   , RONLY , 
                                var_rmonEventLogTable, 4, { 9,2,1,1 } },
    { RMONLOGINDEX                     , ASN_INTEGER   , RONLY , 
                                var_rmonEventLogTable, 4, { 9,2,1,2 } },
    { RMONLOGTIME                      , ASN_TIMETICKS , RONLY , 
                                var_rmonEventLogTable, 4, { 9,2,1,3 } },
    { RMONLOGDESCRIPTION               , ASN_OCTET_STR , RONLY , 
                                var_rmonEventLogTable, 4, { 9,2,1,4 } },
    /* 66 */                    
    
};
/*    (L = length of the oidsuffix) */

/* ------------------ Symbol / Macro definition ------------------ */
/* the max RMON var length should be for LOG description */
#define MAX_RMON_VAR_LENGTH  MAX_OID_LEN * sizeof(oid)

/* create the object value data-struct for RMON interface usage :
 *      - the IF_VAR_VALUE with the type ==  rmonprobe_oid_value_t
 *      - normally, the MAX value will be assigned for each var get routine.
 */
#define CREATE_IF_VAR_VALUE(val_ds, vptype, len)      \
        (val_ds) = (rmonprobe_oid_value_t *) malloc(     \
                        sizeof(rmonprobe_oid_value_t));     \
        memset((val_ds), 0 , sizeof(rmonprobe_oid_value_t));    \
        (val_ds)->type = (vptype);                         \
        (val_ds)->value_length = (len);                  \
        (val_ds)->var_val = (u_char *) malloc((len));    \
        memset((val_ds)->var_val, 0 , (len))
        
/* destroy  the object value data-struct for RMON interface usage :
 *  - should be matched with the calling to CREATE_IF_VAR_VALUE macro.
 */
#define DESTROY_IF_VAR_VALUE(val_ds)     \
        if ((val_ds)->var_val != NULL){  \
            free((val_ds)->var_val);     \
        }                                \
        free((val_ds))
        
/* get parameter assignment :
 *  - this macro definition can used in this file only.
 *      (for some )
 */
#define GET_PARAM_ASSIGNENT(parms, in_name, len, snmp_st, rmonp_st, val_ds) \
        (parms).ecos_vp = vp;                   \
        (parms).namelen = &(len);               \
        (parms).name = (in_name);                  \
        (parms).exact = exact;                  \
        (parms).snmp_status = &(snmp_st);       \
        (parms).rmonp_status = &(rmonp_st);     \
        ((parms).oid_val) = &(val_ds)

/* var_val pasing for SNMP MIB var handler return */
#define RETURN_VAR_VAL_PARSING(val_ds, string_ret, oid_str_ret, long_ret)  \
        if ((val_ds)->type == ASN_OCTET_STR){                          \
            *var_len = (val_ds)->value_length;                         \
            memset((string_ret),0,sizeof((string_ret)));               \
            memcpy((string_ret),(val_ds)->var_val,*var_len);           \
        } else if((val_ds)->type == ASN_OBJECT_ID){                    \
            *var_len = (val_ds)->value_length * sizeof(oid);                     \
            memcpy((oid_str_ret), (val_ds)->var_val, *var_len);        \
        } else {                                                        \
            *var_len = (val_ds)->value_length;                         \
            (long_ret) = *((long *)((val_ds)->var_val));               \
        }
        

/* ------------------ function section ------------------ */


void 
init_rmonlite_mib(void)
{

  /* register ourselves with the agent to handle our mib tree */
  REGISTER_MIB("mibII/rmonlite", rmonlite_variables, variable5,
               rmon_variables_oid);

}

/*
 * var_rmonStatEtStatsTable():
 *   Handle this table separately from the scalar value case.
 */
unsigned char *
var_rmonStatEtStatsTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{

    /* variables we may use later */
    static long long_ret;
    static unsigned char string[OWNER_STRING_LEN];
    static oid objid[MAX_OID_LEN];
    
    struct rmonp_snmpop_get_parameters get_parms;
    rmonprobe_oid_value_t   *oid_val;
    int     i;
    int     snmp_status = SNMP_ERR_NOERROR;
    int     rmonp_status = 0;
    oid     *temp_name;
    short   temp_namelen;
    
    temp_name = malloc(sizeof(oid) * MAX_OID_LEN);
    memset(temp_name, 0 , sizeof(oid) * MAX_OID_LEN);

    if (snmp_oid_compare(vp->name, vp->namelen, name, *length) > 0){
        temp_namelen  = vp->namelen;
        memcpy(temp_name, vp->name, temp_namelen * sizeof(oid) );
        
    } else {
        temp_namelen  = *length;
        memcpy(temp_name, name, temp_namelen * sizeof(oid) );
    }
    
    /* create and assing the max value_len for each var */
    CREATE_IF_VAR_VALUE(oid_val, (u_char)vp->type, MAX_RMON_VAR_LENGTH);
    
    /* get_parms assignment */
    GET_PARAM_ASSIGNENT(get_parms, temp_name, temp_namelen, 
                    snmp_status, rmonp_status, oid_val);
    
    /* Note :
     *  - oid_val here is not allocated yet, we past this pointer to be the 
     *      parameter and will be malloc in rmon probe for the size could be 
     *      decided only in rmon probe.
     *  - this malloc will be free later.
     */
    snmp_call_callbacks(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_RMONPROBE_GET,
                        &get_parms);

    if (snmp_status != SNMP_ERR_NOERROR){
        if (temp_name != NULL){
            free(temp_name);
        }
        
        if (oid_val != NULL) {
            DESTROY_IF_VAR_VALUE(oid_val);
        }

        /* ---- special process for RMON control row create request ---- */
        /* if row creat request is processing, let the procedure been the 
         *  as normal get case to allowed write_method could be assigned 
         *  properly.
         */
        if ((vp->magic == RMONETHERSTATSSTATUS) && 
                        (snmp_status ==SNMP_ERR_NOSUCHNAME)){
            
            *write_method = write_rmonEntryStatus;
        }
        
        *var_len = 0;
        return NULL;
    }
    
    if (!exact){
        /* assign the processed name and the length */
        *length = temp_namelen;
        memcpy(name,temp_name,temp_namelen * sizeof(oid));
    }

    /* parse value for return */
    RETURN_VAR_VAL_PARSING(oid_val, string, objid, long_ret);
    
    /* Note :
     *  - oid_val should be free after we retrive this value and before the 
     *      end of this routine to prevent the memory leak.
     */
    if (temp_name != NULL){
        free(temp_name);
    }
    
    if (oid_val != NULL) {
        DESTROY_IF_VAR_VALUE(oid_val);
    }
    
    *write_method = 0;
    /* oid parsing process */
    switch(vp->magic) {
        /* etherStat */
        case RMONETHERSTATSINDEX                 :
            long_ret = name[(*length)-1];
            return (unsigned char *) &long_ret;
            
        case RMONETHERSTATSDATASOURCE            :
            *write_method = write_rmonNormalObj;
            return (unsigned char *) objid;

        case RMONETHERSTATSDROPEVENTS            :  
        case RMONETHERSTATSOCTETS                :
        case RMONETHERSTATSPKTS                  :
        case RMONETHERSTATSBROADCASTPKTS         :
        case RMONETHERSTATSMULTICASTPKTS         :
        case RMONETHERSTATSCRCALIGNERRORS        :
        case RMONETHERSTATSUNDERSIZEPKTS         :
        case RMONETHERSTATSOVERSIZEPKTS          :
        case RMONETHERSTATSFRAGMENTS             :
        case RMONETHERSTATSJABBERS               :
        case RMONETHERSTATSCOLLISIONS            :
        case RMONETHERSTATSPKTS64OCTETS          :
        case RMONETHERSTATSPKTS65TO127OCTETS     :
        case RMONETHERSTATSPKTS128TO255OCTETS    :
        case RMONETHERSTATSPKTS256TO511OCTETS    :
        case RMONETHERSTATSPKTS512TO1023OCTETS   :
        case RMONETHERSTATSPKTS1024TO1518OCTETS  :
            return (unsigned char *) &long_ret;
            
        case RMONETHERSTATSOWNER                 :
            *write_method = write_rmonNormalObj;
            return (unsigned char *) string;
            
        case RMONETHERSTATSSTATUS                :        
            *write_method = write_rmonEntryStatus;
            return (unsigned char *) &long_ret;
            
        default:
            ERROR_MSG("Err:eCos.snmp.mib2.rmon-Invalid etherStats var!\n");
    }       /* switch */
    
    return NULL;
}


/*
 * var_rmonHistCtrlTable():
 *   Handle this table separately from the scalar value case.
 */
unsigned char *
var_rmonHistCtrlTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static unsigned char string[OWNER_STRING_LEN];
    static oid objid[MAX_OID_LEN];

    struct rmonp_snmpop_get_parameters get_parms;
    rmonprobe_oid_value_t   *oid_val;
    int     i;
    int     snmp_status = SNMP_ERR_NOERROR;
    int     rmonp_status = 0;
    oid     *temp_name;
    short   temp_namelen;

    temp_name = malloc(sizeof(oid) * MAX_OID_LEN);
    memset(temp_name, 0 , sizeof(oid) * MAX_OID_LEN);

    if (snmp_oid_compare(vp->name, vp->namelen, name, *length) > 0){
        temp_namelen  = vp->namelen;
        memcpy(temp_name, vp->name, temp_namelen * sizeof(oid));
        
    } else {
        temp_namelen  = *length;
        memcpy(temp_name, name, temp_namelen * sizeof(oid));
    }

    /* create and assing the max value_len for each var */
    CREATE_IF_VAR_VALUE(oid_val, (u_char)vp->type, MAX_RMON_VAR_LENGTH);
    
    /* get_parms assignment */
    GET_PARAM_ASSIGNENT(get_parms, temp_name, temp_namelen, 
                    snmp_status, rmonp_status, oid_val);
    
    /* Note :
     *  - oid_val here is not allocated yet, we past this pointer to be the 
     *      parameter and will be malloc in rmon probe for the size could be 
     *      decided only in rmon probe.
     *  - this malloc will be free later.
     */    
    snmp_call_callbacks(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_RMONPROBE_GET,
                        &get_parms);
    
    if (snmp_status != SNMP_ERR_NOERROR){
        if (temp_name != NULL){
            free(temp_name);
        }
        
        if (oid_val != NULL) {
            DESTROY_IF_VAR_VALUE(oid_val);
        }
        
        /* ---- special process for RMON control row create request ---- */
        /* if row creat request is processing, let the procedure been the 
         *  as normal get case to allowed write_method could be assigned 
         *  properly.
         */
        if ((vp->magic == RMONHISTORYCONTROLSTATUS) && 
                        (snmp_status ==SNMP_ERR_NOSUCHNAME)){
            
            *write_method = write_rmonEntryStatus;
        }
        
        *var_len = 0;
        return NULL;
    }

    if (!exact){
        /* assign the processed name and the length */
        *length = temp_namelen;
        memcpy(name,temp_name,temp_namelen * sizeof(oid));
    }

    /* parse value for return */
    RETURN_VAR_VAL_PARSING(oid_val, string, objid, long_ret);

    /* Note :
     *  - oid_val should be free after we retrive this value and before the 
     *      end of this routine to prevent the memory leak.
     */
    if(temp_name!=NULL){
        free(temp_name);
    }
    if (oid_val != NULL) {
        
        DESTROY_IF_VAR_VALUE(oid_val);
    }
    
    *write_method = 0;
    /* oid parsing process */
    switch(vp->magic) {
        /* History Control */
        case RMONHISTORYCONTROLINDEX             : 
            return (unsigned char *) &long_ret;
        case RMONHISTORYCONTROLDATASOURCE        : 
            *write_method = write_rmonNormalObj;
            return (unsigned char *) objid;

        case RMONHISTORYCONTROLBUCKETSREQUESTED  : 
            *write_method = write_rmonNormalObj;
            return (unsigned char *) &long_ret;

        case RMONHISTORYCONTROLBUCKETSGRANTED    : 
            return (unsigned char *) &long_ret;
            
        case RMONHISTORYCONTROLINTERVAL          : 
            *write_method = write_rmonNormalObj;
            return (unsigned char *) &long_ret;
            
        case RMONHISTORYCONTROLOWNER             : 
            *write_method = write_rmonNormalObj;
            return (unsigned char *) string;

        case RMONHISTORYCONTROLSTATUS            : 
            *write_method = write_rmonEntryStatus;
            return (unsigned char *) &long_ret;
            
        default:
            ERROR_MSG("Err:eCos.snmp.mib2.rmon-Invalid histControl var!\n");
            break;
    }

    return NULL;
}


/*
 * var_rmonHistEtStatTable():
 *   Handle this table separately from the scalar value case.
 */
unsigned char *
var_rmonHistEtStatTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static oid objid[MAX_OID_LEN];
    
    unsigned char string[OWNER_STRING_LEN];    /* dummy definition */
    struct rmonp_snmpop_get_parameters get_parms;
    rmonprobe_oid_value_t   *oid_val;
    int     i;
    int     snmp_status = SNMP_ERR_NOERROR;
    int     rmonp_status = 0;
    oid     *temp_name;
    short   temp_namelen;

    temp_name = malloc(sizeof(oid) * MAX_OID_LEN);
    memset(temp_name, 0 , sizeof(oid) * MAX_OID_LEN);


    if (snmp_oid_compare(vp->name, vp->namelen, name, *length) > 0){
        temp_namelen  = vp->namelen;
        memcpy(temp_name, vp->name, temp_namelen * sizeof(oid) );
        
    } else {
        temp_namelen  = *length;
        memcpy(temp_name, name, temp_namelen * sizeof(oid));
    }
    
    /* create and assing the max value_len for each var */
    CREATE_IF_VAR_VALUE(oid_val, (u_char)vp->type, MAX_RMON_VAR_LENGTH);

    /* get_parms assignment */
    GET_PARAM_ASSIGNENT(get_parms, temp_name, temp_namelen, 
                    snmp_status, rmonp_status, oid_val);

    /* Note :
     *  - oid_val here is not allocated yet, we past this pointer to be the 
     *      parameter and will be malloc in rmon probe for the size could be 
     *      decided only in rmon probe.
     *  - this malloc will be free later.
     */
    snmp_call_callbacks(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_RMONPROBE_GET,
                        &get_parms);
    
    if (snmp_status != SNMP_ERR_NOERROR){
        if(temp_name != NULL){
            free(temp_name);
        }
        
        if (oid_val != NULL) {
            
            DESTROY_IF_VAR_VALUE(oid_val);
        }
        *var_len = 0;
        return NULL;
    }

    if (!exact){
         /* assign the processed name and the length */
        *length = temp_namelen;
        memcpy(name,temp_name,temp_namelen * sizeof(oid));
    }
    
   /* parse value for return */
    RETURN_VAR_VAL_PARSING(oid_val, string, objid, long_ret);
    
    /* Note :
     *  - oid_val should be free after we retrive this value and before the 
     *      end of this routine to prevent the memory leak.
     */
    if(temp_name != NULL){
        free(temp_name);
    }
    
    if (oid_val != NULL) {
        
        DESTROY_IF_VAR_VALUE(oid_val);
    }
    
    *write_method = 0;
    /* oid parsing process */
    switch(vp->magic) {
        /* etherHistory */
        case RMONETHERHISTORYINDEX               :    
        case RMONETHERHISTORYSAMPLEINDEX         :    
        case RMONETHERHISTORYINTERVALSTART       :    
        case RMONETHERHISTORYDROPEVENTS          :    
        case RMONETHERHISTORYOCTETS              :    
        case RMONETHERHISTORYPKTS                :    
        case RMONETHERHISTORYBROADCASTPKTS       :    
        case RMONETHERHISTORYMULTICASTPKTS       :    
        case RMONETHERHISTORYCRCALIGNERRORS      :    
        case RMONETHERHISTORYUNDERSIZEPKTS       :    
        case RMONETHERHISTORYOVERSIZEPKTS        :    
        case RMONETHERHISTORYFRAGMENTS           :    
        case RMONETHERHISTORYJABBERS             :    
        case RMONETHERHISTORYCOLLISIONS          :    
        case RMONETHERHISTORYUTILIZATION         :
            return (unsigned char *) &long_ret;

        default:
            ERROR_MSG("Err:eCos.snmp.mib2.rmon-Invalid histStats var!\n");
            break;
    }
    
    return NULL;
}


/*
 * var_rmonAlarmTable():
 *   Handle this table separately from the scalar value case.
 */
unsigned char *
var_rmonAlarmTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static unsigned char string[OWNER_STRING_LEN];
    static oid objid[MAX_OID_LEN];
    
    struct rmonp_snmpop_get_parameters get_parms;
    rmonprobe_oid_value_t   *oid_val;
    int     i;
    int     snmp_status = SNMP_ERR_NOERROR;
    int     rmonp_status = 0;
    oid     *temp_name;
    short   temp_namelen;

    temp_name = malloc(sizeof(oid) * MAX_OID_LEN);
    memset(temp_name, 0 , sizeof(oid) * MAX_OID_LEN);

    if (snmp_oid_compare(vp->name, vp->namelen, name, *length) > 0){
        temp_namelen  = vp->namelen;
        memcpy(temp_name, vp->name, temp_namelen * sizeof(oid) );
        
    } else {
        temp_namelen  = *length;
        memcpy(temp_name, name, temp_namelen * sizeof(oid));
    }
    
    /* create and assing the max value_len for each var */
    CREATE_IF_VAR_VALUE(oid_val, (u_char)vp->type, MAX_RMON_VAR_LENGTH);

    /* get_parms assignment */
    GET_PARAM_ASSIGNENT(get_parms, temp_name, temp_namelen, 
                    snmp_status, rmonp_status, oid_val);

    /* Note :
     *  - oid_val here is not allocated yet, we past this pointer to be the 
     *      parameter and will be malloc in rmon probe for the size could be 
     *      decided only in rmon probe.
     *  - this malloc will be free later.
     */
    snmp_call_callbacks(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_RMONPROBE_GET,
                        &get_parms);

    if (snmp_status != SNMP_ERR_NOERROR){
        if (temp_name != NULL){
            free(temp_name);
        }
        
        if (oid_val != NULL) {
            DESTROY_IF_VAR_VALUE(oid_val);
        }
        
        /* ---- special process for RMON control row create request ---- */
        /* if row creat request is processing, let the procedure been the 
         *  as normal get case to allowed write_method could be assigned 
         *  properly.
         */
        if ((vp->magic == RMONALARMSTATUS) && 
                        (snmp_status ==SNMP_ERR_NOSUCHNAME)){
            
            *write_method = write_rmonEntryStatus;
        }
        
        *var_len = 0;
        return NULL;
    }

    if (!exact){
         /* assign the processed name and the length */
        *length = temp_namelen;
        memcpy(name, temp_name, temp_namelen * sizeof(oid));
    }

   /* parse value for return */
    RETURN_VAR_VAL_PARSING(oid_val, string, objid, long_ret);

    /* Note :
     *  - oid_val should be free after we retrive this value and before the 
     *      end of this routine to prevent the memory leak.
     */
    if (temp_name != NULL){
        free(temp_name);
    }
    
    if (oid_val != NULL) {
        DESTROY_IF_VAR_VALUE(oid_val);
    }
    
    *write_method = 0;
    /* oid parsing process */
    switch(vp->magic) {
        /* Alarm Group */
        case RMONALARMINDEX                      :        
            return (unsigned char *) &long_ret;
        case RMONALARMINTERVAL                   :        
            *write_method = write_rmonNormalObj;
            return (unsigned char *) &long_ret;

        case RMONALARMVARIABLE                   :        
            *write_method = write_rmonNormalObj;
            return (unsigned char *) objid;
        
        case RMONALARMSAMPLETYPE                 :        
            *write_method = write_rmonNormalObj;
            return (unsigned char *) &long_ret;

        case RMONALARMVALUE                      :        
            return (unsigned char *) &long_ret;

        case RMONALARMSTARTUPALARM               :        
            *write_method = write_rmonNormalObj;
            return (unsigned char *) &long_ret;

        case RMONALARMRISINGTHRESHOLD            :        
            *write_method = write_rmonNormalObj;
            return (unsigned char *) &long_ret;

        case RMONALARMFALLINGTHRESHOLD           :        
            *write_method = write_rmonNormalObj;
            return (unsigned char *) &long_ret;

        case RMONALARMRISINGEVENTINDEX           :        
            *write_method = write_rmonNormalObj;
            return (unsigned char *) &long_ret;

        case RMONALARMFALLINGEVENTINDEX          :        
            *write_method = write_rmonNormalObj;
            return (unsigned char *) &long_ret;

        case RMONALARMOWNER                      :        
            *write_method = write_rmonNormalObj;
            return (unsigned char *) string;

        case RMONALARMSTATUS                     :       
            *write_method = write_rmonEntryStatus;
            return (unsigned char *) &long_ret;

        default:
            ERROR_MSG("Err:eCos.snmp.mib2.rmon-Invalid alarm var!");
            break;
    }
    
    return NULL;
}


/*
 * var_rmonEventTable():
 *   Handle this table separately from the scalar value case.
 */
unsigned char *
var_rmonEventTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static unsigned char string[MAX_EVENT_STRING_LEN];
    struct timeval last;
    
    oid objid[MAX_OID_LEN];        /* dummy definition */
    struct rmonp_snmpop_get_parameters get_parms;
    rmonprobe_oid_value_t   *oid_val;
    int     snmp_status = SNMP_ERR_NOERROR;
    int     rmonp_status = 0;
    oid     *temp_name;
    short   temp_namelen;
    int     i;

    temp_name = malloc(sizeof(oid) * MAX_OID_LEN);
    memset(temp_name, 0 , sizeof(oid) * MAX_OID_LEN);

    if (snmp_oid_compare(vp->name, vp->namelen, name, *length) > 0){
        temp_namelen  = vp->namelen;
        memcpy(temp_name, vp->name, temp_namelen * sizeof(oid) );
        
    } else {
        temp_namelen  = *length;
        memcpy(temp_name, name, temp_namelen * sizeof(oid));
    }
    
    /* create and assing the max value_len for each var */
    CREATE_IF_VAR_VALUE(oid_val, (u_char)vp->type, MAX_RMON_VAR_LENGTH);

    /* get_parms assignment */
    GET_PARAM_ASSIGNENT(get_parms, temp_name, temp_namelen, 
                    snmp_status, rmonp_status, oid_val);

    /* Note :
     *  - oid_val here is not allocated yet, we past this pointer to be the 
     *      parameter and will be malloc in rmon probe for the size could be 
     *      decided only in rmon probe.
     *  - this malloc will be free later.
     */
    snmp_call_callbacks(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_RMONPROBE_GET,
                        &get_parms);

    if (snmp_status != SNMP_ERR_NOERROR){
        if (temp_name != NULL){
            free(temp_name);
        }
        
        if (oid_val != NULL) {
            DESTROY_IF_VAR_VALUE(oid_val);
        }

        /* ---- special process for RMON control row create request ---- */
        /* if row creat request is processing, let the procedure been the 
         *  as normal get case to allowed write_method could be assigned 
         *  properly.
         */
        if ((vp->magic == RMONEVENTSTATUS) && 
                        (snmp_status ==SNMP_ERR_NOSUCHNAME)){
            
            *write_method = write_rmonEntryStatus;
        }
        
        *var_len = 0;
        return NULL;
    }

    if (!exact){
         /* assign the processed name and the length */
        *length = temp_namelen;
        memcpy(name,temp_name,temp_namelen * sizeof(oid));
    }

   /* parse value for return */
    RETURN_VAR_VAL_PARSING(oid_val, string, objid, long_ret);
    
    /* Note :
     *  - oid_val should be free after we retrive this value and before the 
     *      end of this routine to prevent the memory leak.
     */
    if (temp_name != NULL){
        free(temp_name);
    }
    
    if (oid_val != NULL) {
        DESTROY_IF_VAR_VALUE(oid_val);
    }
    
    *write_method = 0;
    /* oid parsing process */
    switch(vp->magic) {
        /* Event Group */                                     
        case RMONEVENTINDEX                      :          
            long_ret = name[(*length)-1];
            return (unsigned char *) &long_ret;
        case RMONEVENTDESCRIPTION                :          
            *write_method = write_rmonNormalObj;
            return (unsigned char *) string;

        case RMONEVENTTYPE                       :          
            *write_method = write_rmonNormalObj;
            return (unsigned char *) &long_ret;

        case RMONEVENTCOMMUNITY                  :          
            *write_method = write_rmonNormalObj;
            return (unsigned char *) string;

        case RMONEVENTLASTTIMESENT               :          
            return (unsigned char *) &long_ret;

        case RMONEVENTOWNER                      :          
            *write_method = write_rmonNormalObj;
            return (unsigned char *) string;

        case RMONEVENTSTATUS                     :          
            *write_method = write_rmonEntryStatus;
            return (unsigned char *) &long_ret;

        default:
            ERROR_MSG("Err:eCos.snmp.mib2.rmon-Invalid event var!");
            break;
    }
    
    return NULL;
}


/*
 * var_rmonEventLogTable():
 *   Handle this table separately from the scalar value case.
 */
unsigned char *
var_rmonEventLogTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static unsigned char string[MAX_LOG_STRING_LEN];
    struct timeval now;
    
    oid objid[MAX_OID_LEN];
    struct rmonp_snmpop_get_parameters get_parms;
    rmonprobe_oid_value_t   *oid_val;
    int     snmp_status = SNMP_ERR_NOERROR;
    int     rmonp_status = 0;
    oid     *temp_name;
    short   temp_namelen;
    int     i;

    temp_name = malloc(sizeof(oid) * MAX_OID_LEN);
    memset(temp_name, 0 , sizeof(oid) * MAX_OID_LEN);

    if (snmp_oid_compare(vp->name, vp->namelen, name, *length) > 0){
        temp_namelen  = vp->namelen;
        memcpy(temp_name, vp->name, temp_namelen * sizeof(oid) );
        
    } else {
        temp_namelen  = *length;
        memcpy(temp_name, name, temp_namelen * sizeof(oid));
    }
    
    /* create and assing the max value_len for each var */
    CREATE_IF_VAR_VALUE(oid_val, (u_char)vp->type, MAX_RMON_VAR_LENGTH);

    /* get_parms assignment */
    GET_PARAM_ASSIGNENT(get_parms, temp_name, temp_namelen, 
                    snmp_status, rmonp_status, oid_val);

    /* Note :
     *  - oid_val here is not allocated yet, we past this pointer to be the 
     *      parameter and will be malloc in rmon probe for the size could be 
     *      decided only in rmon probe.
     *  - this malloc will be free later.
     */
    snmp_call_callbacks(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_RMONPROBE_GET,
                        &get_parms);

    if (snmp_status != SNMP_ERR_NOERROR){
        if (temp_name != NULL){
            free(temp_name);
        }
        
        if (oid_val != NULL) {
            DESTROY_IF_VAR_VALUE(oid_val);
        }
        *var_len = 0;
        return NULL;
    }

    if (!exact){
         /* assign the processed name and the length */
        *length = temp_namelen;
        memcpy(name,temp_name,temp_namelen * sizeof(oid));
    }
    
   /* parse value for return */
    RETURN_VAR_VAL_PARSING(oid_val, string, objid, long_ret);
    
    /* Note :
     *  - oid_val should be free after we retrive this value and before the 
     *      end of this routine to prevent the memory leak.
     */
    if (temp_name != NULL){
        free(temp_name);
    }
    
    if (oid_val != NULL) {
        DESTROY_IF_VAR_VALUE(oid_val);
    }
    
    *write_method = 0;
    /* oid parsing process */
    switch(vp->magic) {
        /* Event Log Table */                                 
        case RMONLOGEVENTINDEX                   :          
            return (unsigned char *) &long_ret;

        case RMONLOGINDEX                        :          
        case RMONLOGTIME                         :          
            return (unsigned char *) &long_ret;

        case RMONLOGDESCRIPTION                  :       
            return (unsigned char *) string;
        default:
            ERROR_MSG("Err:eCos.snmp.mib2.rmon-Invalid log var!");
            break;
    }
    
    return NULL;
}

/* =========== Routines for write method ============ */


/*  Function : 
 *      write_rmonNormalObj()
 *  
 *  Purpose : 
 *      Set the normal object(except entry status variable).
 *  
 *  Return : 
 *      SNMP Error ID. The error id should be SNMPv2 error and the upper 
 *      layer will handle the return error status for SNMPv1.
 *
 *  Note :
 *      the snmp set varification will be done in RMON-Lite porbe callback
 *      routine. (varidication @ Type/Value/length)
 */   
int
write_rmonNormalObj(int    action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    struct rmonp_snmpop_set_parameters set_parms;
    rmonprobe_oid_value_t *oid_val;
    int     snmp_status = SNMP_ERR_NOERROR;
    int     rmonp_status = 0;
    int     i;

    /* donothing in below action and return no error */
    if (action == RESERVE2 || action == FREE || action == ACTION || 
                    action == UNDO ){
        return SNMP_ERR_NOERROR;
    }
    
    CREATE_IF_VAR_VALUE(oid_val, var_val_type, MAX_RMON_VAR_LENGTH);
    
    if (var_val_type == ASN_OBJECT_ID){
        oid_val->value_length = var_val_len / sizeof(oid) ;
    } else {
        oid_val->value_length = var_val_len;
    }

    set_parms.name = name;
    set_parms.namelen = (short *)name_len;
    set_parms.action = (action == RESERVE1) ? SNMP_ACT_RESERVE1 : 
                    (action == RESERVE2) ? SNMP_ACT_RESERVE2 : 
                    (action == ACTION) ? SNMP_ACT_ACTION : 
                    (action == COMMIT) ? SNMP_ACT_COMMIT : 
                    (action == FREE) ? SNMP_ACT_FREE : 
                    (action == UNDO) ? SNMP_ACT_UNDO : -1 ;
                    
    set_parms.oid_val = oid_val;
    
    set_parms.snmp_status = &snmp_status;
    set_parms.rmonp_status = &rmonp_status;
    
    if (var_val_type == ASN_OBJECT_ID){
        /* for ASN_OBJECT_ID type, the incomming value length is indicated 
         *  as real byte count but the oid number. So we can use this length 
         *  for memory alloc here.
         */
        memcpy(set_parms.oid_val->var_val, var_val, var_val_len);
        
    } else {
        memcpy(set_parms.oid_val->var_val, var_val, var_val_len);
    }
    
    snmp_call_callbacks(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_RMONPROBE_SET,
                        &set_parms);

    if (oid_val->var_val != NULL) {
        free(oid_val->var_val);
    }
        
    if (oid_val != NULL) {
        free(oid_val);
    }
    
    return snmp_status;
}


#define ROW_STAUTS_VALUD            1
#define ROW_STATUS_CREATREQUEST     2
#define ROW_CREAT_UNDERCREATION     3
#define ROW_STATUS_INVALUD          4
/*  Function : 
 *    write_rmonEntryStatus()
 *  
 *  Purpose : 
 *    Set the entryStatus object in EtherStatsTable, HistoryControlTable
 *      AlarmTable and EventTable
 *  
 *  return : 
 *      SNMP Error ID. The error id should be SNMPv2 error and the upper 
 *      layer will handle the return error status for SNMPv1.
 *
 *  note :
 *      - the Reserve1 action will doing the status value testing.
 *      - only the Commit action will really process as Row-Action on servicing
 *          the row create/delete/modify/enable.
 *      - row creat request :
 *          1. for set row as valid/underCreation/invalid, the RESERVE1 action
 *              will redirect to write_rmonNormalObj().
 *          2. this routine will service for below situation :
 *              a. set row as creatRequest, for both RESERVE1 and COMMIT 
 *                  action.
 *              b. set row as valid/invalid/underCreation when COMMIT action 
 *                  is assigned.
 */
int
write_rmonEntryStatus(int action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    
    struct rmonp_snmpop_set_parameters set_parms;
    rmonprobe_oid_value_t *oid_val;
    int     snmp_status = SNMP_ERR_NOERROR;
    int     rmonp_status = 0;
    int     i;

    
    /* donothing in below action and return no error */
    if (action == RESERVE1){ 
        if (*(u_long *)var_val != ROW_STATUS_CREATREQUEST ){
            return (write_rmonNormalObj(action, var_val, var_val_type,
                                var_val_len, statP, name, name_len));
        }
    } else if (action == RESERVE2 || action == FREE || action == ACTION || 
                    action == UNDO ){
        return SNMP_ERR_NOERROR;
    } else if (action == COMMIT){
        if (*(u_long *)var_val == ROW_STATUS_CREATREQUEST ){
            return SNMP_ERR_NOERROR;
        }
    }
    
    /* create the oid_val for the assigning value */
    CREATE_IF_VAR_VALUE(oid_val, var_val_type, var_val_len);
    
    /* ----- assigning the set_parameter ----- */
    set_parms.action = SNMP_ACT_ROW_PROCESS;
    set_parms.name = name;
    set_parms.namelen = (short *)name_len;
    set_parms.snmp_status = &snmp_status;
    set_parms.rmonp_status = &rmonp_status;
                    
    set_parms.oid_val = oid_val;
    memcpy(set_parms.oid_val->var_val, var_val, sizeof(u_long));
    
    snmp_call_callbacks(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_RMONPROBE_SET,
                        &set_parms);

    if (oid_val != NULL) {
        
        DESTROY_IF_VAR_VALUE(oid_val);
    }
    
    return snmp_status;

}

/* End of rmonlite_mib.c */

