/*
 * $Id: ssConfigGroup.c,v 1.7 2007/02/08 10:20:34 madhu Exp $
 */

/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include <ucd-snmp/mibgroup/ng700ss/netgear_mib.h>
#include "mibgroup/ng700ss/ssConfigGroup.h"

void print_oid(int *, int);

/* callback parameters */
static ngrmib_parameters_t *ssconfgrp_parms = NULL;

/* Index of IP address retrieved when perfect match */
static int ip_index = -1;

/* Creation row */
static ngr_ipfilter_entry_t         *ipf_creation_row = NULL;
static ngr_mirror_entry_t           *mirror_creation_row = NULL;
static ngr_tftpserver_entry_t       tftpserver_creation_row;
static ngr_boottftpserver_entry_t     boottftp_creation_row;

/* path to this MIB wrt the global MIB tree */
oid ssConfigGroup_variables_oid[] = { 1,3,6,1,4,1,4526,11,11 };

/* Variables that this MIB supports */
/* Note: This implementation supports only VLAN Mode and IP Access List */
struct variable7 ssConfigGroup_variables[] = {
#define CONFIGVERSWPRIMARY        1
    {CONFIGVERSWPRIMARY,  ASN_OCTET_STR,  RONLY ,  var_ssConfigGroup, 1,  { 1 }},
#define CONFIGVERHWCHIPSET        2
    {CONFIGVERHWCHIPSET,  ASN_OCTET_STR,  RONLY ,  var_ssConfigGroup, 1,  { 2 }},
#define CONFIGBOOTTFTPOPERATION        3
    {CONFIGBOOTTFTPOPERATION,  ASN_INTEGER,  RWRITE,  var_ssConfigGroup, 1,  { 3 }},
#define CONFIGBOOTTFTPSERVERIP        4
    {CONFIGBOOTTFTPSERVERIP,  ASN_IPADDRESS,  RWRITE,  var_ssConfigGroup, 1,  { 4 }},
#define CONFIGBOOTIMAGEFILENAME        5
    {CONFIGBOOTIMAGEFILENAME,  ASN_OCTET_STR,  RWRITE,  var_ssConfigGroup, 1,  { 5 }},
#define CONFIGPORT        6
    {CONFIGPORT,  ASN_INTEGER,  RONLY,   var_configPortTable, 3,  { 6, 1, 1 }},
#define CONFIGPORTSPEED        7
    {CONFIGPORTSPEED,  ASN_INTEGER,  RWRITE,  var_configPortTable, 3,  { 6, 1, 2 }},
#define CONFIGPORTOPERSTATUS        8
    {CONFIGPORTOPERSTATUS,  ASN_INTEGER,  RONLY,   var_configPortTable, 3,  { 6, 1, 3 }},
#define CONFIGPORTFLOWCONTROL        9
    {CONFIGPORTFLOWCONTROL,  ASN_INTEGER,  RWRITE,  var_configPortTable, 3,  { 6, 1, 4 }},
#define CONFIGPORTFLOWCONTROLOPER        10
    {CONFIGPORTFLOWCONTROLOPER,  ASN_INTEGER,  RONLY,   var_configPortTable, 3,  { 6, 1, 5 }},
#define CONFIGPORTPRIORITY        11
    {CONFIGPORTPRIORITY,  ASN_INTEGER,  RWRITE,  var_configPortTable, 3,  { 6, 1, 6 }},
#define CONFIGVLANMODE        12
    {CONFIGVLANMODE,  ASN_INTEGER,  RWRITE,  var_ssConfigGroup, 1,  { 7 }},
#define CONFIGMIRRORID        13
    {CONFIGMIRRORID,  ASN_INTEGER,  RONLY,   var_configMirrorTable, 3,  { 8, 1, 1 }},
#define CONFIGMIRRORMODE        14
    {CONFIGMIRRORMODE,  ASN_INTEGER,  RWRITE,  var_configMirrorTable, 3,  { 8, 1, 2 }},
#define CONFIGMIRRORMON        15
    {CONFIGMIRRORMON,  ASN_INTEGER,  RWRITE,  var_configMirrorTable, 3,  { 8, 1, 3 }},
#define CONFIGMIRRORSRC        16
    {CONFIGMIRRORSRC,  ASN_OCTET_STR,  RWRITE,  var_configMirrorTable, 3,  { 8, 1, 4 }},
#define CONFIGMIRRORSTATUS        17
    {CONFIGMIRRORSTATUS,  ASN_INTEGER,  RWRITE,  var_configMirrorTable, 3,  { 8, 1, 5 }},
#define CONFIGIPASSIGNMENTMODE        18
    {CONFIGIPASSIGNMENTMODE,  ASN_INTEGER,  RWRITE,  var_ssConfigGroup, 1,  { 12 }},
#define CONFIGPHYSADDRESS        19
    {CONFIGPHYSADDRESS,  ASN_OCTET_STR,  RONLY ,  var_ssConfigGroup, 1,  { 13 }},
#define CONFIGPASSWORDADMIN        20
    {CONFIGPASSWORDADMIN,  ASN_OCTET_STR,  RWRITE,  var_ssConfigGroup, 1,  { 15 }},
#define CONFIGIPADDRESS        21
    {CONFIGIPADDRESS,  ASN_IPADDRESS,  RWRITE,  var_ssConfigGroup, 1,  { 16 }},
#define CONFIGNETMASK        22
    {CONFIGNETMASK,  ASN_IPADDRESS,  RWRITE,  var_ssConfigGroup, 1,  { 17 }},
#define CONFIGGATEWAY        23
    {CONFIGGATEWAY,  ASN_IPADDRESS,  RWRITE,  var_ssConfigGroup, 1,  { 18 }},
#define CONFIGSAVE        24
    {CONFIGSAVE,  ASN_INTEGER,  RWRITE,  var_ssConfigGroup, 1,  { 19 }},
#define CONFIGVERBOOTROMIMAGE        25
    {CONFIGVERBOOTROMIMAGE,  ASN_OCTET_STR,  RONLY ,  var_ssConfigGroup, 1,  { 21 }},
#define CONFIGRESTOREDEFAULTS        26
    {CONFIGRESTOREDEFAULTS,  ASN_INTEGER,  RWRITE,  var_ssConfigGroup, 1,  { 22 }},
#define CONFIGTFTPSERVERIPADDRESS        27
    {CONFIGTFTPSERVERIPADDRESS,  ASN_IPADDRESS,  RWRITE,  var_ssConfigGroup, 1,  { 32 }},
#define CONFIGTFTPSERVERFILENAME        28
    {CONFIGTFTPSERVERFILENAME,  ASN_OCTET_STR,  RWRITE,  var_ssConfigGroup, 1,  { 33 }},
#define CONFIGTFTPOPERATION        29
    {CONFIGTFTPOPERATION,  ASN_INTEGER,  RWRITE,  var_ssConfigGroup, 1,  { 34 }},
#define CONFIGIPFILTER        30
    {CONFIGIPFILTER,  ASN_INTEGER,  RWRITE,  var_ssConfigGroup, 1,  { 35 }},
#define CONFIGIPFILTERIPADDRESS        31
    {CONFIGIPFILTERIPADDRESS,  ASN_IPADDRESS,  RONLY,   var_configIpFilterTable, 3,  { 36, 1, 1 }},
#define CONFIGIPFILTERSTATUS        32
    {CONFIGIPFILTERSTATUS,  ASN_INTEGER,  RWRITE,  var_configIpFilterTable, 3,  { 36, 1, 2 }},
#define CONFIGUSERNAME        33
    {CONFIGUSERNAME,  ASN_OCTET_STR,  RWRITE,  var_ssConfigGroup, 1,  { 37 }},
};
/*    (L = length of the oidsuffix) */
#define SS_CONF_TFTP_NAME  64
#define SS_CONF_MAX_FILE_SIZE 63
#define SS_CONF_MAX_UNAME_SIZE 20
#define SS_CONF_PORT_SPEED_MIN 1 
#define SS_CONF_PORT_SPEED_MAX 8 
#define SS_CONF_PORT_PRIO_MIN 1 
#define SS_CONF_PORT_PRIO_MAX 4 


/** Initializes the ssConfigGroup module */
    void
init_ssConfigGroup(void)
{
    AGENT_ENTER;

    /* Allocate memory for callback argument */
    ssconfgrp_parms = (ngrmib_parameters_t *) \
                      sal_alloc(sizeof(ngrmib_parameters_t), SS_CONF_GROUP_MIB);

    /* set group name */
    if (ssconfgrp_parms != NULL) {
        ssconfgrp_parms->group = GROUP_CONFIG; 

        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(SS_CONF_GROUP_MIB, ssConfigGroup_variables, variable7,
                ssConfigGroup_variables_oid);
    }

    AGENT_EXIT;
}

/*
 *  This function is called whenever a system configuration
 *  has to be done. We keep this function for (g)setting the 
 *  VLAN mode information.
 */
    unsigned char *
var_ssConfigGroup(struct variable *vp, 
        oid     *name, 
        size_t  *length, 
        int     exact, 
        size_t  *var_len, 
        WriteMethod **write_method)
{
    AGENT_ENTER;
    AGENT_OID_PRINT(*length, name);
    static long long_ret, loop;
    static unsigned char string[SPRINT_MAX_LEN];

    /* validate */
    if (header_generic(vp,name,length,exact,var_len,write_method)
            == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
        case CONFIGVERSWPRIMARY:
            ssconfgrp_parms->group_obj = SS_CONFIG_VERSWPRIMARY;
            break;
            
        case CONFIGVERHWCHIPSET:
            ssconfgrp_parms->group_obj = SS_CONFIG_VERHWCHIPSET;
            break;
            
        case CONFIGBOOTTFTPOPERATION:
            *write_method = write_configBootTftpOperation;
            long_ret=boottftp_creation_row.boottftpmode;
            return (unsigned char *)&long_ret;
            break;
            
        case CONFIGBOOTTFTPSERVERIP:
            *write_method = write_configBootTftpServerIp;
            *var_len = IP_ADDRESS_LEN;
            memcpy(string,boottftp_creation_row.boottftp_ipaddr,*var_len);
            return (unsigned char *)string;
            break;
            
        case CONFIGBOOTIMAGEFILENAME:
            *write_method = write_configBootImageFileName;
            *var_len=strlen(boottftp_creation_row.file_name);
            memcpy(string,boottftp_creation_row.file_name,*var_len);
            return (unsigned char *)string;
            break;
            
        case CONFIGIPASSIGNMENTMODE:
            *write_method = write_configIpAssignmentMode;
            ssconfgrp_parms->group_obj = SS_CONFIG_IPASSIGNMENTMODE;
            break;
            
        case CONFIGPHYSADDRESS:
            ssconfgrp_parms->group_obj = SS_CONFIG_PHYSADDRESS;
            break;
            
        case CONFIGPASSWORDADMIN:
            *write_method = write_configPasswordAdmin;
            ssconfgrp_parms->group_obj = SS_CONFIG_PASSWORDADMIN;
            break;
            
        case CONFIGIPADDRESS:
            *write_method = write_configIpAddress;
            ssconfgrp_parms->group_obj = SS_CONFIG_IPADDRESS;
            break;
            
        case CONFIGNETMASK:
            *write_method = write_configNetMask;
            ssconfgrp_parms->group_obj = SS_CONFIG_NETMASK;
            break;
            
        case CONFIGGATEWAY:
            *write_method = write_configGateway;
            ssconfgrp_parms->group_obj = SS_CONFIG_GATEWAY;
            break;
            
        case CONFIGSAVE:
            *write_method = write_configSave;
            ssconfgrp_parms->group_obj = SS_CONFIG_SAVE;
            break;
            
        case CONFIGVERBOOTROMIMAGE:
            ssconfgrp_parms->group_obj = SS_CONFIG_VERBOOTROMIMAGE;
            break;
            
        case CONFIGRESTOREDEFAULTS:
            *write_method = write_configRestoreDefaults;
            ssconfgrp_parms->group_obj = SS_CONFIG_RESTOREDEFAULTS;
            break;
            
        case CONFIGTFTPSERVERIPADDRESS:
            *write_method = write_configTftpServerIpAddress;
            *var_len = IP_ADDRESS_LEN;
            memcpy(string,tftpserver_creation_row.tftp_ipaddr,*var_len);
            return (unsigned char *)string;
            break;
            
        case CONFIGTFTPSERVERFILENAME:
            *write_method = write_configTftpServerFileName;
            *var_len=strlen(tftpserver_creation_row.file_name);
            memcpy(string,tftpserver_creation_row.file_name,*var_len);
            return (unsigned char *)string;
            break;
            
        case CONFIGTFTPOPERATION:
            *write_method = write_configTftpOperation;
            long_ret=tftpserver_creation_row.tftpmode;
            return (unsigned char *)&long_ret;
            break;
            
        case CONFIGUSERNAME:
            *write_method = write_configUserName;
            ssconfgrp_parms->group_obj = SS_CONFIG_USERNAME;
            break;
            
        case CONFIGIPFILTER:
            /* Currently, this is Read-Only */
#if 0
            *write_method = write_configIpFilter;
#else
            *write_method = OID_NOT_WRITABLE;
#endif /* if 0 */
            ssconfgrp_parms->group_obj = SS_CONFIG_IPFILTER;
            break;
        case CONFIGVLANMODE:
            *write_method = write_configVLANMode;
            ssconfgrp_parms->group_obj = SS_CONFIG_VLANMODE;
            break;
            
        default:
            ERROR_MSG("Undefined reference in Config Group table");
            break;
    }

    /* Callback */
    ssconfgrp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
            SNMPD_CALLBACK_NETGEAR,
            ssconfgrp_parms);

    /* Give the value back */

    /* String Handling */

    switch(vp->magic) {
        case CONFIGVERSWPRIMARY:
        case CONFIGVERHWCHIPSET:
        case CONFIGPASSWORDADMIN:
        case CONFIGUSERNAME:
        case CONFIGVERBOOTROMIMAGE:
            *var_len=ssconfgrp_parms->value;
            memcpy(string,ssconfgrp_parms->buf,*var_len);
            return (unsigned char *)string;
        case CONFIGIPADDRESS:
        case CONFIGNETMASK:
        case CONFIGGATEWAY:
            *var_len = IP_ADDRESS_LEN;
            memcpy(string,ssconfgrp_parms->buf,*var_len);
            return (unsigned char *)string;
        case CONFIGPHYSADDRESS:
            /* MAC Address */
            *var_len =MAC_ADDRESS_LEN;
            for (loop = 0; loop < *var_len; loop++) {
                string[loop] = ssconfgrp_parms->buf[loop];
            }
            return (unsigned char *) string;
        case CONFIGIPASSIGNMENTMODE:
        case CONFIGRESTOREDEFAULTS:
        case CONFIGIPFILTER:
        case CONFIGSAVE:
        case CONFIGVLANMODE:
            long_ret = (long)ssconfgrp_parms->value;
            AGENT_EXIT;
            return (unsigned char *)&long_ret;
    }
    return NULL;
}

static int
header_configPortTable(struct variable *vp,
        oid *name,
        size_t *length,
        int exact,
        size_t *var_len,
        WriteMethod **write_method)
{
#define SS_CONFIG_PORT_LEN     12
#define SS_CONFIG_PORT_TOTAL_LEN    13

    oid newname[MAX_OID_LEN];
    int i, found = FALSE, result;

    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    for(i = 1; i <= interface_count; i++) {
        newname[SS_CONFIG_PORT_LEN] = (oid)i;
        result = snmp_oid_compare(
                name, *length, newname, (int)vp->namelen + 1);
        if ((exact && (result == 0)) || (!exact && (result < 0))) {
            found = TRUE;
            ssconfgrp_parms->index = i;
            break;
        }
    }

    if ( found == FALSE ) {
        return MATCH_FAILED;
    }

    memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));

    *length = vp->namelen + 1;
    *write_method = 0;
    *var_len = sizeof(long);

    return MATCH_SUCCEEDED;
}

/*
 *  Function implementing the Port configuration table.
 */
unsigned char *
var_configPortTable(struct variable *vp,
        oid     *name,
        size_t  *length,
        int     exact,
        size_t  *var_len,
        WriteMethod **write_method)
{
    AGENT_ENTER;
    static long long_ret;

    /* Validate */
    snmp_mib_setInterfaceCount();
    if (header_configPortTable(vp,name,length,exact,var_len,write_method)
            == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
        case CONFIGPORT:
            ssconfgrp_parms->group_obj = SS_CONFIG_PORT;
            break;
        case CONFIGPORTSPEED:
            *write_method = write_configPortSpeed;
            ssconfgrp_parms->group_obj = SS_CONFIG_PORTSPEED;
            break;
        case CONFIGPORTOPERSTATUS:
            ssconfgrp_parms->group_obj = SS_CONFIG_PORTOPERSTATUS;
            break;
        case CONFIGPORTFLOWCONTROL:
            *write_method = write_configPortFlowControl;
            ssconfgrp_parms->group_obj = SS_CONFIG_PORTFLOWCONTROL;
            break;
        case CONFIGPORTFLOWCONTROLOPER:
            ssconfgrp_parms->group_obj = SS_CONFIG_PORTFLOWCONTROLOPER;
            break;
        case CONFIGPORTPRIORITY:
            *write_method = write_configPortPriority;
            ssconfgrp_parms->group_obj = SS_CONFIG_PORTPRIORITY;
            break;
        default:
            ERROR_MSG("Undefined MIB reference in Port config table");
            break;
    }

    /* Callback */
    ssconfgrp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
            SNMPD_CALLBACK_NETGEAR,
            ssconfgrp_parms);

    /* Give the value back */
    long_ret = (long)ssconfgrp_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}

int
header_configMirrorTable(struct variable *vp,
        oid *name,
        size_t *length,
        int exact,
        size_t *var_len,
        WriteMethod **write_method)
{
#define SS_CONFIG_MID_LEN           12
#define SS_CONFIG_MIRROR_TOTAL_LEN  13
#define SS_MAX_MIRROR_ROWS 1

    oid newname[MAX_OID_LEN];
    oid lowest[MAX_OID_LEN];
    int i = 0, found = FALSE, result;
    int mirror_count; 
    /* In case we are creating a new row, we need to set write method first */
    *write_method = write_mirrorEntry;
    if (mirror_creation_row) {
        mirror_count = 1;
    } else {
        /* Get count of number of Mirror entries on the switch */
        ssconfgrp_parms->callback_rw = CALLBACK_ACT_READ;
        ssconfgrp_parms->group_obj = SS_CONFIG_MIRROR_COUNT;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                SNMPD_CALLBACK_NETGEAR,
                ssconfgrp_parms);
        mirror_count = ssconfgrp_parms->value;
    }

    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
    if (mirror_count) //This table has only one row always
    {
        newname[SS_CONFIG_MID_LEN] = 1;
        result = snmp_oid_compare(name, *length, newname, (int) vp->namelen + 1);
        if ((exact && (result == 0)) || (!exact && (result < 0)))
        {
            found = TRUE;
        }
    }

    if (!found) {
        AGENT_EXIT;
        return MATCH_FAILED;
    }

    memcpy( (char *)name,(char *)newname, ((int) vp->namelen +1) * sizeof(oid));
    *length = vp->namelen + 1;
    *var_len = sizeof(long);    /* default to 'long' results */

    AGENT_EXIT;
    return MATCH_SUCCEEDED;
}

void print_oid(int *o, int len)
{
    int i;
    for(i=0;i<len;i++)
    {
        AGENT_DBG(("%d ", o[i]));
    }
    AGENT_DBG(("\n"));
}

/*
 * Function implementing system-wide mirror configuration
 */
unsigned char *
var_configMirrorTable(struct variable *vp,
        oid     *name,
        size_t  *length,
        int     exact,
        size_t  *var_len,
        WriteMethod **write_method)
{
    AGENT_ENTER;
    static unsigned char string[SPRINT_MAX_LEN];
    static long long_ret;

    /* Validate. */
    snmp_mib_setInterfaceCount();
    if (header_configMirrorTable(vp,name,length,exact,var_len,write_method)
            == MATCH_FAILED )
    {
        AGENT_EXIT;
        return NULL;
    }

    ssconfgrp_parms->index = 1;
    switch(vp->magic) {
        case CONFIGMIRRORID:
            long_ret = ssconfgrp_parms->index;
            return (unsigned char *) &long_ret;
            break;
        case CONFIGMIRRORMODE:
            ssconfgrp_parms->group_obj = SS_CONFIG_MIRRORMODE;
            if (mirror_creation_row)
            {
                long_ret = mirror_creation_row->mmode;
                return (unsigned char *) &long_ret;
            }
            break;
        case CONFIGMIRRORMON:
            ssconfgrp_parms->group_obj = SS_CONFIG_MIRRORMON;
            if (mirror_creation_row)
            {
                long_ret = mirror_creation_row->mmon_port;
                return (unsigned char *) &long_ret;
            }
            break;
        case CONFIGMIRRORSRC:
            ssconfgrp_parms->group_obj = SS_CONFIG_MIRRORSRC;
            if (mirror_creation_row)
            {
                *var_len = (interface_count/INTERFACE_BYTE_COUNT);
                memcpy (string, mirror_creation_row->msrc_plist, *var_len);
                return (unsigned char *) string;
            }
            break;
        case CONFIGMIRRORSTATUS:
            ssconfgrp_parms->group_obj = SS_CONFIG_MIRRORSTATUS;
            if (mirror_creation_row)
            {
                long_ret = mirror_creation_row->mstatus;
                return (unsigned char *) &long_ret;
            }
            break;
        default:
            ERROR_MSG("Undefined MIB object reference in Mirror Table");
            break;
    }

    /* Callback */
    ssconfgrp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
            SNMPD_CALLBACK_NETGEAR,
            ssconfgrp_parms);

    /* Give the value back */
    if (vp->magic == CONFIGMIRRORSRC)
    {
        memcpy (string, ssconfgrp_parms->buf, (interface_count/8));
        AGENT_EXIT;
        return (unsigned char *) string;
    }

    long_ret = (long)ssconfgrp_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}


/*
 * Write/Create function for Mirror Table
 */

int
write_mirrorEntry(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    int mirror_id;
    long value;
    int var;
    unsigned char  string[7];
    

    mirror_id = (int)name[SS_CONFIG_MID_LEN];
    ssconfgrp_parms->index=mirror_id;
    var = name[SS_CONFIG_MID_LEN - 1] + 12;
    
    value = *(long *) var_val;

    
    /* This table can have only one row. */
    if (mirror_creation_row == NULL && 
            (var == CONFIGMIRRORSTATUS && value == MIB_ROWSTATUS_CREATEANDWAIT))
    {
        ssconfgrp_parms->group_obj = SS_CONFIG_MIRROR_COUNT;
        ssconfgrp_parms->callback_rw = CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                SNMPD_CALLBACK_NETGEAR,
                ssconfgrp_parms);
        if (ssconfgrp_parms->index < 1 || ssconfgrp_parms->index > SS_MAX_MIRROR_ROWS)
        {
            return SNMP_ERR_WRONGVALUE;
        }

        mirror_creation_row = (ngr_mirror_entry_t *)
            sal_alloc(sizeof(ngr_mirror_entry_t),
                    SS_CONF_GROUP_MIB);
        if (mirror_creation_row == NULL)
        {
            return SNMP_ERR_RESOURCEUNAVAILABLE;
        }
        memset(mirror_creation_row, 0, sizeof(ngr_mirror_entry_t));
        mirror_creation_row->mid = mirror_id;
        ssconfgrp_parms->group_obj = SS_CONFIG_MIRRORSTATUS;
        ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
        mirror_creation_row->mstatus = 3;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                SNMPD_CALLBACK_NETGEAR,
                ssconfgrp_parms);
        AGENT_EXIT;
        return SNMP_ERR_NOERROR;	
    }else if (mirror_creation_row != NULL &&
            (var == CONFIGMIRRORSTATUS && value == MIB_ROWSTATUS_CREATEANDWAIT))
    {
        if(mirror_id!=mirror_creation_row->mid){
            return SNMP_ERR_RESOURCEUNAVAILABLE;
        }
    }

    switch (action)
    {
        case RESERVE1:
            switch(var)
            {
                case CONFIGMIRRORMODE:
                    if (var_val_type != ASN_INTEGER) {
                        fprintf(stderr, "write to ssConfig not ASN_INTEGER\n");
                        return SNMP_ERR_WRONGTYPE;
                    }
                    if (var_val_len > sizeof(long)) {
                        fprintf(stderr, "write to ssConfig bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }
                    if (value < 0 || value > 3)
                    {
                        fprintf(stderr, "write to ssConfig wrong value\n");
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;
                case CONFIGMIRRORMON:
                    if (var_val_type != ASN_INTEGER) {
                        fprintf(stderr, "write to ssConfig not ASN_INTEGER\n");
                        return SNMP_ERR_WRONGTYPE;
                    }
                    if (var_val_len > sizeof(long)) {
                        fprintf(stderr, "write to ssConfig bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }
                    if (value < 0 || value > interface_count)
                    {
                        fprintf(stderr, "write to ssConfig wrong value\n");
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;
                case CONFIGMIRRORSRC:
                    if (var_val_type != ASN_OCTET_STR) {
                        fprintf(stderr, "write to ssConfig not ASN_OCTET_STR\n");
                        return SNMP_ERR_WRONGTYPE;
                    }
                    if (var_val_len !=
                            (sizeof(char)*(interface_count/INTERFACE_BYTE_COUNT))) {
                        fprintf(stderr, "write to ssConfig bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }
                    break;
                case CONFIGMIRRORSTATUS:
                    if (var_val_type != ASN_INTEGER) {
                        fprintf(stderr, "write to ssConfig not ASN_INTEGER\n");
                        return SNMP_ERR_WRONGTYPE;
                    }

                    if (var_val_len > sizeof(long)) {
                        fprintf(stderr, "write to ssConfig: bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }

                    if ((value != MIB_ROWSTATUS_ACTIVE) &&
                            (value != MIB_ROWSTATUS_NOTREADY) &&
                           ( value != MIB_ROWSTATUS_CREATEANDWAIT) &&
                            (value != MIB_ROWSTATUS_DESTROY)) {
                        fprintf(stderr, "write to ssConfig: bad value %d\n", value);
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;
                default:
                    return SNMP_ERR_INCONSISTENTNAME;
            }
            break;

        case RESERVE2:
            break;

        case FREE:
            break;

        case ACTION:
            break;
        case UNDO:
            break;
        case COMMIT:
            if (mirror_creation_row)
            {
                if (mirror_id != mirror_creation_row->mid)
                {
                    return SNMP_ERR_INCONSISTENTNAME; 
                }
            }
            switch(var)
            {
                case CONFIGMIRRORMODE:
                    if((mirror_creation_row == NULL) || mirror_id != mirror_creation_row->mid)
                    {
                        ssconfgrp_parms->group_obj = SS_CONFIG_MIRRORMODE;
                        ssconfgrp_parms->value = (unsigned int) value;
                    }else if(mirror_creation_row)
                    {
                        mirror_creation_row->mmode = (unsigned int) value;
                    }
                    break;
                case CONFIGMIRRORMON:
                    if((mirror_creation_row == NULL) || 
                            mirror_id != mirror_creation_row->mid)
                    {
                        ssconfgrp_parms->group_obj = SS_CONFIG_MIRRORMON;
                        ssconfgrp_parms->value = (unsigned int) value;
                    }else if(mirror_creation_row)
                    {
                        mirror_creation_row->mmon_port = (unsigned int) value;
                    }
                    break;
                case CONFIGMIRRORSRC:
                    if((mirror_creation_row == NULL) || 
                            mirror_id != mirror_creation_row->mid)
                    {
                        memset (ssconfgrp_parms->buf, 0,strlen(ssconfgrp_parms->buf));
                        memcpy(ssconfgrp_parms->buf, var_val, var_val_len);
                        ssconfgrp_parms->value = var_val_len;
                        ssconfgrp_parms->group_obj = SS_CONFIG_MIRRORSRC;
                        ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
                    }else if(mirror_creation_row)
                    {
                        memset (mirror_creation_row->msrc_plist, 0,  var_val_len);
                        memcpy (mirror_creation_row->msrc_plist, var_val, var_val_len);
                    }
                    break;
                case CONFIGMIRRORSTATUS:
                    ssconfgrp_parms->group_obj = SS_CONFIG_MIRRORSTATUS;
                    ssconfgrp_parms->value = value;
                    if (mirror_creation_row == NULL)
                    {
                        if (value == MIB_ROWSTATUS_ACTIVE)
                        {
                            return SNMP_ERR_WRONGVALUE; /* Already active */
                        }
                    }else if(mirror_creation_row)
                    {
                        memset(&ssconfgrp_parms->group_data.mirror, 0,
                                                sizeof(ngr_mirror_entry_t));
                        memcpy(&ssconfgrp_parms->group_data.mirror,
                                mirror_creation_row, sizeof(ngr_mirror_entry_t));
                        if (value == MIB_ROWSTATUS_DESTROY)
                        {
                            sal_free (mirror_creation_row);
                            mirror_creation_row = NULL;
                            return SNMP_ERR_NOERROR;
                        } else if ((var == CONFIGMIRRORSTATUS) && (value == 5))
                        {
                            return SNMP_ERR_NOERROR;
                        }
                    }
                    break;
                default:
                    return SNMP_ERR_INCONSISTENTNAME;
            }
            /* call backend */
            if (mirror_creation_row == NULL || 
                    (mirror_creation_row &&
                     var == CONFIGMIRRORSTATUS && (value == 1 || value == 6)))
            {
                ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
                snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_NETGEAR,
                        ssconfgrp_parms);
                if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR)
                {
                    AGENT_EXIT;
                    return SNMP_ERR_COMMITFAILED;
                }
                /* If activating createAndWait, free mem */
                if (mirror_creation_row && mirror_id == mirror_creation_row->mid)
                {
                    sal_free(mirror_creation_row);
                    mirror_creation_row=NULL;
                }
            }
            break;
        default:
                    return SNMP_ERR_INCONSISTENTNAME;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

/*
 * Function to validate IP filter table access.
 */
    static int
header_configIpFilterTable(struct variable *vp,
        oid *name,
        size_t *length,
        int exact,
        size_t *var_len,
        WriteMethod **write_method)
{
#define IPFILTER_NAME_LENGTH    12
#define IPFILTER_TOTAL_OID_LENGTH 16
    AGENT_ENTER;
    AGENT_OID_PRINT(*length, name);

    oid newname[IPFILTER_TOTAL_OID_LENGTH];
    oid lowest[IPFILTER_TOTAL_OID_LENGTH];
    int num_ip_addr, ip_loop;
    int found = FALSE, loop_start;

    ip_index = -1;
    *write_method = write_configIpFilterStatus;
    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    /* Get the number of Addresses added to IP filter table */
    ssconfgrp_parms->group_obj = SS_CONFIG_IPCOUNT;
    ssconfgrp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
            SNMPD_CALLBACK_NETGEAR,
            ssconfgrp_parms);
    num_ip_addr = (int) ssconfgrp_parms->value;

    /* In case we have a row under creation, increment count */
    if (ipf_creation_row) {
        num_ip_addr++;
    }

    /* Get entry for each index and compare */
    ssconfgrp_parms->group_obj = SS_CONFIG_IPFILTERIPADDRESS;
    ssconfgrp_parms->callback_rw = CALLBACK_ACT_READ;
    for (loop_start = 0;  loop_start < num_ip_addr; loop_start++) {
        AGENT_DBG(("At index %d", loop_start));
        if (ipf_creation_row && loop_start == num_ip_addr -1) {
            memcpy (ssconfgrp_parms->buf, ipf_creation_row->cfgip, IP_ADDRESS_LEN);
            ipf_creation_row->cfgindex = loop_start;
        } else {
            ssconfgrp_parms->index = loop_start;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
        }

        /* Set value in newname */
        for (ip_loop = 0; ip_loop < IP_ADDRESS_LEN ; ip_loop++) {
            newname[IPFILTER_NAME_LENGTH + ip_loop] = 
                ssconfgrp_parms->buf[ip_loop];
            AGENT_DBG(("buf %d is %d", ip_loop, ssconfgrp_parms->buf[ip_loop]));
        }

        AGENT_OID_PRINT(IPFILTER_TOTAL_OID_LENGTH, newname);
        if (exact) {
            if (snmp_oid_compare(name, *length, newname, 
                        IPFILTER_TOTAL_OID_LENGTH) == 0) {
                memcpy(lowest, newname, 
                        IPFILTER_TOTAL_OID_LENGTH * sizeof(oid));
                found = TRUE;
                AGENT_DBG(("Found exact"));
                ip_index = loop_start;
                break; /* No need to continue further */
            }
        } else {
            if ((snmp_oid_compare(newname, IPFILTER_TOTAL_OID_LENGTH, 
                            name, *length) > 0) && (found == FALSE || 
                            snmp_oid_compare(newname, 
                                IPFILTER_TOTAL_OID_LENGTH, lowest, 
                                IPFILTER_TOTAL_OID_LENGTH) < 0))  {
                /* This is a get next command. So, continue the loop */
                memcpy((char *) lowest, (char *) newname,
                        IPFILTER_TOTAL_OID_LENGTH * sizeof(oid));
                found = TRUE;
                ip_index = loop_start;
                AGENT_DBG(("Found not exact"));
            }
        }
    }

    if (!found) {
        DEBUGMSGTL(("ng700ss/ssConfigGroup", "... index out of range\n"));
        AGENT_EXIT;
        return MATCH_FAILED;
    }

    memcpy( (char *)name,(char *)lowest, IPFILTER_TOTAL_OID_LENGTH * 
            sizeof(oid));
    *length = IPFILTER_TOTAL_OID_LENGTH;
    *var_len = sizeof(long);

    AGENT_EXIT;
    return MATCH_SUCCEEDED;
}

/*
 * This function will be called whenever there is an IP access list
 * operation to be done.
 */
    unsigned char *
var_configIpFilterTable(struct variable *vp,
        oid     *name,
        size_t  *length,
        int     exact,
        size_t  *var_len,
        WriteMethod **write_method)
{
    AGENT_ENTER;
    static long long_ret;
    int loop;
    static unsigned char string[SPRINT_MAX_LEN];

    /* validate */
    if (header_configIpFilterTable(vp,name,length,exact,var_len,
                write_method) == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    AGENT_OID_PRINT(*length, name);
    AGENT_DBG(("Found index %d", ip_index));

    ssconfgrp_parms->index = ip_index;
    switch(vp->magic) {
        case CONFIGIPFILTERIPADDRESS:
            ssconfgrp_parms->group_obj = SS_CONFIG_IPFILTERIPADDRESS;
            break;

        case CONFIGIPFILTERSTATUS:
            ssconfgrp_parms->group_obj = SS_CONFIG_IPFILTERSTATUS;
            break;

        default:
            ERROR_MSG("Undefined MIB reference in ssIPFilter Table");
            break;
    }

    /* Callback */
    ssconfgrp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
            SNMPD_CALLBACK_NETGEAR,
            ssconfgrp_parms);

    /* Give the value back */
    if (vp->magic == CONFIGIPFILTERIPADDRESS) {
        *var_len = IP_ADDRESS_LEN;
        for (loop = 0; loop < *var_len; loop++) {
            string[loop] = ssconfgrp_parms->buf[loop];
        }
        AGENT_EXIT;
        return (unsigned char *) string;
    }

    long_ret = (long)ssconfgrp_parms->value;
    AGENT_EXIT;
    return (unsigned char *) &long_ret;
}

    int
write_configBootTftpOperation(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    long value=0;
    int size;
    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_INTEGER) {
                fprintf(stderr, "write to ssConfigGroup not ASN_INTEGER\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len > sizeof(long)) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            break;

        case RESERVE2:
            size  = var_val_len;
            value = * (long *) var_val;
            if(value !=1 && value != 2){
                return SNMP_ERR_WRONGVALUE;
            }
            break;

        case FREE:
            /* Release any resources that have been allocated */
            break;

        case ACTION:
            /*
             * The variable has been stored in 'value' for you to use,
             * and you have just been asked to do something with it.
             * Note that anything done here must be reversable in the UNDO case
             */
            break;

        case UNDO:
            /* Back out any changes made in the ACTION case */
            break;

        case COMMIT:
            /* Now do the set */
            value = * (int *) var_val;
            boottftp_creation_row.boottftpmode = value;
            memcpy(&ssconfgrp_parms->group_data,&boottftp_creation_row,
                    sizeof(ngr_boottftpserver_entry_t));
            ssconfgrp_parms->group_obj =SS_CONFIG_BOOTTFTPOPERATION;
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configBootTftpServerIp(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    u_long value;
    int size;
    unsigned char temp_ip[IP_ADDRESS_LEN];
    memset(temp_ip,0,IP_ADDRESS_LEN);


    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_IPADDRESS) {
                fprintf(stderr, "write to ssConfigGroup not ASN_IPADDRESS\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len != sizeof(char) * IP_ADDRESS_LEN) {
                fprintf(stderr,"write to ssTrustMACGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            if (!memcmp(var_val, temp_ip, var_val_len)) {
                AGENT_EXIT;
                return SNMP_ERR_WRONGVALUE;
            }
            break;

        case RESERVE2:
            size  = var_val_len;
            value = * (u_long *) var_val;
            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            memcpy(boottftp_creation_row.boottftp_ipaddr,var_val,var_val_len);
            if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR) {
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configBootImageFileName(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    char value;
    int size, count;
    u_char          *cp;

    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_OCTET_STR) {
                fprintf(stderr, "write to ssConfigGroup not ASN_OCTET_STR\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len > sizeof(char) * SS_CONF_MAX_FILE_SIZE) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }

            break;

        case RESERVE2:
            size  = var_val_len;
            value = * (char *) var_val;

            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            memcpy(boottftp_creation_row.file_name,var_val,var_val_len);
            printf("file_name :%s\n",boottftp_creation_row.file_name);
            if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR) {
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configVLANMode(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;

    ssconfgrp_parms->value = * (int *) var_val;
    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_INTEGER) {
                fprintf(stderr, "write to ssConfigGroup not ASN_INTEGER\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }

            if (var_val_len > sizeof(long)) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }

            if (ssconfgrp_parms->value != MIB_VLAN_MODE_PORTBASED 
                    && ssconfgrp_parms->value != MIB_VLAN_MODE_TAGGED) {
                AGENT_EXIT;
                return SNMP_ERR_WRONGVALUE;
            }
            break;

        case RESERVE2:
            break;

        case FREE:
            /* Release any resources that have been allocated */
            break;

        case ACTION:
            break;

        case UNDO:
            /* Back out any changes made in the ACTION case */
            break;

        case COMMIT:
            ssconfgrp_parms->group_obj = SS_CONFIG_VLANMODE;
            if (snmp_mib_getVlanMode() == ssconfgrp_parms->value) {
                AGENT_EXIT;
                return SNMP_ERR_NOERROR;
            }

            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configIpAssignmentMode(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    long value;
    int size;

    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_INTEGER) {
                fprintf(stderr, "write to ssConfigGroup not ASN_INTEGER\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len > sizeof(long)) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            break;

        case RESERVE2:
            size  = var_val_len;
            value = * (long *) var_val;
            if((value<1) || (value>3))
            {
                return SNMP_ERR_WRONGVALUE;
            }

            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            value = * (int *) var_val;
            ssconfgrp_parms->value = value;
            ssconfgrp_parms->group_obj =SS_CONFIG_IPASSIGNMENTMODE;
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val != MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configPasswordAdmin(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    char value;
    int size;

    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_OCTET_STR) {
                fprintf(stderr, "write to ssConfigGroup not ASN_OCTET_STR\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len >SS_CONF_MAX_UNAME_SIZE) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }

            break;

        case RESERVE2:
            size  = var_val_len;
            value = * (char *) var_val;

            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            ssconfgrp_parms->group_obj =SS_CONFIG_PASSWORDADMIN;
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            memset(ssconfgrp_parms->buf,0,sizeof(ssconfgrp_parms->buf));
            memcpy(ssconfgrp_parms->buf,var_val,var_val_len);
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val =! MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }

            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configIpAddress(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    u_long value;
    int size;
    unsigned char temp_ip[IP_ADDRESS_LEN];
    memset(temp_ip,0,IP_ADDRESS_LEN);

    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_IPADDRESS) {
                fprintf(stderr, "write to ssConfigGroup not ASN_IPADDRESS\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len != sizeof(char) * IP_ADDRESS_LEN) {
                fprintf(stderr,"write to ssTrustMACGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            if (!memcmp(var_val, temp_ip, var_val_len)) {
                AGENT_EXIT;
                return SNMP_ERR_WRONGVALUE;
            }
            break;

        case RESERVE2:
            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            memcpy(ssconfgrp_parms->buf, var_val, var_val_len);
            ssconfgrp_parms->group_obj = SS_CONFIG_IPADDRESS;
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;

            /* Callback */
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val =! MIB_STATUS_ERROR) {
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configNetMask(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    u_long value;
    int size;
    unsigned char   temp_ip[IP_ADDRESS_LEN];
    memset (temp_ip, 0,IP_ADDRESS_LEN);

    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_IPADDRESS) {
                fprintf(stderr, "write to ssConfigGroup not ASN_IPADDRESS\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len != sizeof(char) * IP_ADDRESS_LEN) {
                fprintf(stderr,"write to ssTrustMACGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            if (!memcmp(var_val, temp_ip, var_val_len)) {
                AGENT_EXIT;
                return SNMP_ERR_WRONGVALUE;
            }

            break;

        case RESERVE2:
            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            memcpy(ssconfgrp_parms->buf, var_val, var_val_len);
            ssconfgrp_parms->group_obj = SS_CONFIG_NETMASK;
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;

            /* Callback */
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val =! MIB_STATUS_ERROR) {
                return SNMP_ERR_COMMITFAILED;
            }

            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configGateway(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    u_long value;
    int size;
    unsigned char temp_ip[IP_ADDRESS_LEN];
    memset(temp_ip,0,IP_ADDRESS_LEN);

    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_IPADDRESS) {
                fprintf(stderr, "write to ssConfigGroup not ASN_IPADDRESS\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len != sizeof(char) * IP_ADDRESS_LEN) {
                fprintf(stderr,"write to ssTrustMACGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            if (!memcmp(var_val, temp_ip, var_val_len)) {
                AGENT_EXIT;
                return SNMP_ERR_WRONGVALUE;
            }
            break;

        case RESERVE2:
            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            memcpy(ssconfgrp_parms->buf, var_val, var_val_len);
            ssconfgrp_parms->group_obj =SS_CONFIG_GATEWAY;
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;

            /* Callback */
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val =! MIB_STATUS_ERROR) {
                return SNMP_ERR_COMMITFAILED;
            }


            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configSave(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    long value;
    int size;

    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_INTEGER) {
                fprintf(stderr, "write to ssConfigGroup not ASN_INTEGER\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len > sizeof(long)) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            break;

        case RESERVE2:
            size  = var_val_len;
            value = * (long *) var_val;
            if(value!=1 && value!=2){
                return SNMP_ERR_WRONGVALUE;
            }
            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            value = * (int *) var_val;
            ssconfgrp_parms->value = value;
            ssconfgrp_parms->group_obj =SS_CONFIG_SAVE;
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configRestoreDefaults(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    long value;
    int size;

    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_INTEGER) {
                fprintf(stderr, "write to ssConfigGroup not ASN_INTEGER\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len > sizeof(long)) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            break;

        case RESERVE2:
            size  = var_val_len;
            value = * (long *) var_val;
            if(value !=1 && value != 2){
                return SNMP_ERR_WRONGVALUE;
            }

            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            value = * (int *) var_val;
            ssconfgrp_parms->value = value;
            ssconfgrp_parms->group_obj =SS_CONFIG_RESTOREDEFAULTS;
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }

            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configTftpServerIpAddress(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    u_long value;
    int size;
    unsigned char temp_ip[IP_ADDRESS_LEN];
    memset(temp_ip,0,IP_ADDRESS_LEN);

    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_IPADDRESS) {
                fprintf(stderr, "write to ssConfigGroup not ASN_IPADDRESS\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len != sizeof(char) * IP_ADDRESS_LEN) {
                fprintf(stderr,"write to ssTrustMACGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            if (!memcmp(var_val, temp_ip, var_val_len)) {
                AGENT_EXIT;
                return SNMP_ERR_WRONGVALUE;
            }
            break;

        case RESERVE2:
            size  = var_val_len;
            value = * (u_long *) var_val;

            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            ssconfgrp_parms->group_obj = SS_CONFIG_TFTPSERVERIPADDRESS;
            memcpy(tftpserver_creation_row.tftp_ipaddr, var_val, var_val_len);
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR) {
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configTftpServerFileName(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    char value;
    int size;

    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_OCTET_STR) {
                fprintf(stderr, "write to ssConfigGroup not ASN_OCTET_STR\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len > sizeof(char) * SS_CONF_TFTP_NAME) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            break;

        case RESERVE2:
            size  = var_val_len;
            value = * (char *) var_val;

            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            memcpy(tftpserver_creation_row.file_name,var_val,var_val_len);
            if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR) {
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configTftpOperation(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    long value;
    int size;

    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_INTEGER) {
                fprintf(stderr, "write to ssConfigGroup not ASN_INTEGER\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len > sizeof(long)) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            break;

        case RESERVE2:
            size  = var_val_len;
            value = * (long *) var_val;
            if(value!=1 && value!=2){
                return SNMP_ERR_WRONGVALUE;
            }
            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            value = * (int *) var_val;
            tftpserver_creation_row.tftpmode = value;
            memcpy(&ssconfgrp_parms->group_data.tftpserver,
                    &tftpserver_creation_row,sizeof(ngr_tftpserver_entry_t));
            ssconfgrp_parms->group_obj =SS_CONFIG_TFTPOPERATION;
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configIpFilter(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;

    ssconfgrp_parms->value = * (int *) var_val;
    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_INTEGER) {
                fprintf(stderr, "write to ssConfigGroup not ASN_INTEGER\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len > sizeof(long)) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            if (ssconfgrp_parms->value != MIB_STATUS_ENABLED && 
                    ssconfgrp_parms->value != MIB_STATUS_DISABLED) {
                AGENT_EXIT;
                return SNMP_ERR_WRONGVALUE;
            }
            break;

        case RESERVE2:
            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            ssconfgrp_parms->group_obj = SS_CONFIG_IPFILTER;
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


    int
write_configUserName(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    char value;
    int size;

    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_OCTET_STR) {
                fprintf(stderr, "write to ssConfigGroup not ASN_OCTET_STR\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len > SS_CONF_MAX_UNAME_SIZE) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            break;

        case RESERVE2:
            size  = var_val_len;
            value = * (char *) var_val;

            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            ssconfgrp_parms->group_obj =SS_CONFIG_USERNAME;
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            memset(ssconfgrp_parms->buf,0,sizeof(ssconfgrp_parms->buf));
            memcpy(ssconfgrp_parms->buf,var_val,var_val_len);
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val =! MIB_STATUS_ERROR) {
                return SNMP_ERR_COMMITFAILED;
            }

            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

    int
write_configPortSpeed(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    long value;
    int curr_value;

    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_INTEGER) {
                fprintf(stderr, "write to ssConfigGroup not ASN_INTEGER\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len > sizeof(long)) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            break;

        case RESERVE2:
            value = * (long *) var_val;
            if (value < SS_CONF_PORT_SPEED_MIN || value > SS_CONF_PORT_SPEED_MAX) {
                fprintf(stderr,"write to ssConfGroup: bad value\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGVALUE;
            }

            break;

        case FREE:
            break;

        case ACTION:
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_READ;
            ssconfgrp_parms->group_obj = SS_CONFIG_PORTSPEED;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            break;

        case UNDO:
            /* Back out any changes made in the ACTION case */
            break;

        case COMMIT:
            curr_value = ssconfgrp_parms->value;
            ssconfgrp_parms->value = * (long *) var_val;
            if (ssconfgrp_parms->value == curr_value) {
                AGENT_DBG(("Current value already set"));
                AGENT_EXIT;
                return SNMP_ERR_NOERROR;
            }

            /* Let us do the commit */
            ssconfgrp_parms->group_obj = SS_CONFIG_PORTSPEED;
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}
    int
write_configPortFlowControl(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    int curr_value;
    AGENT_ENTER;

    ssconfgrp_parms->value = * (int *) var_val;
    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_INTEGER) {
                fprintf(stderr, "write to ssConfigGroup not ASN_INTEGER\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len > sizeof(long)) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            if (ssconfgrp_parms->value != MIB_STATUS_ENABLED &&
                    ssconfgrp_parms->value != MIB_STATUS_DISABLED) {
                AGENT_EXIT;
                return SNMP_ERR_WRONGVALUE;
            }
            break;

        case RESERVE2:
            break;

        case FREE:
            break;

        case ACTION:
            break;

        case UNDO:
            break;

        case COMMIT:
            ssconfgrp_parms->value = * (long *) var_val;
            /* Let us do the commit */
            ssconfgrp_parms->group_obj = SS_CONFIG_PORTFLOWCONTROL;
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}
    int
write_configPortPriority(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
    AGENT_ENTER;
    long value;
    int curr_value;

    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_INTEGER) {
                fprintf(stderr, "write to ssConfigGroup not ASN_INTEGER\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len > sizeof(long)) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            value = * (long *) var_val;
            if (value < SS_CONF_PORT_PRIO_MIN || value > SS_CONF_PORT_PRIO_MAX) {
                fprintf(stderr,"write to ssConfGroup: bad value\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGVALUE;
            }
            break;

        case RESERVE2:
            break;

        case FREE:
            break;

        case ACTION:
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_READ;
            ssconfgrp_parms->group_obj = SS_CONFIG_PORTFLOWCONTROL;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            break;

        case UNDO:
            break;

        case COMMIT:
            curr_value = ssconfgrp_parms->value;
            ssconfgrp_parms->value = * (long *) var_val;
            if (ssconfgrp_parms->value == curr_value) {
                AGENT_DBG(("Current value already set"));
                AGENT_EXIT;
                return SNMP_ERR_NOERROR;
            }

            /* Let us do the commit */
            ssconfgrp_parms->group_obj = SS_CONFIG_PORTPRIORITY;
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}



    int
write_configIpFilterStatus(int      action,
        u_char   *var_val,
        u_char   var_val_type,
        size_t   var_val_len,
        u_char   *statP,
        oid      *name,
        size_t   name_len)
{
#define IP_ADDRESS_START        12
    AGENT_ENTER;
    int value;

    value = *(long *) var_val;
    if (ipf_creation_row == NULL && 
            (value == MIB_ROWSTATUS_CREATEANDGO || value == MIB_ROWSTATUS_ACTIVE)) {
        ipf_creation_row = (ngr_ipfilter_entry_t *) 
            sal_alloc(sizeof(ngr_ipfilter_entry_t), 
                    SS_CONF_GROUP_MIB);

        if (ipf_creation_row == NULL) {
            AGENT_DBG(("Unable to allocate memory"));
            return SNMP_ERR_RESOURCEUNAVAILABLE;
        }

        /* zero out all fields */
        memset(ipf_creation_row, 0, sizeof(ngr_ipfilter_entry_t));

        /* Copy IP Address */
        ipf_creation_row->cfgip[0] = name[IP_ADDRESS_START];
        ipf_creation_row->cfgip[1] = name[IP_ADDRESS_START+1];
        ipf_creation_row->cfgip[2] = name[IP_ADDRESS_START+2];
        ipf_creation_row->cfgip[3] = name[IP_ADDRESS_START+3];
        AGENT_DBG(("IP ADDRESS = %d.%d.%d.%d", ipf_creation_row->cfgip[0],
                    ipf_creation_row->cfgip[2], ipf_creation_row->cfgip[1],
                    ipf_creation_row->cfgip[3]));
    } else if (ipf_creation_row != NULL && 
            (value == MIB_ROWSTATUS_CREATEANDGO || value == MIB_ROWSTATUS_ACTIVE)) {
        //if (ipf_creation_row->cfgindex != ip_index) {
        if (ipf_creation_row->cfgip[0] != name[IP_ADDRESS_START] &&
                ipf_creation_row->cfgip[1] != name[IP_ADDRESS_START+1] &&
                ipf_creation_row->cfgip[2] != name[IP_ADDRESS_START+2] &&
                ipf_creation_row->cfgip[3] != name[IP_ADDRESS_START+3]) {
            AGENT_DBG(("Only one row can be under creation"));
            return SNMP_ERR_RESOURCEUNAVAILABLE;
        }
    }

    ssconfgrp_parms->group_obj = SS_CONFIG_IPFILTERSTATUS;
    ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
    switch ( action ) {
        case RESERVE1:
            if (var_val_type != ASN_INTEGER) {
                fprintf(stderr, "write to ssConfigGroup not ASN_INTEGER\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGTYPE;
            }
            if (var_val_len > sizeof(long)) {
                fprintf(stderr,"write to ssConfigGroup: bad length\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGLENGTH;
            }
            if (value != MIB_ROWSTATUS_ACTIVE && 
                    value != MIB_ROWSTATUS_CREATEANDGO && 
                    value != MIB_ROWSTATUS_DESTROY) {
                fprintf(stderr,"write to ssConfigGroup: wrong Value\n");
                AGENT_EXIT;
                return SNMP_ERR_WRONGVALUE;
            }
            break;

        case RESERVE2:
            break;

        case FREE:
            /* Release any resources that have been allocated */
            break;

        case ACTION:
            if (ip_index == -1) {
                return SNMP_ERR_WRONGVALUE;
            }
            break;

        case UNDO:
            /* Back out any changes made in the ACTION case */
            break;

        case COMMIT:
            ssconfgrp_parms->value = value;
            ssconfgrp_parms->index = ip_index;
            if (ipf_creation_row && ipf_creation_row->cfgindex == ip_index) {
                if (value == MIB_ROWSTATUS_CREATEANDGO 
                        || value == MIB_ROWSTATUS_ACTIVE) { /* createAndGo */
                    memcpy (&ssconfgrp_parms->group_data.ipfilter,
                            ipf_creation_row, 
                            sizeof(ngr_ipfilter_entry_t));
                }
            } else if (value == MIB_ROWSTATUS_ACTIVE 
                    || value == MIB_ROWSTATUS_CREATEANDGO) {
                sal_free (ipf_creation_row);
                ipf_creation_row = NULL;
                return SNMP_ERR_WRONGVALUE;
            }
            ssconfgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssconfgrp_parms);
            /*if (ipf_creation_row && 
              ipf_creation_row->cfgindex == ip_index && */
            if (ipf_creation_row && 
                    (value == MIB_ROWSTATUS_CREATEANDGO || 
                     value == MIB_ROWSTATUS_ACTIVE)) {
                sal_free (ipf_creation_row);
                ipf_creation_row = NULL;
            }
            if (ssconfgrp_parms->ret_val == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
    }
