/*
 * $Id: ssHostGroup.c,v 1.1 2006/12/11 06:25:14 ganesh Exp $
 */

/* This should always be included first before anything else */
#include <config.h>
#include <ctype.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include <ucd-snmp/mibgroup/ng700ss/netgear_mib.h>
#include "mibgroup/ng700ss/ssHostGroup.h"

#define SS_HOST_MAX_ALLOWED    4
#define SS_HOST_MAX_STR_SIZE   20

/* callback parameters */
static ngrmib_parameters_t *sshostgrp_parms = NULL;

/* OID that defines the path to this MIB */
oid ssHostGroup_variables_oid[] = { 1,3,6,1,4,1,4526,11,2 };

/* Row under creation */
static ngr_host_entry_t         *creation_row = NULL;

/* Variables that this MIb supports */
struct variable7 ssHostGroup_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

#define HOSTINDEX       1
    {HOSTINDEX,  ASN_INTEGER,  RONLY,   var_hostTable, 3,  { 1, 1, 1 }},
#define HOSTNAME        2
    {HOSTNAME,  ASN_OCTET_STR,  RWRITE,  var_hostTable, 3,  { 1, 1, 2 }},
#define HOSTIP      3
    {HOSTIP,  ASN_IPADDRESS,  RWRITE,  var_hostTable, 3,  { 1, 1, 3 }},
#define HOSTCOMM        4
    {HOSTCOMM,  ASN_OCTET_STR,  RWRITE,  var_hostTable, 3,  { 1, 1, 4 }},
#define HOSTATTRIBUTE       5
    {HOSTATTRIBUTE,  ASN_INTEGER,  RWRITE,  var_hostTable, 3,  { 1, 1, 5 }},
#define HOSTTRAPSNMPBOOTUP      6
    {HOSTTRAPSNMPBOOTUP,  ASN_INTEGER,  RWRITE,  var_hostTable, 3,  { 1, 1, 6 }},
#define HOSTTRAPSNMPLINKUPDOWN      7
    {HOSTTRAPSNMPLINKUPDOWN,  ASN_INTEGER,  RWRITE,  var_hostTable, 3,  { 1, 1, 7 }},
#define HOSTTRAPTRAPINSERTREMOVE        8
    {HOSTTRAPTRAPINSERTREMOVE,  ASN_INTEGER,  RWRITE,  var_hostTable, 3,  { 1, 1, 8 }},
#define HOSTTRAPTRAPABNORMALTPRXERROR       9
    {HOSTTRAPTRAPABNORMALTPRXERROR,  ASN_INTEGER,  RWRITE,  var_hostTable, 3,  { 1, 1, 9 }},
#define HOSTTRAPTRAPABNORMALTPTXERROR       10
    {HOSTTRAPTRAPABNORMALTPTXERROR,  ASN_INTEGER,  RWRITE,  var_hostTable, 3,  { 1, 1, 10 }},
#define HOSTTRAPTRAPABNORMALFIBERRXERROR        11
    {HOSTTRAPTRAPABNORMALFIBERRXERROR,  ASN_INTEGER,  RWRITE,  var_hostTable, 3,  { 1, 1, 11 }},
#define HOSTTRAPTRAPABNORMALFIBERTXERROR        12
    {HOSTTRAPTRAPABNORMALFIBERTXERROR,  ASN_INTEGER,  RWRITE,  var_hostTable, 3,  { 1, 1, 12 }},
#define HOSTADMIN       13
    {HOSTADMIN,  ASN_INTEGER,  RWRITE,  var_hostTable, 3,  { 1, 1, 13 }},
#define HOSTSTATUS      14
    {HOSTSTATUS,  ASN_INTEGER,  RWRITE,  var_hostTable, 3,  { 1, 1, 14 }},
};


/** Initializes the ssHostGroup module */
void
init_ssHostGroup(void)
{
    AGENT_ENTER;

    DEBUGMSGTL(("ssHostGroup", "Initializing\n"));

    /* Allocate memory for callback argument */
    sshostgrp_parms = (ngrmib_parameters_t *) \
                sal_alloc(sizeof(ngrmib_parameters_t), SS_HOST_GROUP_MIB);

    /* Set group name */
    if (sshostgrp_parms != NULL) {
        sshostgrp_parms->group = GROUP_HOST;

        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(SS_HOST_GROUP_MIB, ssHostGroup_variables, variable7,
                   ssHostGroup_variables_oid);
    }

    AGENT_EXIT;
}

/*
 * Top level MIB function. Nothing is actually done here.
 */
unsigned char *
var_ssHostGroup(struct variable *vp,
                oid     *name,
                size_t  *length,
                int     exact,
                size_t  *var_len,
                WriteMethod **write_method)
{
    AGENT_ENTER;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

  /*
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }

    /*  No OIDs in this, so AGENT_EXIT;
        return */
    AGENT_EXIT;
    return NULL;
}


static int
header_hostTable(struct variable *vp,
           oid *name,
           size_t *length,
           int exact,
           size_t *var_len,
           WriteMethod **write_method)
{
#define SS_HOST_ID_LEN     12
#define SS_HOST_TOTAL_LEN  13

    oid newname[MAX_OID_LEN];
    int i = 0, result, found = FALSE;
    int host_id, curr_stat;

    /* In case we are creating a new row, we need to set write method first */
    *write_method = write_hostEntry;

    if (creation_row) {
        AGENT_DBG(("SNMP Host under creation = %d", creation_row->hindex));
    } else {
        AGENT_DBG(("No host under creation"));
    }

    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));


    /* We loop through 4 indexes, and proceed if they are valid */
    for (i = 1; i <= SS_HOST_MAX_ALLOWED; i++ ) {
        /* Check if this group is active or valid */
        sshostgrp_parms->index = i;
        sshostgrp_parms->group_obj = SS_HOST_STATUS;
        sshostgrp_parms->callback_rw = CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                            SNMPD_CALLBACK_NETGEAR,
                            sshostgrp_parms);
        curr_stat = sshostgrp_parms->value;

        /* If row is not active, check if it is under creation */
        if (curr_stat != MIB_ROWSTATUS_ACTIVE) {
            if (!(creation_row && creation_row->hindex == i)) {
                continue;
            }
        } else { /* If row is active and creation_row is set, destroy it */
            if (creation_row && creation_row->hindex == i) {
                sal_free(creation_row);
                creation_row = NULL;
            }
        }

        newname[SS_HOST_ID_LEN] = (oid) i ;
        result = snmp_oid_compare(name, *length, newname, 
                                    (int) vp->namelen + 1);
        if ((exact && (result == 0)) || (!exact && (result < 0))) {
            found = TRUE;
            break;
        }
    }

    if (!found) {
        DEBUGMSGTL(("ssTVlan", "Invalid Index"));
        AGENT_EXIT;
        return MATCH_FAILED;
    }

    memcpy( (char *)name,(char *)newname, ((int) vp->namelen +1) * sizeof(oid));

    *length = vp->namelen + 1;
    *var_len = sizeof(long);    /* default to 'long' results */

    AGENT_EXIT;
    return MATCH_SUCCEEDED;
}

/*
 * This table is called whenever there is a creation or modification to be done
 * on the host table
 */
unsigned char *
var_hostTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    AGENT_ENTER;
    AGENT_OID_PRINT(*length, name);

    static long long_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    int loop;

    /* Validate */
    if (header_hostTable(vp,name,length,exact,var_len, write_method) ==
                                                            MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    sshostgrp_parms->index = name[SS_HOST_ID_LEN];
    switch(vp->magic) {
        case HOSTINDEX:
            if (creation_row && 
                            creation_row->hindex == sshostgrp_parms->index){
                sshostgrp_parms->value = creation_row->hindex;
            } else {
                sshostgrp_parms->group_obj = SS_HOST_INDEX;
            }
            break;
        case HOSTNAME:
            /* Not supported right now */
            return NULL;
#if 0
            if (creation_row && 
                            creation_row->hindex == sshostgrp_parms->index){
                strcpy (sshostgrp_parms->buf, creation_row->hname);
            } else {
                sshostgrp_parms->group_obj = SS_HOST_NAME;
            }
#endif
            break;
        case HOSTIP:
            if (creation_row && 
                            creation_row->hindex == sshostgrp_parms->index){
                for (loop = 0; loop < IP_ADDRESS_LEN; loop++) {
                    sshostgrp_parms->buf[loop] = creation_row->hip[loop];
                }
            } else {
                sshostgrp_parms->group_obj = SS_HOST_IP;
            }
            break;
        case HOSTCOMM:
            if (creation_row && 
                            creation_row->hindex == sshostgrp_parms->index){
                strcpy (sshostgrp_parms->buf, creation_row->hcomm);
            } else {
                sshostgrp_parms->group_obj = SS_HOST_COMM;
            }
            break;
        case HOSTATTRIBUTE:
            if (creation_row && 
                            creation_row->hindex == sshostgrp_parms->index){
                sshostgrp_parms->value = creation_row->hattrib;
            } else {
                sshostgrp_parms->group_obj = SS_HOST_ATTRIBUTE;
            }
            break;
        case HOSTTRAPSNMPBOOTUP:
            if (creation_row && 
                            creation_row->hindex == sshostgrp_parms->index){
                sshostgrp_parms->value = creation_row->htrapbootup;
            } else {
                sshostgrp_parms->group_obj = SS_HOST_TRAPSNMPBOOTUP;
            }
            break;
        case HOSTTRAPSNMPLINKUPDOWN:
            if (creation_row && 
                            creation_row->hindex == sshostgrp_parms->index){
                sshostgrp_parms->value = creation_row->htraplupd;
            } else {
                sshostgrp_parms->group_obj = SS_HOST_TRAPSNMPLINKUPDOWN;
            }
            break;
        case HOSTTRAPTRAPINSERTREMOVE:
            return NULL;
#if 0
            if (creation_row && 
                            creation_row->hindex == sshostgrp_parms->index){
                sshostgrp_parms->value = creation_row->htrapinsrem;
            } else {
                sshostgrp_parms->group_obj = SS_HOST_TRAPTRAPINSERTREMOVE;
            }
#endif
            break;
        case HOSTTRAPTRAPABNORMALTPRXERROR:
            return NULL;
#if 0
            if (creation_row && 
                            creation_row->hindex == sshostgrp_parms->index){
                sshostgrp_parms->value = creation_row->htrapabntprx;
            } else {
                sshostgrp_parms->group_obj = SS_HOST_TRAPTRAPABNORMALTPRXERROR;
            }
#endif
            break;
        case HOSTTRAPTRAPABNORMALTPTXERROR:
            return NULL;
#if 0
            if (creation_row && 
                            creation_row->hindex == sshostgrp_parms->index){
                sshostgrp_parms->value = creation_row->htrapabntptx;
            } else {
                sshostgrp_parms->group_obj = SS_HOST_TRAPTRAPABNORMALTPTXERROR;
            }
#endif
            break;
        case HOSTTRAPTRAPABNORMALFIBERRXERROR:
            return NULL;
#if 0
            if (creation_row && 
                            creation_row->hindex == sshostgrp_parms->index){
                sshostgrp_parms->value = creation_row->htrapabnfbrrx;
            } else {
                sshostgrp_parms->group_obj =
                                        SS_HOST_TRAPTRAPABNORMALFIBERRXERROR;
            }
#endif
            break;
        case HOSTTRAPTRAPABNORMALFIBERTXERROR:
            return NULL;
#if 0
            if (creation_row && 
                            creation_row->hindex == sshostgrp_parms->index){
                sshostgrp_parms->value = creation_row->htrapabnfbrtx;
            } else {
                sshostgrp_parms->group_obj = 
                                        SS_HOST_TRAPTRAPABNORMALFIBERTXERROR;
            }
#endif
            break;
        case HOSTADMIN:
            if (creation_row && 
                            creation_row->hindex == sshostgrp_parms->index){
                sshostgrp_parms->value = creation_row->hadmstat;
            } else {
                sshostgrp_parms->group_obj = SS_HOST_ADMIN;
            }
            break;
        case HOSTSTATUS:
            if (creation_row && 
                            creation_row->hindex == sshostgrp_parms->index){
                sshostgrp_parms->value = creation_row->hstatus;
            } else {
                sshostgrp_parms->group_obj = SS_HOST_STATUS;
            }
            break;
        default:
            AGENT_EXIT;
            return NULL;
    }

    /* Callback */
    if (!(creation_row && 
                    creation_row->hindex == sshostgrp_parms->index)){
        sshostgrp_parms->callback_rw = CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                            SNMPD_CALLBACK_NETGEAR,
                            sshostgrp_parms);
    }

    /* Give the value back */
    switch(vp->magic) {
        case HOSTNAME:
        case HOSTCOMM:
            strcpy (string, sshostgrp_parms->buf);
            *var_len = strlen(string);
            AGENT_EXIT;
            return (unsigned char *) string;

        case HOSTIP:
            *var_len = IP_ADDRESS_LEN;
            for (loop = 0; loop < *var_len; loop++) {
                string[loop] = sshostgrp_parms->buf[loop];
            }
            AGENT_EXIT;
            return (unsigned char *) string;
    }

    long_ret = (long)sshostgrp_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}

/*
 * Single write function for all host group members
 */
int
write_hostEntry(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    AGENT_ENTER;
    AGENT_OID_PRINT(name_len, name);
    long            value = 0;
    int             var, count;
    u_char          *cp;
    int hindex, curr_status;

    var = name[SS_HOST_ID_LEN-1];
    sshostgrp_parms->index = hindex = (int) name[SS_HOST_ID_LEN];
    sshostgrp_parms->group_obj = SS_HOST_STATUS;
    sshostgrp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_NETGEAR,
                        sshostgrp_parms);
    curr_status = sshostgrp_parms->value;
    if (var_val) {
        value = *(long *) var_val;
    }

    /* We do not support some of the variables */
    if (var == HOSTNAME || var == HOSTTRAPTRAPINSERTREMOVE || 
        var == HOSTTRAPTRAPABNORMALTPTXERROR || 
        var == HOSTTRAPTRAPABNORMALTPRXERROR || 
        var == HOSTTRAPTRAPABNORMALFIBERTXERROR || 
        var == HOSTTRAPTRAPABNORMALFIBERRXERROR) {
        return SNMP_ERR_NOTWRITABLE;
    }

    /* Do not activate if not createAndWait for non-existing entries */
    if (!(creation_row && hindex == creation_row->hindex) && 
            (curr_status != MIB_ROWSTATUS_ACTIVE && var != HOSTSTATUS && 
             value != MIB_ROWSTATUS_CREATEANDWAIT)) {
        return SNMP_ERR_WRONGVALUE;
    }

    /* Create an entry if createAndWait */
    if (creation_row == NULL && (var == HOSTSTATUS && 
                                       value == MIB_ROWSTATUS_CREATEANDWAIT)) {
        /*  Create if the host group does not already exist */
        if (hindex < 1 || hindex > SS_HOST_MAX_ALLOWED ) {
            return SNMP_ERR_WRONGVALUE;
        }

        if (curr_status == MIB_ROWSTATUS_ACTIVE) {
            AGENT_DBG(("Host group already exists"));
            return SNMP_ERR_WRONGVALUE;
        }

        creation_row = (ngr_host_entry_t *) 
                         sal_alloc(sizeof(ngr_host_entry_t), SS_HOST_GROUP_MIB);

        if (creation_row == NULL) {
            AGENT_DBG(("Unable to allocate memory"));
            return SNMP_ERR_RESOURCEUNAVAILABLE;
        }

        /* zero out all fields */
        memset(creation_row, 0, sizeof(ngr_host_entry_t));

        /* fill out some default values */
        creation_row->hindex = sshostgrp_parms->index;
        
        creation_row->hattrib = HOST_ATTRIB_RONLY;
        
        creation_row->htrapbootup = creation_row->htraplupd = 
             creation_row->htrapinsrem = creation_row->htrapabntprx = 
             creation_row->htrapabntptx = creation_row->htrapabnfbrrx =  
             creation_row->htrapabnfbrtx = creation_row->hadmstat = 
                                                        MIB_STATUS_DISABLED;

        creation_row->hstatus = MIB_ROWSTATUS_NOTREADY; /* Not ready */
    } else if (creation_row != NULL && (var == HOSTSTATUS && 
                                       value == MIB_ROWSTATUS_CREATEANDWAIT)) {
        /* We can have only one row under creation */
        if (hindex != creation_row->hindex) {
            AGENT_DBG(("We can have only one HOST under creation"));
            return SNMP_ERR_RESOURCEUNAVAILABLE;
        }
    }


    switch ( action ) {
        case RESERVE1:
            /* Global check */
            if (hindex < 1 || hindex > SS_HOST_MAX_ALLOWED) {
                fprintf(stderr, "write to ssHostGroup with wrong value\n");
                return SNMP_ERR_WRONGVALUE;
            }
            switch(var) {
                case HOSTNAME:
                case HOSTCOMM:
                    if (var_val_type != ASN_OCTET_STR) {
                      fprintf(stderr,  
                                "write to ssHostGroup not ASN_OCTET_STR\n");
                      AGENT_EXIT;
                      return SNMP_ERR_WRONGTYPE;
                    }

                    if (var_val_len > sizeof(char) * SS_HOST_MAX_STR_SIZE) {
                      fprintf(stderr,"write to ssHostGroup: bad length\n");
                      AGENT_EXIT;
                      return SNMP_ERR_WRONGLENGTH;
                    }
                    
                    for(cp = var_val, count = 0; count < (int)var_val_len; 
                                                             count++, cp++){
                        if (!isprint(*cp)){
                            AGENT_EXIT;
                            return SNMP_ERR_WRONGVALUE;
                        }
                    }
                    break;
                
                case HOSTIP:
                    if (var_val_type != ASN_IPADDRESS) {
                      fprintf(stderr, 
                              "write to ssHostGroup not ASN_IPADDRESS\n");
                      AGENT_EXIT;
                      return SNMP_ERR_WRONGTYPE;
                    }

                    if (var_val_len != sizeof(u_long)) {
                      fprintf(stderr,"write to ssHostGroup: bad length\n");
                      AGENT_EXIT;
                      return SNMP_ERR_WRONGLENGTH;
                    }
                    break;

                case HOSTATTRIBUTE:
                case HOSTTRAPSNMPBOOTUP:
                case HOSTTRAPSNMPLINKUPDOWN:
                case HOSTTRAPTRAPINSERTREMOVE:
                case HOSTTRAPTRAPABNORMALTPRXERROR:
                case HOSTTRAPTRAPABNORMALTPTXERROR:
                case HOSTTRAPTRAPABNORMALFIBERRXERROR:
                case HOSTTRAPTRAPABNORMALFIBERTXERROR:
                case HOSTADMIN:
                    if (var_val_type != ASN_INTEGER) {
                      fprintf(stderr, "write to ssHostGroup not ASN_INTEGER\n");
                      AGENT_EXIT;
                      return SNMP_ERR_WRONGTYPE;
                    }
                    if (var_val_len > sizeof(long)) {
                      fprintf(stderr,"write to ssHostGroup: bad length\n");
                      AGENT_EXIT;
                      return SNMP_ERR_WRONGLENGTH;
                    }
                    if (value < 1 || value > 2) {
                      fprintf(stderr,"write to ssHostGroup: bad value\n");
                      AGENT_EXIT;
                      return SNMP_ERR_WRONGVALUE;
                    }
                    break;

                case HOSTSTATUS:
                    if (var_val_type != ASN_INTEGER) {
                        fprintf(stderr, 
                                 "write to ssHostGroup not ASN_INTEGER\n");
                        return SNMP_ERR_WRONGTYPE;
                    }

                    if (var_val_len > sizeof(long)) {
                        fprintf(stderr, "write to ssHostGroup: bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }

                    if (value != MIB_ROWSTATUS_ACTIVE && 
                            value != MIB_ROWSTATUS_NOTREADY && 
                            value != MIB_ROWSTATUS_CREATEANDWAIT && 
                            value != MIB_ROWSTATUS_DESTROY) {
                        fprintf(stderr, "write to ssHostGroup: bad value\n");
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;
            }
        break;

        case RESERVE2:
        break;

        case FREE:
        break;

        case ACTION:
          break;

        case UNDO:
          break;

        case COMMIT:
            switch(var) {
                case HOSTNAME:
                    if ((creation_row == NULL) || 
                                          hindex != creation_row->hindex) {
                        sshostgrp_parms->group_obj = SS_HOST_NAME;
                        memcpy(sshostgrp_parms->buf, var_val, var_val_len);
                        sshostgrp_parms->value = var_val_len;
                    } else {
                        memset(creation_row->hname, 0, 
                                              sizeof(creation_row->hname));
                        memcpy (creation_row->hname, var_val, var_val_len);
                    }
                    break;

                case HOSTIP:
                    if ((creation_row == NULL) || 
                                           hindex != creation_row->hindex) {
                        memcpy(sshostgrp_parms->buf, var_val, var_val_len);
                        sshostgrp_parms->value = var_val_len;
                        sshostgrp_parms->group_obj = SS_HOST_IP;
                    } else {
                        memset(creation_row->hip, 0, 
                                             sizeof(creation_row->hip));
                        memcpy (creation_row->hip, var_val, var_val_len);
                    }
                    break;

                case HOSTCOMM:
                    if ((creation_row == NULL) || 
                                           hindex != creation_row->hindex) {
                        sshostgrp_parms->group_obj = SS_HOST_COMM;
                        memcpy(sshostgrp_parms->buf, var_val, var_val_len);
                        sshostgrp_parms->value = var_val_len;
                    } else {
                        AGENT_DBG(("Community Set"));
                        memset(creation_row->hcomm, 0, 
                                              sizeof(creation_row->hcomm));
                        memcpy (creation_row->hcomm, var_val, var_val_len);
                    }
                    break;

                case HOSTATTRIBUTE:
                    if ((creation_row == NULL) || 
                                           hindex != creation_row->hindex) {
                        sshostgrp_parms->group_obj = SS_HOST_ATTRIBUTE;
                        AGENT_DBG(("ATTRIB: Index %d value %d",
                            sshostgrp_parms->index, value));
                        sshostgrp_parms->value = value;
                    } else {
                        creation_row->hattrib = value;
                    }
                    break;

                case HOSTTRAPSNMPBOOTUP:
                    if ((creation_row == NULL) || 
                                           hindex != creation_row->hindex) {
                        sshostgrp_parms->group_obj = SS_HOST_TRAPSNMPBOOTUP;
                        AGENT_DBG(("Bootup trap: Index %d value %d",
                            sshostgrp_parms->index, value));
                        sshostgrp_parms->value = value;
                    } else {
                        creation_row->htrapbootup = value;
                    }
                    break;

                case HOSTTRAPSNMPLINKUPDOWN:
                    if ((creation_row == NULL) || 
                                              hindex != creation_row->hindex) {
                        sshostgrp_parms->group_obj = SS_HOST_TRAPSNMPLINKUPDOWN;
                        sshostgrp_parms->value  = value;
                    } else {
                        creation_row->htraplupd = value;
                    }
                    break;

                case HOSTTRAPTRAPINSERTREMOVE:
                    if ((creation_row == NULL) || 
                                             hindex != creation_row->hindex) {
                        sshostgrp_parms->group_obj = SS_HOST_TRAPTRAPINSERTREMOVE;
                        sshostgrp_parms->value    = value;
                    } else {
                        creation_row->htrapinsrem = value;
                    }
                    break;

                case HOSTTRAPTRAPABNORMALTPRXERROR:
                    if ((creation_row == NULL) || 
                                            hindex != creation_row->hindex) {
                        sshostgrp_parms->group_obj = SS_HOST_TRAPTRAPABNORMALTPRXERROR;
                        sshostgrp_parms->value     = value;
                    } else {
                        creation_row->htrapabntprx = value;
                    }
                    break;

                case HOSTTRAPTRAPABNORMALTPTXERROR:
                    if ((creation_row == NULL) || 
                                            hindex != creation_row->hindex) {
                        sshostgrp_parms->group_obj = SS_HOST_TRAPTRAPABNORMALTPTXERROR;
                        sshostgrp_parms->value     = value;
                    } else {
                        creation_row->htrapabntptx = value;
                    }
                    break;

                case HOSTTRAPTRAPABNORMALFIBERRXERROR:
                    if ((creation_row == NULL) || 
                                            hindex != creation_row->hindex) {
                        sshostgrp_parms->group_obj = SS_HOST_TRAPTRAPABNORMALFIBERRXERROR;
                        sshostgrp_parms->value     = value;
                    } else {
                        creation_row->htrapabnfbrrx = value;
                    }
                case HOSTTRAPTRAPABNORMALFIBERTXERROR:
                    if ((creation_row == NULL) || hindex != creation_row->hindex) {
                        sshostgrp_parms->group_obj = SS_HOST_TRAPTRAPABNORMALFIBERTXERROR;
                        sshostgrp_parms->value     = value;
                    } else {
                        creation_row->htrapabnfbrtx = value;
                    }
                    break;

                case HOSTADMIN:
                    if ((creation_row == NULL) || hindex != creation_row->hindex) {
                        sshostgrp_parms->group_obj = SS_HOST_ADMIN;
                        sshostgrp_parms->value     = value;
                    } else {
                        creation_row->hadmstat = value;
                    }
                    break;

                case HOSTSTATUS:
                    sshostgrp_parms->group_obj = SS_HOST_STATUS;
                    sshostgrp_parms->value = value;
                    if ((creation_row == NULL) || 
                                     hindex != creation_row->hindex) {
                        
                        if (value == MIB_ROWSTATUS_ACTIVE) {
                            return SNMP_ERR_NOERROR; /* Already active */
                        } else if (value == MIB_ROWSTATUS_NOTREADY || 
                                    value == MIB_ROWSTATUS_CREATEANDWAIT) {
                            /* Already active, so cant set not ready */
                            return SNMP_ERR_WRONGVALUE;
                        }
                    } else  if (creation_row) { /* row under creation */
                        if (value == MIB_ROWSTATUS_DESTROY) { /* Destroy */
                            sal_free (creation_row);
                            creation_row = NULL;
                            return SNMP_ERR_NOERROR;
                        } else if (value == MIB_ROWSTATUS_ACTIVE) { /* active */
                            if (creation_row->hip[0] == 0 || 
                                creation_row->hcomm[0] == 0) {
                                return SNMP_ERR_WRONGVALUE;
                            }
                            memcpy (&sshostgrp_parms->group_data.host,
                                    creation_row, 
                                    sizeof(ngr_host_entry_t));
                        }
                    }
                    break;
            }

            /* call backend if already existing row, or to activate row
               under creation */
            if ((creation_row == NULL || hindex != creation_row->hindex) ||
                          (sshostgrp_parms->index == creation_row->hindex && 
                          var == HOSTSTATUS && value == MIB_ROWSTATUS_ACTIVE)) {
                AGENT_DBG(("Calling write"));
                sshostgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
                snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                SNMPD_CALLBACK_NETGEAR,
                                sshostgrp_parms);
                if (sshostgrp_parms->ret_val == MIB_STATUS_ERROR) {
                     AGENT_EXIT;
                     return SNMP_ERR_COMMITFAILED;
                }
                /* If activating createAndWait, free mem */
                if (creation_row && hindex == creation_row->hindex) {
                    sal_free (creation_row);
                    creation_row = NULL;
                }
            }
        break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}
