/*
 * $Id: ssIgmpSnoopGroup.c,v 1.1 2006/12/11 06:25:14 ganesh Exp $
 */

/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include <ucd-snmp/mibgroup/ng700ss/netgear_mib.h>
#include "mibgroup/ng700ss/ssIgmpSnoopGroup.h"

/* callback argument */
static ngrmib_parameters_t *igmpsnoop_parms = NULL;

/* OID that defines where this MIb is placed WRT global MIB */
oid ssIgmpSnoopGroup_variables_oid[] = { 1,3,6,1,4,1,4526,11,23 };

/* Variables this MIB supports */
struct variable7 ssIgmpSnoopGroup_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define IGMPSNOOPINGCFG               1
    {IGMPSNOOPINGCFG,  ASN_INTEGER,  RWRITE,  var_ssIgmpSnoopGroup, 1,  { 1 }},
#define FILTERUNKNOWNMULTICAST        2
    {FILTERUNKNOWNMULTICAST,  ASN_INTEGER,  RWRITE,  var_ssIgmpSnoopGroup, 1,  { 2 }},
};

/** Initializes the ssIgmpSnoopGroup module */
void
init_ssIgmpSnoopGroup(void)
{
    AGENT_ENTER;

    /* Allocate memory for callback argument */
    igmpsnoop_parms = (ngrmib_parameters_t *) \
                sal_alloc(sizeof(ngrmib_parameters_t), SS_IGMPSNOOP_GROUP_MIB);

    /* Let us set the group here.*/
    if (igmpsnoop_parms != NULL) {
        igmpsnoop_parms->group = GROUP_IGMPSNOOP; 

        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(SS_IGMPSNOOP_GROUP_MIB, ssIgmpSnoopGroup_variables, variable7,
                   ssIgmpSnoopGroup_variables_oid);
    }
    
    AGENT_EXIT;
}

/*
 *   This function is called every time the agent gets a request for
 *   a scalar variable that might be found within the mib section
 */
unsigned char *
var_ssIgmpSnoopGroup(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    static long long_ret;
    int test_i;

    AGENT_ENTER;
    AGENT_OID_PRINT(*length, name);

    AGENT_DBG (("Validating Header"));
    /* validation */
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED) {
        AGENT_EXIT;
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
        case IGMPSNOOPINGCFG:
            *write_method = write_igmpSnoopingCfg;
            igmpsnoop_parms->group_obj = SS_IGMP_SNOOPINGCFG;
            AGENT_DBG (("Setting IGMPSNOOPINGCFG"));
            break;

        case FILTERUNKNOWNMULTICAST:
            *write_method = write_filterUnknownMulticast;
            igmpsnoop_parms->group_obj = SS_IGMP_FILTER_UNKNOWNMULTICAST;
            AGENT_DBG (("Setting FILTERUNKNOWNMULTICAST"));
            break;

        default:
            ERROR_MSG("Undefined OID in ssIgmpSnoopGroup");
            break;
    }

    igmpsnoop_parms->callback_rw = CALLBACK_ACT_READ;
    AGENT_DBG (("Calling Callback"));
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        igmpsnoop_parms);
    
    long_ret = (long)igmpsnoop_parms->value;
    AGENT_EXIT;
    return (unsigned char * ) &long_ret;
}

/*
 * Enable or disable IGMP Snooping on the 
 * switch here.
 */
int
write_igmpSnoopingCfg(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    int value = 0;
    AGENT_ENTER;
    AGENT_OID_PRINT(name_len, name);
    
    if (var_val) {
        value = * (int *) var_val;
    }
    igmpsnoop_parms->group_obj = SS_IGMP_SNOOPINGCFG;
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          if (value != MIB_STATUS_ENABLED && 
                value != MIB_STATUS_DISABLED) {
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case ACTION:
            break;
            
        case UNDO:
            /* Nothing to undo over here */
            break;

        case COMMIT:
            /* Let us get the current state of IGMP Snooping */
            igmpsnoop_parms->callback_rw = CALLBACK_ACT_READ;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                                SNMPD_CALLBACK_NETGEAR,
                                igmpsnoop_parms);
            if(value == igmpsnoop_parms->value) {
                AGENT_EXIT;
                return SNMP_ERR_NOERROR;
            }

            /* Finally, do commit */
            igmpsnoop_parms->value = value;
            igmpsnoop_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                            SNMPD_CALLBACK_NETGEAR,
                            igmpsnoop_parms);
            if (igmpsnoop_parms->ret_val == MIB_STATUS_ERROR) {
                 AGENT_EXIT;
                 return SNMP_ERR_COMMITFAILED;
            }
            break;

        case FREE:
            break;
    }
    
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}


int
write_filterUnknownMulticast(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    int value = 0;

    AGENT_ENTER;
    AGENT_OID_PRINT(name_len, name);
    if (var_val) {
        value = * (int *) var_val;
    }

    igmpsnoop_parms->group_obj = SS_IGMP_FILTER_UNKNOWNMULTICAST;
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          if (value != MIB_STATUS_ENABLED && value != MIB_STATUS_DISABLED) {
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case ACTION:
            break;
            
        case UNDO:
            /* Nothing to undo over here */
            break;

        case COMMIT:
            /* Let us get the current state of MCAST */
            igmpsnoop_parms->callback_rw = CALLBACK_ACT_READ;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                                SNMPD_CALLBACK_NETGEAR,
                                igmpsnoop_parms);
            if (igmpsnoop_parms->value == value) {
                AGENT_EXIT;
                return SNMP_ERR_NOERROR;
            }

            /* Finally, do commit */
            igmpsnoop_parms->value = value;
            igmpsnoop_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                            SNMPD_CALLBACK_NETGEAR,
                            igmpsnoop_parms);
            if (igmpsnoop_parms->ret_val == MIB_STATUS_ERROR) {
                 AGENT_EXIT;
                 return SNMP_ERR_COMMITFAILED;
            }
            break;
        case FREE:
            break;
    }
    
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

