/*
 * $Id: ssManagementVlan.c,v 1.1 2006/12/11 06:25:14 ganesh Exp $
 */
/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include <ucd-snmp/mibgroup/ng700ss/netgear_mib.h>
#include "mibgroup/ng700ss/ssManagementVlan.h"

/* Management VLAN callback argument */
static ngrmib_parameters_t *mvlan_parms = NULL;

/* OID for indexing into this MIB */
oid ssManagementVlan_variables_oid[] = { 1,3,6,1,4,1,4526,11,25 };

/* Variables that this MIB supports */
struct variable7 ssManagementVlan_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define MVLANID        1
    {MVLANID,  ASN_INTEGER,  RWRITE,  var_ssManagementVlan, 1,  { 1 }},
};


/* Initializes the ssManagementVlan module */
void
init_ssManagementVlan(void)
{
    AGENT_ENTER;

    /* Allocate memory for callback argument */
    mvlan_parms = (ngrmib_parameters_t *) \
                  sal_alloc(sizeof(ngrmib_parameters_t), SS_MGMTVLAN_GROUP_MIB);

    /* set group name */
    if (mvlan_parms != NULL) {
        mvlan_parms->group = GROUP_MGMT_VLAN; 

        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(SS_MGMTVLAN_GROUP_MIB, ssManagementVlan_variables, variable7,
                   ssManagementVlan_variables_oid);
    }
}

/*
 *   This function is called every time the agent gets a request for
 *   a scalar variable that might be found within our mib section
 *   registered above.  
 */
unsigned char *
var_ssManagementVlan(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    static long long_ret;
    AGENT_ENTER;
    AGENT_OID_PRINT(*length, name);
    
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
        case MVLANID:
            *write_method = write_mvlanId;
            mvlan_parms->group_obj = SS_M_VLANID;
        break;
        
        default:
            ERROR_MSG("Default in management VLAN MIB");
        break;
    }
    
    mvlan_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        mvlan_parms);

    long_ret = (long) mvlan_parms->value;
    AGENT_EXIT;
    return (unsigned char * ) &long_ret;
}


int
write_mvlanId(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    int value = 0;

    AGENT_ENTER;
    AGENT_OID_PRINT(name_len, name);

    if (snmp_mib_getVlanMode() == MIB_VLAN_MODE_PORTBASED) {
        fprintf (stderr, "Vlan mode is port based");
        AGENT_EXIT;
        return SNMP_ERR_NOTWRITABLE;
    }

    if (var_val) {
        value = * (int *) var_val;
    }

    mvlan_parms->group_obj = SS_M_VLANID;
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssManagementVlan not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssManagementVlan: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          if (value < 0 && value > 4094) {
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
            break;

        case ACTION:
            /* Let us get the current value first */
            mvlan_parms->callback_rw = CALLBACK_ACT_READ;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                                SNMPD_CALLBACK_NETGEAR,
                                mvlan_parms);
            if(value == mvlan_parms->value) {
                AGENT_EXIT;
                return SNMP_ERR_NOERROR;
            }
            break;

        case UNDO:
            /* Nothing to undo, as it is all or none */
            break;

        case COMMIT:       
            /* Finally, the commit part!! */
            mvlan_parms->value = value;
            mvlan_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                SNMPD_CALLBACK_NETGEAR,
                                mvlan_parms);
            if (mvlan_parms->ret_val == MIB_STATUS_ERROR) {
                 AGENT_EXIT;
                 return SNMP_ERR_COMMITFAILED;
            }
            break;

        case FREE:        /* Free any resources allocated */
            break;

    }
    
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

