/*
 * $Id: ssMiscGroup.c,v 1.2 2007/01/25 17:58:36 ganesh Exp $
 */

/* This should always be included first before anything else */
#include <config.h>
#include <ctype.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include <ucd-snmp/mibgroup/ng700ss/netgear_mib.h>
#include "mibgroup/ng700ss/ssMiscGroup.h"

/* callback parameters */
static ngrmib_parameters_t *ssmiscgrp_parms = NULL;

/*
 * Position (OID) of ssMiscGroup with respect to
 * the global MIB tree
 */
oid ssMiscGroup_variables_oid[] = { 1,3,6,1,4,1,4526,11,3 };

/*
 *   This variable defines function callbacks and type AGENT_EXIT;
        return information
 *   for the ssMiscGroup mib section
 */
struct variable4 ssMiscGroup_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define MISCRESET		1
{MISCRESET,  ASN_INTEGER,  RWRITE,  var_ssMiscGroup, 1,  { 2 }},
#define MISCSTATISTICSRESET	2	
{MISCSTATISTICSRESET,  ASN_INTEGER,  RWRITE,  var_ssMiscGroup, 1,  { 3 }},
};


/** Initializes the ssMiscGroup module */
void
init_ssMiscGroup(void)
{
    AGENT_ENTER;

    /* Allocate memory for callback argument */
    ssmiscgrp_parms = (ngrmib_parameters_t *) \
                sal_alloc(sizeof(ngrmib_parameters_t), SS_MISC_GROUP_MIB);

    /* Set the group */
    if(ssmiscgrp_parms != NULL) {
        ssmiscgrp_parms->group = GROUP_SSMISC;

        /* Register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(SS_MISC_GROUP_MIB, ssMiscGroup_variables, variable4,
                   ssMiscGroup_variables_oid);
    }
}


/*
 * This function will be called whenever there is
 * a get or set on ssMiscGroup MIB variable
 */
unsigned char *
var_ssMiscGroup(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    AGENT_ENTER;
    static long long_ret;

	/*
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	*/

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ){
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
    case MISCRESET:
        *write_method = write_miscReset;
	ssmiscgrp_parms->group_obj = SS_MISC_RESET;
        AGENT_DBG (("Setting MISC RESET"));
        break;
    case MISCSTATISTICSRESET:
        *write_method = write_miscStatisticsReset;
	ssmiscgrp_parms->group_obj = SS_MISC_STATS_RESET;
        AGENT_DBG (("Setting MISC STATS_RESET"));
        break;
    default:
      ERROR_MSG("");
    }

    ssmiscgrp_parms->callback_rw = CALLBACK_ACT_READ; /* Get */
    AGENT_DBG (("Calling Callback"));
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_NETGEAR,
                        ssmiscgrp_parms);
    long_ret=(long) ssmiscgrp_parms->value;
    AGENT_EXIT;
    return (u_char *)&long_ret;
}

int
write_miscReset(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    AGENT_ENTER;
    int value = 0;

    if (var_val) {
        value = *(int *)var_val;
    }
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssMiscGroup not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssMiscGroup: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          if (value != MIB_STATUS_ENABLED  && value != MIB_STATUS_DISABLED ) {
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
          break;

        case ACTION:
          break;

        case UNDO:
          break;

        case COMMIT:
             /* Now do the set */
	     ssmiscgrp_parms->value = value;
	     ssmiscgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
	     snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
	                         SNMPD_CALLBACK_NETGEAR,
				 ssmiscgrp_parms);
	     if (ssmiscgrp_parms->ret_val == MIB_STATUS_ERROR)
	     {
	         AGENT_EXIT;
		 return SNMP_ERR_COMMITFAILED;
	     }
          break;
    }
    return SNMP_ERR_NOERROR;
}


int
write_miscStatisticsReset(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    long value;
    int size;

    if (var_val) {
        value = * (int *) var_val;
    }

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssMiscGroup not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssMiscGroup: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          if (value != MIB_STATUS_ENABLED  && value != MIB_STATUS_DISABLED ) {
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;
        case RESERVE2:
          break;

        case FREE:
          break;

        case ACTION:
          break;

        case UNDO:
          break;

        case COMMIT:
             /* Now do the set */
	     ssmiscgrp_parms->value = value;
	     ssmiscgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
	     snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
	                         SNMPD_CALLBACK_NETGEAR,
				 ssmiscgrp_parms);
	     if (ssmiscgrp_parms->ret_val == MIB_STATUS_ERROR)
	     {
	         AGENT_EXIT;
		 return SNMP_ERR_COMMITFAILED;
	     }
          break;
    }
    return SNMP_ERR_NOERROR;
}

