/*
 * $Id: ssPVlanGroup.c,v 1.1 2006/12/11 06:25:14 ganesh Exp $
 */

/* This should always be included first before anything else */
#include <config.h>
#include <ctype.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include <ucd-snmp/mibgroup/ng700ss/netgear_mib.h>
#include "mibgroup/ng700ss/ssPVlanGroup.h"

#define C_PVLAN_NAME_LEN    8

/* callback argument */
static ngrmib_parameters_t *pvlan_parms = NULL;

/* OID specifying the path to this MIB WRT the global tree */
oid ssPVlanGroup_variables_oid[] = { 1,3,6,1,4,1,4526,11,12 };

/* Row under creation */
static ngr_pvlan_entry_t        *creation_row = NULL;

/* Variables that this MIB supports */
struct variable7 ssPVlanGroup_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define PVLANID        1
    {PVLANID,  ASN_INTEGER,  RONLY,   var_pvlanTable, 3,  { 1, 1, 1 }},
#define PVLANNAME        2
    {PVLANNAME,  ASN_OCTET_STR,  RWRITE,  var_pvlanTable, 3,  { 1, 1, 2 }},
#define PVLANMEMBER        3
    {PVLANMEMBER,  ASN_OCTET_STR,  RWRITE,  var_pvlanTable, 3,  { 1, 1, 3 }},
#define PVLANSTATUS       5 
    {PVLANSTATUS,  ASN_INTEGER,  RWRITE,  var_pvlanTable, 3,  { 1, 1, 5 }},
};

/** Initializes the ssPVlanGroup module */
void
init_ssPVlanGroup(void)
{
    AGENT_ENTER;

    /* place any other initialization junk you need here */
     pvlan_parms = (ngrmib_parameters_t*) \
                sal_alloc(sizeof(ngrmib_parameters_t), SS_PVLAN_GROUP_MIB);

    /* set group name */
    if (pvlan_parms != NULL) {
        pvlan_parms->group = GROUP_PVLAN; 

        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(SS_PVLAN_GROUP_MIB, ssPVlanGroup_variables, variable7,
                   ssPVlanGroup_variables_oid);
    }
}

/*
 * Top level MIB entry function. Since access is made on the MIB tables,
 * nothing will be done here.
 */
unsigned char *
var_ssPVlanGroup(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    AGENT_ENTER;
    
    /* Validate */
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ){
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
        default:
          ERROR_MSG("Invalid OID");
          break;
    }
    AGENT_EXIT;
    return NULL;
}


static int
header_pvlanTable(struct variable *vp,
           oid *name,
           size_t *length,
           int exact,
           size_t *var_len,
           WriteMethod **write_method)
{
#define SS_PVLAN_ID_LEN     12
#define SS_PVLAN_TOTAL_LEN  13

    oid newname[MAX_OID_LEN];
    int i = 0, result, found = FALSE;
    int vlan_id, vlan_count; 

    /* In case we are creating a new row, we need to set write method first */
    *write_method = write_pvlanEntry;

    /* Get count of number of VLANs on the switch */
    pvlan_parms->group_obj = SS_PVLAN_COUNT;
    pvlan_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_NETGEAR,
                        pvlan_parms);
    vlan_count = pvlan_parms->value;
    AGENT_DBG(("PVLAN Count %d", vlan_count));
    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    if (creation_row) {
        /* Check if VLAN(s) has(ve) been deleted */
        if (vlan_count < creation_row->pvid - 1) {
            creation_row->pvid = vlan_count + 1;
            vlan_count++;
        }else if (vlan_count >= creation_row->pvid) { /* Added? */
            sal_free (creation_row);
            creation_row = NULL;
        } else {
            /* increment count, as this will be last in sorted list */
            vlan_count++;
        }
    } else {
        AGENT_DBG(("No vlan under creation"));
    }


    /* Loop through all the indexes, plus one under creation */
    for (; i < vlan_count; i++ ) {
        if (creation_row && i == vlan_count - 1) { 
            vlan_id = creation_row->pvid;
        } else {
            /* For PVLAN, at index 'n' you have VID 'n+1' */
            vlan_id = (i + 1);
        }
                
        AGENT_DBG(("VLAN ID found = %d", vlan_id));

        if (vlan_id < 1 || vlan_id > interface_count) {
            AGENT_DBG(("Invalid VLAN ID"));
            continue;
        }

        newname[SS_PVLAN_ID_LEN] = (oid) vlan_id ;
        result = snmp_oid_compare(name, *length, newname, 
                                    (int) vp->namelen + 1);
        if ((exact && (result == 0)) || (!exact && (result < 0))) {
            found = TRUE;
            AGENT_DBG(("Found entry at %d", i));
            break;
        }
    }

    if (!found) {
        DEBUGMSGTL(("ssTVlan", "Invalid Index"));
        AGENT_EXIT;
        return MATCH_FAILED;
    }

    memcpy( (char *)name,(char *)newname, ((int) vp->namelen +1) * sizeof(oid));
    *length = vp->namelen + 1;
    *var_len = sizeof(long);    /* default to 'long' results */

    AGENT_EXIT;
    return MATCH_SUCCEEDED;
}

/*
 * This function will be used for doing read/write on all the 
 * PVLAN MIB variables. However, it will not create a new row
 * as none of the variables are read-create.
 */
unsigned char *
var_pvlanTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    static long long_ret;
    static unsigned char string[SPRINT_MAX_LEN];

    AGENT_ENTER;

    /* return if not port VLAN */
    if (snmp_mib_getVlanMode() != MIB_VLAN_MODE_PORTBASED) {
        AGENT_EXIT;
        return NULL;
    }

    /* Validate */
    snmp_mib_setInterfaceCount();
    if (header_pvlanTable(vp,name,length,exact,var_len,
                                write_method) == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    AGENT_OID_PRINT(*length, name);
    pvlan_parms->index = (int)name[SS_PVLAN_ID_LEN];
    AGENT_DBG(("Index for params is %d", pvlan_parms->index));
    switch(vp->magic) {
        case PVLANID:
            long_ret = pvlan_parms->index;
            return (unsigned char *) &long_ret;

        case PVLANNAME:
            if (creation_row && pvlan_parms->index == creation_row->pvid) {
                strcpy (string, creation_row->pvname);
                *var_len = strlen(string);
                return (unsigned char *) string;
            }
            pvlan_parms->group_obj = SS_PVLAN_NAME;
            break;

        case PVLANMEMBER:
            if (creation_row && pvlan_parms->index == creation_row->pvid) {
                memcpy (string, creation_row->pvports, 
                                        (interface_count/INTERFACE_BYTE_COUNT));
                *var_len = (interface_count/INTERFACE_BYTE_COUNT);
                return (unsigned char *) string;
            }
            pvlan_parms->group_obj = SS_PVLAN_MEMBER;
            break;
        case PVLANSTATUS:
            if (creation_row && pvlan_parms->index == creation_row->pvid) {
                long_ret = creation_row->pvstatus;
                return (unsigned char *) &long_ret;
            }
            pvlan_parms->group_obj = SS_PVLAN_STATUS;
            break;

        default:
            ERROR_MSG("Undefined reference in PVLAN MIB");
    }

    pvlan_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_NETGEAR,
                        pvlan_parms);

    switch(vp->magic) {
        case PVLANNAME:
            *var_len = pvlan_parms->value;
            strcpy (string, pvlan_parms->buf);
            AGENT_EXIT;
            return (unsigned char *)string;

        case PVLANMEMBER:
            *var_len = (interface_count/INTERFACE_BYTE_COUNT);
            memcpy (string, pvlan_parms->buf, *var_len);
            AGENT_EXIT;
            return (unsigned char *)string;
    }

    long_ret = (long) pvlan_parms->value;
    AGENT_EXIT;
    return (unsigned char *) &long_ret;
}

/*
 * Write/Create function for tagged VLAN
 */
int
write_pvlanEntry(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    AGENT_ENTER;
    AGENT_OID_PRINT(name_len, name);
    long            value = 0;
    int             var, count;
    u_char          *cp;
    int             vid_count;

    var = name[SS_PVLAN_ID_LEN-1];
    pvlan_parms->index  = (int) name[SS_PVLAN_ID_LEN];
    AGENT_DBG(("PVLAN: Var being set = %d", var));

    if (var_val) {
        value = *(long *) var_val;
    }
 
    AGENT_DBG(("Value being set %d", value));
    pvlan_parms->group_obj = SS_PVLAN_COUNT;
    pvlan_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_NETGEAR,
                        pvlan_parms);
    vid_count = pvlan_parms->value;
    AGENT_DBG(("VID Count and index %d, %d", vid_count, 
                                            pvlan_parms->index));

    /* Create an entry if createAndWait */
    if (creation_row == NULL && 
                (var == PVLANSTATUS && value == MIB_ROWSTATUS_CREATEANDWAIT)) {
        AGENT_DBG(("Creating VLAN"));
        /* Creation VLAN must not exist, and count must be one less than  
           VLAN being created */
        if (vid_count >= pvlan_parms->index || 
                          (pvlan_parms->index > (vid_count + 1))) {
            AGENT_DBG(("VLAN already exists"));
            return SNMP_ERR_WRONGVALUE;
        }

        creation_row = (ngr_pvlan_entry_t *) 
                       sal_alloc(sizeof(ngr_pvlan_entry_t), SS_PVLAN_GROUP_MIB);

        if (creation_row == NULL) {
            AGENT_DBG(("Unable to allocate memory"));
            return SNMP_ERR_RESOURCEUNAVAILABLE;
        }

        /* zero out all fields */
        memset(creation_row, 0, sizeof(ngr_pvlan_entry_t));

        /* Get the vlan id */
        creation_row->pvid = pvlan_parms->index;

        /* Set the row status to notReady */
        creation_row->pvstatus = MIB_ROWSTATUS_NOTREADY;
    } else if (creation_row != NULL && 
                (var == PVLANSTATUS && value == MIB_ROWSTATUS_CREATEANDWAIT)) {
        /* We can have only one row under creation */
        if (pvlan_parms->index != creation_row->pvid) {
            AGENT_DBG(("We can have only one VLAN under creation"));
            return SNMP_ERR_RESOURCEUNAVAILABLE;
        }
    }
    
    switch ( action ) {
        case RESERVE1:
            switch(var) {
                case PVLANNAME:
                    if (var_val_type != ASN_OCTET_STR) {
                        fprintf(stderr, 
                                "write to TVlan not ASN_OCTET_STR\n");
                        return SNMP_ERR_WRONGTYPE;
                    }
                    if (var_val_len > sizeof(char) * C_PVLAN_NAME_LEN) {
                        fprintf(stderr, "write to TVlan: bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }
                    for(cp = var_val, count = 0; count < (int)var_val_len; 
                                                             count++, cp++){
                        if (!isprint(*cp)){
                            AGENT_EXIT;
                            return SNMP_ERR_WRONGVALUE;
                        }
                    }
                    break;

                case PVLANMEMBER:
                    if (var_val_type != ASN_OCTET_STR) {
                        fprintf(stderr, "write to TVlan not ASN_OCTET_STR\n");
                        return SNMP_ERR_WRONGTYPE;
                    }

                    if (var_val_len > 
                        (sizeof(char)*(interface_count/INTERFACE_BYTE_COUNT))) {
                        fprintf(stderr, "write to TVlan: bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }
                    break;

                case PVLANSTATUS:
                    if (var_val_type != ASN_INTEGER) {
                        fprintf(stderr, "write to TVlan not ASN_INTEGER\n");
                        return SNMP_ERR_WRONGTYPE;
                    }

                    if (var_val_len > sizeof(long)) {
                        fprintf(stderr, "write to TVlan: bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }

                    if (value != MIB_ROWSTATUS_ACTIVE && 
                            value != MIB_ROWSTATUS_NOTREADY && 
                            value != MIB_ROWSTATUS_CREATEANDWAIT && 
                            value != MIB_ROWSTATUS_DESTROY) {
                        fprintf(stderr, "write to TVlan: bad value\n");
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;
            }
        break;

        case RESERVE2:
        break;

        case FREE:
        break;

        case ACTION:
          switch (var) {
              case PVLANSTATUS:
                  pvlan_parms->group_obj = SS_PVLAN_STATUS;
                  pvlan_parms->callback_rw = CALLBACK_ACT_READ;
                  snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                      SNMPD_CALLBACK_NETGEAR,
                                      pvlan_parms);
                  if ((creation_row == NULL) || 
                              pvlan_parms->index != creation_row->pvid) {
                      /* cannot activate or destroy something  never created */
                      if (value != MIB_ROWSTATUS_CREATEANDWAIT && 
                                pvlan_parms->value != MIB_ROWSTATUS_ACTIVE) {
                          return SNMP_ERR_WRONGVALUE;
                      }
                   } else if (creation_row && 
                                  pvlan_parms->index == creation_row->pvid){
                        if (pvlan_parms->value == 1) {
                            sal_free(creation_row);
                            creation_row = NULL;
                            return SNMP_ERR_WRONGVALUE;
                        }
                    }
                    break;
                }
                break;

        case UNDO:
          break;

        case COMMIT:
            switch(var) {
                case PVLANNAME:
                    if ((creation_row == NULL) || 
                                pvlan_parms->index != creation_row->pvid) {
                        pvlan_parms->group_obj = SS_PVLAN_NAME;
                        memcpy(pvlan_parms->buf, var_val, var_val_len);
                        pvlan_parms->value = var_val_len;
                    } else if (creation_row) {
                        memset(creation_row->pvname, 0, 
                                             sizeof(creation_row->pvname));
                        memcpy(creation_row->pvname, var_val, var_val_len);
                    }
                    break;

                case PVLANMEMBER:
                    memset (pvlan_parms->buf, 0, sizeof(pvlan_parms->buf));
                    if (!memcmp(pvlan_parms->buf, var_val, var_val_len)) {
                        AGENT_EXIT;
                        return SNMP_ERR_WRONGVALUE;
                    }
                    
                    if ((creation_row == NULL) || 
                                pvlan_parms->index != creation_row->pvid) {
                        pvlan_parms->group_obj = SS_PVLAN_MEMBER;
                        memcpy(pvlan_parms->buf, var_val, var_val_len);
                        pvlan_parms->value = var_val_len;
                    } else  if (creation_row) {
                        memset(creation_row->pvports, 0, 
                                             sizeof(creation_row->pvports));
                        memcpy(creation_row->pvports, var_val, var_val_len);
                    }
                    break;

                case PVLANSTATUS:
                    pvlan_parms->group_obj = SS_PVLAN_STATUS;
                    pvlan_parms->value = value;
                    if ((creation_row == NULL) || 
                                pvlan_parms->index != creation_row->pvid) {
                        if (value == MIB_ROWSTATUS_ACTIVE) {
                            return SNMP_ERR_NOERROR; /* Already active */
                        } else if (value == MIB_ROWSTATUS_NOTREADY || 
                                   value == MIB_ROWSTATUS_CREATEANDWAIT || 
                                   (value == MIB_ROWSTATUS_DESTROY && 
                                    pvlan_parms->index == 1)) {
                            /* Already active, so cant set not ready 
                             * Or, Cannot destroy default vlan*/
                            return SNMP_ERR_WRONGVALUE;
                        }
                    } else  if (creation_row) { /* row under creation */
                        if (value == MIB_ROWSTATUS_DESTROY) { /* Destroy */
                            sal_free (creation_row);
                            creation_row = NULL;
                            return SNMP_ERR_NOERROR;
                        } else if (value == MIB_ROWSTATUS_ACTIVE) { /* active */
                            memset (pvlan_parms->buf, 0, sizeof(pvlan_parms->buf));
                            if (!memcmp(creation_row->pvports, pvlan_parms->buf, 
                                       interface_count/INTERFACE_BYTE_COUNT)) {
                                AGENT_EXIT;
                                return SNMP_ERR_COMMITFAILED;
                            }
                            memcpy (&pvlan_parms->group_data.pvlan,
                                    creation_row, 
                                    sizeof(ngr_pvlan_entry_t));
                        }
                    }
                    break;
            }

            /* call backend if already existing row, or to activate row
               under creation */
            if ((creation_row == NULL || 
                          pvlan_parms->index != creation_row->pvid) ||
                (pvlan_parms->index == creation_row->pvid && 
                                      var == PVLANSTATUS && value == 1)) {
                    pvlan_parms->callback_rw = CALLBACK_ACT_WRITE;
                    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                    SNMPD_CALLBACK_NETGEAR,
                                    pvlan_parms);
                    if (pvlan_parms->ret_val == MIB_STATUS_ERROR) {
                         AGENT_EXIT;
                         return SNMP_ERR_COMMITFAILED;
                    }
                    /* If activating createAndWait, free mem */
                    if (creation_row && 
                                 pvlan_parms->index == creation_row->pvid) {
                        sal_free (creation_row);
                        creation_row = NULL;
                    }
            }
        break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

void free_pvlan_createAndWaitMemory(void) {
    if (creation_row) {
        sal_free(creation_row);
        creation_row = NULL;
    }

    return;
}
