/*
 * $Id: ssPortTrunkGroup.c,v 1.3 2007/02/12 05:36:46 ganesh Exp $
 */

/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include <ucd-snmp/mibgroup/ng700ss/netgear_mib.h>
#include "mibgroup/ng700ss/ssPortTrunkGroup.h"

/* callback parameters */
static ngrmib_parameters_t *sstrunkgrp_parms = NULL;

/* Path to this MIB WRT the global MIB tree */
oid ssPortTrunkGroup_variables_oid[] = { 1,3,6,1,4,1,4526,11,14 };

/* Maximum number of trunks possible */
static int max_trunk = 0;

/* Variables that this MIB supports */
struct variable7 ssPortTrunkGroup_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define PORTTRUNKID        1
    {PORTTRUNKID,  ASN_INTEGER,  RONLY,   var_portTrunkTable, 3,  { 1, 1, 1 }},
#define PORTTRUNKMEMBER        2
    {PORTTRUNKMEMBER,  ASN_OCTET_STR,  RONLY,   var_portTrunkTable, 3,  { 1, 1, 2 }},
#define PORTTRUNKMEMBERVALUE        3
    {PORTTRUNKMEMBERVALUE,  ASN_OCTET_STR,  RWRITE,  var_portTrunkTable, 3,  { 1, 1, 3 }},
#define PORTTRUNKENABLE        4
    {PORTTRUNKENABLE,  ASN_INTEGER,  RWRITE,  var_portTrunkTable, 3,  { 1, 1, 4 }},
};

/** Initializes the ssPortTrunkGroup module */
void
init_ssPortTrunkGroup(void)
{
    AGENT_ENTER;

    /* Allocate memory for callback argument */
    sstrunkgrp_parms = (ngrmib_parameters_t *) \
                sal_alloc(sizeof(ngrmib_parameters_t), SS_TRUNK_GROUP_MIB);

    /* set group name */
    if (sstrunkgrp_parms != NULL) {
        sstrunkgrp_parms->group = GROUP_TRUNK; 

        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(SS_TRUNK_GROUP_MIB, ssPortTrunkGroup_variables, variable7,
                   ssPortTrunkGroup_variables_oid);
    }
}

/*
 * var_ssPortTrunkGroup(): This function is called every time the agent 
 * gets a request for a scalar variable that might be found within our mib 
 * section.
 */
unsigned char *
var_ssPortTrunkGroup(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* Validate */
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
        default:
          ERROR_MSG("");
    }

    AGENT_EXIT;
    return NULL;
}

/*
 * TVLAN validation function
 */
static int
header_trunkTable(struct variable *vp,
           oid *name,
           size_t *length,
           int exact,
           size_t *var_len,
           WriteMethod **write_method)
{
#define SS_TRUNK_NAME_LEN     12
#define SS_TRUNK_TOTAL_LEN    (13+(interface_count/INTERFACE_BYTE_COUNT))
    oid newname[MAX_OID_LEN];
    int i, result, found = FALSE, j;

    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    if (!max_trunk) {
        sstrunkgrp_parms->group_obj = SS_PORT_TRUNK_MAX;
        sstrunkgrp_parms->callback_rw = CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                            SNMPD_CALLBACK_NETGEAR,
                            sstrunkgrp_parms);
        max_trunk = (int)sstrunkgrp_parms->value;
        AGENT_DBG(("Numbe of trunks = %d", max_trunk));
    }

    /* Loop thru the trunk groups and check which is under creation */
    sstrunkgrp_parms->group_obj = SS_PORT_TRUNK_MEMBER;
    for (i = 1; i <= max_trunk; i++ ) {
        newname[SS_TRUNK_NAME_LEN] = (oid) i ;

        /* get the ports that can be member of this trunk*/
        sstrunkgrp_parms->index = i;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                            SNMPD_CALLBACK_NETGEAR,
                            sstrunkgrp_parms);
        for (j=0; j < (interface_count/INTERFACE_BYTE_COUNT); j++) {
            newname[SS_TRUNK_NAME_LEN+j+1] = (oid)sstrunkgrp_parms->buf[j];
        }
        
        result = snmp_oid_compare(name, *length, newname,SS_TRUNK_TOTAL_LEN);
        if ((exact && (result == 0)) || (!exact && (result < 0))) {
            found = TRUE;
            break;
        }
    }

    if (!found) {
        DEBUGMSGTL(("trunk", "... index out of range\n"));
        AGENT_EXIT;
        return MATCH_FAILED;
    }
    
    memcpy( (char *)name,(char *)newname, SS_TRUNK_TOTAL_LEN * sizeof(oid));
    *length = SS_TRUNK_TOTAL_LEN;
    *var_len = sizeof(long);    /* default to 'long' results */

    AGENT_EXIT;
    return MATCH_SUCCEEDED;
}

/*
 * var_portTrunkTable():This function is called whenever there is a 
 * request on the registered OIDs of this group.
 */
unsigned char *
var_portTrunkTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static unsigned char string[SPRINT_MAX_LEN];

    snmp_mib_setInterfaceCount();
    if (header_trunkTable(vp,name,length,exact,var_len,write_method) 
                                                    == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    sstrunkgrp_parms->index = (int) name[SS_TRUNK_NAME_LEN];
    switch(vp->magic) {
        case PORTTRUNKID:
             sstrunkgrp_parms->group_obj = SS_PORT_TRUNK_ID;
             break;
        case PORTTRUNKMEMBER:
             sstrunkgrp_parms->group_obj = SS_PORT_TRUNK_MEMBER;
             break;
        case PORTTRUNKMEMBERVALUE:
            *write_method = write_portTrunkMemberValue;
             sstrunkgrp_parms->group_obj = SS_PORT_TRUNK_MEMBERVALUE;
             break;
        case PORTTRUNKENABLE:
            *write_method = write_portTrunkEnable;
             sstrunkgrp_parms->group_obj = SS_PORT_TRUNK_ENABLE;
             break;
        default:
            ERROR_MSG("Undefined MIB reference in trunk table");
    }

    /* Callback */
    sstrunkgrp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_NETGEAR,
                        sstrunkgrp_parms);

    /* Give the value back */
    switch(vp->magic) {
        case PORTTRUNKMEMBER:
        case PORTTRUNKMEMBERVALUE:
            *var_len = (interface_count/INTERFACE_BYTE_COUNT);
            memcpy(string, sstrunkgrp_parms->buf, *var_len);
            AGENT_EXIT;
            return (unsigned char *) string;
    }

    long_ret = (long)sstrunkgrp_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}

static bool verify_port_bmp (unsigned char *bmp, 
                             unsigned char *port)
{
    int i=0;
    int start, end;
    
    start = snmp_get_bit_position_set(bmp, 0, interface_count);
    i = start;
    while (i != -1) {
        end = i;
        i++;
        i = snmp_get_bit_position_set(bmp, i, interface_count);
    }
    AGENT_DBG(("Trunk port: Start: %d , End: %d", start, end));
    i = snmp_get_bit_position_set(port, 0, interface_count);
    while (i != -1) {
        if ((i < start) || (i > end)) {
            AGENT_EXIT;
            return FALSE;
        }
        i++;
        i = snmp_get_bit_position_set(port, i, interface_count);
        AGENT_DBG(("Trunk port: Port %d is set\n", i));
    }
    AGENT_EXIT;
    return TRUE;
}

/*
 * Adds or removes a port from the trunk group.
 */
int
write_portTrunkMemberValue(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    char value;
    int size;
    int trunk_id;

    trunk_id  =  name[SS_TRUNK_NAME_LEN];
    sstrunkgrp_parms->index = trunk_id;
    sstrunkgrp_parms->group_obj = SS_PORT_TRUNK_MEMBER;
    sstrunkgrp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                           SNMPD_CALLBACK_NETGEAR,
                                 sstrunkgrp_parms);
    AGENT_DBG(("Trunk member for trunk %d is %x:%x:%x:%x:%x:%x\n",
                trunk_id,
                sstrunkgrp_parms->buf[0],
                sstrunkgrp_parms->buf[1],
                sstrunkgrp_parms->buf[2],
                sstrunkgrp_parms->buf[3],
                sstrunkgrp_parms->buf[4],
                sstrunkgrp_parms->buf[5]));
    if (!verify_port_bmp(sstrunkgrp_parms->buf, var_val)) {
        fprintf(stderr,"write to ssPortTrunkGroup: Bad value\n");
        AGENT_EXIT;
        return SNMP_ERR_WRONGVALUE;
    }
    
    
    sstrunkgrp_parms->group_obj = SS_PORT_TRUNK_MEMBERVALUE;
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to ssPortTrunkGroup not ASN_OCTET_STR\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char) * (interface_count/INTERFACE_BYTE_COUNT)) {
              fprintf(stderr,"write to ssPortTrunkGroup: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
            /* Nothing to do here, as we need to call backend even 
               if the trunk group is disabled */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
            /* Finally, the commit!! */
            memset (sstrunkgrp_parms->buf, 0, sizeof(sstrunkgrp_parms->buf));
            memcpy (sstrunkgrp_parms->buf,  var_val, var_val_len);
            sstrunkgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                            SNMPD_CALLBACK_NETGEAR,
                            sstrunkgrp_parms);
            if (sstrunkgrp_parms->ret_val != MIB_STATUS_SUCCESS) {
                 AGENT_EXIT;
                 return SNMP_ERR_COMMITFAILED;
            }
          break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

/*
 * Enables or disables a trunk group
 */
int
write_portTrunkEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    int size;
    int trunk_id;
    int value = 0;

    trunk_id  =  name[SS_TRUNK_NAME_LEN];
    sstrunkgrp_parms->index = trunk_id;
    sstrunkgrp_parms->group_obj = SS_PORT_TRUNK_ENABLE;
    if (var_val) {
        value = * (int *) var_val;
    }

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssPortTrunkGroup not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssPortTrunkGroup: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }

          if (value < MIB_STATUS_ENABLED || value > MIB_STATUS_DISABLED)  {
              fprintf(stderr,"write to ssPortTrunkGroup: bad value\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
            /* Let us do an enable or disable */
            sstrunkgrp_parms->value = value;
            sstrunkgrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                            SNMPD_CALLBACK_NETGEAR,
                            sstrunkgrp_parms);
            if (sstrunkgrp_parms->ret_val != MIB_STATUS_SUCCESS) {
                 AGENT_EXIT;
                 return SNMP_ERR_COMMITFAILED;
            }
          break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}
