/*
 * $Id: ssQosGroup.c,v 1.2 2007/01/25 17:58:36 ganesh Exp $
 */

/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include <ucd-snmp/mibgroup/ng700ss/netgear_mib.h>
#include "mibgroup/ng700ss/ssQosGroup.h"

#define MAX_IP_PRIORITY 8
#define MAX_DSCP_VALUES 64

/* Argument that will be passed to the callback function */
static ngrmib_parameters_t *qos_parms = NULL;

/* OID that specifies that path to this MIB */
oid ssQosGroup_variables_oid[] = { 1,3,6,1,4,1,4526,11,16 };

/* Variables that this MIB Supports */
struct variable7 ssQosGroup_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define QOSBASEMODE        1
    {QOSBASEMODE,  ASN_INTEGER,  RWRITE,  var_ssQosGroup, 1,  { 1 }},
#define QOSPORTIDVALUE        2
    {QOSPORTIDVALUE,  ASN_INTEGER,  RONLY,   var_qosPortPriorityTable, 3,  { 2, 1, 1 }},
#define QOSPORTPRIORITYQUEUE        3
    {QOSPORTPRIORITYQUEUE,  ASN_INTEGER,  RWRITE,  var_qosPortPriorityTable, 3,  { 2, 1, 2 }},
#define QOS1PCLASSVALUE        4
    {QOS1PCLASSVALUE,  ASN_INTEGER,  RONLY,   var_qos1pPriorityTable, 3,  { 3, 1, 1 }},
#define QOS1PPRIORITYQUEUE        5
    {QOS1PPRIORITYQUEUE,  ASN_INTEGER,  RWRITE,  var_qos1pPriorityTable, 3,  { 3, 1, 2 }},
#define PRECEDANCEVALUE        6
    {PRECEDANCEVALUE,  ASN_INTEGER,  RONLY,   var_tos2QueueTable, 4,  { 4, 1, 1, 1 }},
#define TOSPRIORITYQUEUE        7
    {TOSPRIORITYQUEUE,  ASN_INTEGER,  RWRITE,  var_tos2QueueTable, 4,  { 4, 1, 1, 2 }},
#define DSCPVALUE        8
    {DSCPVALUE,  ASN_INTEGER,  RONLY,   var_dscp2QueueTable, 4,  { 5, 1, 1, 1 }},
#define DSCPPRIORITYQUEUE        9
    {DSCPPRIORITYQUEUE,  ASN_INTEGER,  RWRITE,  var_dscp2QueueTable, 4,  { 5,1, 1, 2 }},
};

/* Initializes the ssQosGroup module */
void
init_ssQosGroup(void)
{
    AGENT_ENTER;

    /* Allocate memory for callback argument */
    qos_parms = (ngrmib_parameters_t *) \
                sal_alloc(sizeof(ngrmib_parameters_t), SS_QOS_GROUP_MIB);

    /* set group and get some global values */
    if (qos_parms != NULL) {
        qos_parms->group = GROUP_QOS; 

        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(SS_QOS_GROUP_MIB, ssQosGroup_variables, variable7,
                   ssQosGroup_variables_oid);
    }
}

/*
 * This function will be called whenever base mode has
 * to be retrieved or set.
 */
unsigned char *
var_ssQosGroup(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    static long long_ret;
    
    /* validate, validate */
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
        case QOSBASEMODE:
            /* can be modePortBased(1), mode1PBased(2)
               tosMode(3), dscpMode(4) */
            *write_method = write_qosBasemode;
            qos_parms->group_obj = SS_QOS_BASEMODE;
            break;

        default:
            ERROR_MSG("Undefined Access in ssQosGroup");
            break;
    }

    qos_parms->callback_rw = CALLBACK_ACT_READ;

    /* Callback */
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        qos_parms);
    
    /* Give the value back */
    long_ret = (long)qos_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}


/*
 * Port Priority table.
 * NOTE: Currently, only 802.1p and DSCP are supported.
 * This is not supported
 */
unsigned char *
var_qosPortPriorityTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    int if_num = 0;
    
    /* Return if mode is not port based */
    qos_parms->group_obj = SS_QOS_BASEMODE;
    qos_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        qos_parms);
    if (qos_parms->value != SS_QOS_MODE_PORTBASED) {
        return NULL;
    }
    /* Recurse for each port */
    snmp_mib_setInterfaceCount();
    if (header_simple_table(vp,name,length,exact,var_len,write_method, 
                interface_count) == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    if_num = name[(*length) - 1]; /* interface number is 1-based */
    qos_parms->index = if_num;
    switch(vp->magic) {
        case QOSPORTIDVALUE:
            qos_parms->group_obj = SS_QOS_PORTIDVALUE;
            break;
            
        case QOSPORTPRIORITYQUEUE:
            *write_method = write_qosPortPriorityQueue;
            qos_parms->group_obj = SS_QOS_PORTPRIORITYQUEUE;
        break;
        
        default:
            ERROR_MSG("Undefined MIB access in PortPriority table");
            break;
    }

    qos_parms->callback_rw = CALLBACK_ACT_READ;

    /* Callback */
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        qos_parms);
    
    /* Give the value back */
    long_ret = (long)qos_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}
/*
 * 1P Priority table handler
 * This group is supported.
 */
unsigned char *
var_qos1pPriorityTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;

    /* Return if mode is not 1P based */
    qos_parms->group_obj = SS_QOS_BASEMODE;
    qos_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        qos_parms);
    if (qos_parms->value != SS_QOS_MODE_1PBASED) {
        return NULL;
    }

    /*
     * Simple table validation. Done for max possible
     * indexes
     */
    if (header_simple_table(vp,name,length,exact,var_len,
                   write_method, MAX_IP_PRIORITY) == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    qos_parms->index = name[(*length) - 1];

     /* Validation of index, Index range - {1, 8} */
    if ((qos_parms->index < SS_QOS_MIN_PRIORITY) && 
               (qos_parms->index > SS_QOS_MAX_PRIORITY)) {
        AGENT_EXIT;
        return NULL;
    }
        
    switch(vp->magic) {
        case QOS1PCLASSVALUE:
            qos_parms->group_obj = SS_QOS_1PCLASSVALUE;
        break;
        
        case QOS1PPRIORITYQUEUE:
            *write_method = write_qos1pPriorityQueue;
            qos_parms->group_obj = SS_QOS_1PPRIORITYQUEUE;
        break;
        
        default:
            ERROR_MSG("Undefined MIB access in 802.1P Table");
        break;
    }

    qos_parms->callback_rw = CALLBACK_ACT_READ;

    /* Callback */
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        qos_parms);
    
    /* Give the value back */
    long_ret = (long)qos_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}

/*
 * var_tos2QueueTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_ssQosGroup above.
 */
unsigned char *
var_tos2QueueTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    int prec_val = 0;

    /* Return if mode is not tos based */
    qos_parms->group_obj = SS_QOS_BASEMODE;
    qos_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        qos_parms);
    if (qos_parms->value != SS_QOS_MODE_TOSBASED) {
        return NULL;
    }
    
    if (header_simple_table(vp,name,length,exact,var_len,write_method, 
                                SS_TOS_NUM_QUEUES) == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    prec_val  = name[(*length) - 1];
    qos_parms->index = prec_val;

    switch(vp->magic) {
    case PRECEDANCEVALUE:
         qos_parms->group_obj = SS_QOS_PRECEDANCEVALUE;
        break;
    case TOSPRIORITYQUEUE:
        *write_method = write_tosPriorityQueue;
         qos_parms->group_obj = SS_QOS_TOSPRIORITYQUEUE;
        break;
    default:
      ERROR_MSG("");
    }

    qos_parms->callback_rw = CALLBACK_ACT_READ;

    /* Callback */
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        qos_parms);
    
    /* Give the value back */
    long_ret = (long)qos_parms->value;
    AGENT_EXIT;
        return (unsigned char *)&long_ret;
}

/*
 * var_dscp2QueueTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_ssQosGroup above.
 */
unsigned char *
var_dscp2QueueTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    AGENT_ENTER;
    AGENT_OID_PRINT(*length, name);

    /* Return if mode is not DSCP based */
    qos_parms->group_obj = SS_QOS_BASEMODE;
    qos_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        qos_parms);
    if (qos_parms->value != SS_QOS_MODE_DSCPBASED) {
        return NULL;
    }

    /* Validate */
    if (header_simple_table(vp,name,length,exact,var_len,
                       write_method, MAX_DSCP_VALUES) == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    qos_parms->index = name[13];
    AGENT_DBG(("Current Index is %d", qos_parms->index));
    switch(vp->magic) {
        case DSCPVALUE:
            qos_parms->group_obj = SS_QOS_DSCPVALUE;
            break;

        case DSCPPRIORITYQUEUE:
            *write_method = write_dscpPriorityQueue;
            qos_parms->group_obj = SS_DSCP_PRIORITYQUEUE;
            break;
            
        default:
            ERROR_MSG("Undefined MIB reference in ssdscp2QTable");
            break;
    }

    /* Callback */
    qos_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        qos_parms);
    
    /* Give the value back */
    long_ret = qos_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}

/*
 * Function to set QoS Base Mode
 */
int
write_qosBasemode(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    long value;
    int curr_mode;

    qos_parms->group_obj =  SS_QOS_BASEMODE;
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssQosGroup not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }

          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssQosGroup: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          value = * (long *) var_val;
          if (value != SS_QOS_MODE_1PBASED && value != SS_QOS_MODE_DSCPBASED) {
              fprintf(stderr,"write to ssQosGroup: bad value\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
            /* Release any resources that have been allocated */
          break;

        case ACTION:
            /* let us get current QoS mode */
            qos_parms->callback_rw = CALLBACK_ACT_READ;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                SNMPD_CALLBACK_NETGEAR,
                                qos_parms);
            curr_mode = qos_parms->value;
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             qos_parms->value = * (long *) var_val;
             if (curr_mode == qos_parms->value) {
                AGENT_EXIT;
                return SNMP_ERR_NOERROR;
             }

             /* Finally, let us do the commit */
             qos_parms->callback_rw = CALLBACK_ACT_WRITE;
             snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                SNMPD_CALLBACK_NETGEAR,
                                qos_parms);
             if (qos_parms->ret_val == MIB_STATUS_ERROR) {
                 AGENT_EXIT;
                 return SNMP_ERR_COMMITFAILED;
             }
          break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

/*
 * QoS Port priority write function
 */
int
write_qosPortPriorityQueue(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    long value;
    int curr_value;
    int if_num = name[12]; /* 1.3.6.1.4.1.4526.11.16.2.1.1 */

    qos_parms->index = if_num;
    qos_parms->group_obj = SS_QOS_PORTPRIORITYQUEUE;
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssQosGroup not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssQosGroup: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }

          value = * (long *) var_val;
          if (value < SS_QOS_PRIO_LOWEST || value > SS_QOS_PRIO_HIGHEST) {
              fprintf(stderr,"write to ssQosGroup: bad value\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }

          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /* Let us get what is currently configured on the port */
             qos_parms->callback_rw = CALLBACK_ACT_READ;
             snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                                 SNMPD_CALLBACK_NETGEAR,
                                 qos_parms);
             curr_value = qos_parms->value;
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             qos_parms->value = * (long *) var_val;
             if (curr_value == value) {
                AGENT_EXIT;
                return SNMP_ERR_NOERROR;
             }

             /* Finally, let us do  a commit */
             qos_parms->callback_rw = CALLBACK_ACT_WRITE;
             snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                SNMPD_CALLBACK_NETGEAR,
                                qos_parms);
             if (qos_parms->ret_val == MIB_STATUS_ERROR) {
                 AGENT_EXIT;
                 return SNMP_ERR_COMMITFAILED;
             }
          break;
    }
    AGENT_EXIT;
        return SNMP_ERR_NOERROR;
}

/*
 * 1P Priority write function
 */
int
write_qos1pPriorityQueue(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    long value;
    int curr_value;
    
    qos_parms->group_obj = SS_QOS_1PPRIORITYQUEUE;
    qos_parms->index = name[12];
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssQosGroup not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssQosGroup: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          value = * (long *) var_val;
          if (value < SS_QOS_PRIO_LOWEST || value > SS_QOS_PRIO_HIGHEST) {
              fprintf(stderr,"write to ssQosGroup: bad value\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
           break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
          /* Let us get the currently configured value */
          qos_parms->callback_rw = CALLBACK_ACT_READ;
          snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                              SNMPD_CALLBACK_NETGEAR,
                              qos_parms);
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          curr_value = qos_parms->value;
          qos_parms->value = * (long *) var_val;
          if (qos_parms->value == curr_value) {
              AGENT_DBG(("Current value already set"));
              AGENT_EXIT;
              return SNMP_ERR_NOERROR;
          }

          /* Let us do the commit */
          qos_parms->callback_rw = CALLBACK_ACT_WRITE;
          snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                  SNMPD_CALLBACK_NETGEAR,
                                  qos_parms);
          if (qos_parms->ret_val == MIB_STATUS_ERROR) {
                 AGENT_EXIT;
                 return SNMP_ERR_COMMITFAILED;
          }
          break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

/*
 * ToS to priority write fucntion
 */
int
write_tosPriorityQueue(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    int curr_val;
    int tos_entry = 0;

    qos_parms->group_obj = SS_QOS_TOSPRIORITYQUEUE;
    tos_entry = name[12];
    qos_parms->index = tos_entry;
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssQosGroup not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssQosGroup: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
            /* let us get current ToS mode */
            qos_parms->callback_rw = CALLBACK_ACT_READ;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                SNMPD_CALLBACK_NETGEAR,
                                qos_parms);
            curr_val = qos_parms->value;
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             qos_parms->value = * (long *) var_val;
             if (curr_val == qos_parms->value) {
                AGENT_EXIT;
                return SNMP_ERR_NOERROR;
             }

             /* Finally, let us do the commit */
             qos_parms->callback_rw = CALLBACK_ACT_WRITE;
             snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                SNMPD_CALLBACK_NETGEAR,
                                qos_parms);
             if (qos_parms->ret_val == MIB_STATUS_ERROR) {
                 AGENT_EXIT;
                 return SNMP_ERR_COMMITFAILED;
             }
          break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

/*
 * DSCP to Priority write function
 */
int
write_dscpPriorityQueue(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    AGENT_ENTER;
    int curr_dscp;
    int index = 0;
    long value;

    qos_parms->group_obj = SS_DSCP_PRIORITYQUEUE;
    qos_parms->index = name[13];
    AGENT_DBG(("Current index = %d", qos_parms->index));
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssQosGroup not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }

          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssQosGroup: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }

          value = * (long *) var_val;
          if (value < SS_QOS_PRIO_LOWEST || value > SS_QOS_PRIO_HIGHEST) {
              fprintf(stderr,"write to ssQosGroup: bad value\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
            /* let us get current QoS mode */
            qos_parms->callback_rw = CALLBACK_ACT_READ;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                SNMPD_CALLBACK_NETGEAR,
                                qos_parms);
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             curr_dscp = qos_parms->value;
             qos_parms->value  = * (long *) var_val;
             AGENT_DBG(("Setting value %d", qos_parms->value));
             if (curr_dscp == qos_parms->value) {
                AGENT_DBG(("Value already set to current"));
                AGENT_EXIT;
                return SNMP_ERR_NOERROR;
             }

             /* Finally, let us do the commit */
             qos_parms->callback_rw = CALLBACK_ACT_WRITE;
             snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                SNMPD_CALLBACK_NETGEAR,
                                qos_parms);
             if (qos_parms->ret_val == MIB_STATUS_ERROR) {
                 AGENT_EXIT;
                 return SNMP_ERR_COMMITFAILED;
             }
          break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}
