/*
 * $Id: ssRateLimitGroup.c,v 1.3 2007/02/12 05:36:46 ganesh Exp $
 *
 * $Copyright: (c) 2004 Broadcom Corp.
 * All Rights Reserved.$
 *
 */
#include <config.h>
#include <ctype.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"
#include <ucd-snmp/mibgroup/ng700ss/netgear_mib.h>
#include "mibgroup/ng700ss/ssRateLimitGroup.h"

static ngrmib_parameters_t *ssratelimit_grp_parms = NULL;


oid ssRateLimitGroup_variables_oid[] = { 1,3,6,1,4,1,4526,11,22 };


struct variable4 ssRateLimitGroup_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

#define INGRESSRATESELECT		1
{INGRESSRATESELECT,  ASN_INTEGER,  RWRITE,  var_portRateLimitCfgTable, 3,  { 1, 1, 1 }},
#define EGRESSRATESELECT		2
{EGRESSRATESELECT,  ASN_INTEGER,  RWRITE,  var_portRateLimitCfgTable, 3,  { 1, 1, 2 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the ssRateLimitGroup module */
void
init_ssRateLimitGroup(void)
{

    DEBUGMSGTL(("ssRateLimitGroup", "Initializing\n"));
    ssratelimit_grp_parms= (ngrmib_parameters_t *) \
                           sal_alloc(sizeof(ngrmib_parameters_t),SS_RATELIMIT_GROUP_MIB);

    if (ssratelimit_grp_parms != NULL) {
        ssratelimit_grp_parms ->group = GROUP_RATE;
        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(SS_RATELIMIT_GROUP_MIB, ssRateLimitGroup_variables, variable4,
                ssRateLimitGroup_variables_oid);
    }

    /* place any other initialization junk you need here */
}

unsigned char *
var_ssRateLimitGroup(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_portRateLimitCfgTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_ssRateLimitGroup above.
 */

    unsigned char *
var_portRateLimitCfgTable(struct variable *vp,
        oid     *name,
        size_t  *length,
        int     exact,
        size_t  *var_len,
        WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    
    snmp_mib_setInterfaceCount();
    if (header_simple_table(vp,name,length,exact,var_len,write_method,interface_count)
            == MATCH_FAILED )
        return NULL;
    ssratelimit_grp_parms->index = name[(*length) - 1];

    switch(vp->magic) {
        case INGRESSRATESELECT:
            *write_method = write_ingressRateSelect;
            ssratelimit_grp_parms->group_obj=SS_INGRESSRATESELECT;
            ssratelimit_grp_parms->callback_rw=CALLBACK_ACT_READ;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssratelimit_grp_parms);
            long_ret = (long)ssratelimit_grp_parms->value;
            return (u_char *)&long_ret;
            
        case EGRESSRATESELECT:
            *write_method = write_egressRateSelect;
            ssratelimit_grp_parms->group_obj=SS_EGRESSRATESELECT;
            ssratelimit_grp_parms->callback_rw=CALLBACK_ACT_READ;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                    SNMPD_CALLBACK_NETGEAR,
                    ssratelimit_grp_parms);
            long_ret = (long)ssratelimit_grp_parms->value;
            return (u_char *)&long_ret;

        default:
            ERROR_MSG("");
    }
                return NULL;
}


int
write_ingressRateSelect(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    long value;
    int size;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssRateLimitGroup not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssRateLimitGroup: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          size  = var_val_len;
          value = * (long *) var_val;
          if((value<1) || (value>11))
          {
              return SNMP_ERR_WRONGVALUE;
          }


          break;

        case FREE:
          break;

        case ACTION:
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          value = * (int *) var_val;
          ssratelimit_grp_parms->value = value;
          ssratelimit_grp_parms->group_obj = SS_INGRESSRATESELECT;
          ssratelimit_grp_parms->callback_rw = CALLBACK_ACT_WRITE;

          /* Callback */
          snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                  SNMPD_CALLBACK_NETGEAR,
                  ssratelimit_grp_parms);
          if (ssratelimit_grp_parms->ret_val == MIB_STATUS_ERROR) {
              return SNMP_ERR_COMMITFAILED;
          }

          break;
    }
    return SNMP_ERR_NOERROR;
}
int
write_egressRateSelect(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    long value;
    int size;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssRateLimitGroup not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssRateLimitGroup: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          size  = var_val_len;
          value = * (long *) var_val;
          if((value<1) || (value>11))
          {
              return SNMP_ERR_WRONGVALUE;
          }


          break;

        case FREE:
          break;

        case ACTION:
          break;

        case UNDO:
          break;

        case COMMIT:
          value = * (int *) var_val;
          ssratelimit_grp_parms->value = value;
          ssratelimit_grp_parms->group_obj = SS_EGRESSRATESELECT;
          ssratelimit_grp_parms->callback_rw = CALLBACK_ACT_WRITE;

          /* Callback */
          snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                  SNMPD_CALLBACK_NETGEAR,
                  ssratelimit_grp_parms);
          if (ssratelimit_grp_parms->ret_val == MIB_STATUS_ERROR) {
              return SNMP_ERR_COMMITFAILED;
          }

          break;
    }
    return SNMP_ERR_NOERROR;

}
