/*
 * $Id: ssSpanGroup.c,v 1.1 2006/12/11 06:25:14 ganesh Exp $
 */

/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include <ucd-snmp/mibgroup/ng700ss/netgear_mib.h>
#include "mibgroup/ng700ss/ssSpanGroup.h"

static ngrmib_parameters_t *spantree_parms = NULL;

/* 
 * Position (OID) of spanning tree group with respect to 
 * the global MIB tree
 */
oid ssSpanGroup_variables_oid[] = { 1,3,6,1,4,1,4526,11,4 };

/* 
 *   This variable defines function callbacks and type AGENT_EXIT;
        return information 
 *   for the ssSpanGroup mib section 
 */
struct variable7 ssSpanGroup_variables[] = {
#define SPANONOFF        1   /* Enable/disable spanning tree */
    {SPANONOFF,  ASN_INTEGER,  RWRITE,  var_ssSpanGroup, 1,  { 1 }},
};


/** Initializes the ssSpanGroup module */
void
init_ssSpanGroup(void)
{
    AGENT_ENTER;

    /* Allocate memory for callback argument */
    spantree_parms = (ngrmib_parameters_t *) \
                sal_alloc(sizeof(ngrmib_parameters_t), SS_SPAN_GROUP_MIB);

    /* Set the group */   
    if(spantree_parms != NULL) {
        spantree_parms->group = GROUP_SPANTREE;

        /* Register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(SS_SPAN_GROUP_MIB, ssSpanGroup_variables, variable7,
                   ssSpanGroup_variables_oid);
    }
        
}

/* 
 * This function will be called whenever there is 
 * a get or set on spanning tree group MIB variable
 */
unsigned char *
var_ssSpanGroup(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    AGENT_ENTER;
    static long long_ret;

    /* Validate the request first */
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
        case SPANONOFF:
            /* We return 1 if enabled, 2 if disabled */
            *write_method = write_spanOnOff;
            spantree_parms->group_obj = SS_SPAN_ONOFF;
        break;

        default:
            ERROR_MSG("Undefined OID in Spanning Tree MIB");
    }

    /* Callback */
    spantree_parms->callback_rw = CALLBACK_ACT_READ; /* Get */
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        spantree_parms);
    
    /* Give the value back */
    long_ret = (long)spantree_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}



/*
 * This function is called to enable or
 * disabled spanning tree protocol on the
 * switch
 */
int
write_spanOnOff(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    int value;

    spantree_parms->group_obj = SS_SPAN_ONOFF;
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssSpanGroup not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssSpanGroup: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          
          value = * (int *) var_val;
          if (value < MIB_STATUS_ENABLED || value > MIB_STATUS_DISABLED) {
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
            /* Now, do the set */
            spantree_parms->value = * (long *) var_val;
            spantree_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                SNMPD_CALLBACK_NETGEAR,
                                spantree_parms);
            if (spantree_parms->ret_val == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
            break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

