/*
 * $Id: ssStaticMulticastGroup.c,v 1.1 2006/12/11 06:25:14 ganesh Exp $
 */

/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include <ucd-snmp/mibgroup/ng700ss/netgear_mib.h>
#include "mibgroup/ng700ss/ssStaticMulticastGroup.h"

/* Argument that will be passed during callback */
static ngrmib_parameters_t *statmcast_parms = NULL;

/* OID array identifying the path to this MIB */
oid ssStaticMulticastGroup_variables_oid[] = { 1,3,6,1,4,1,4526,11,24 };

/* Row under creation */
static ngr_statmcast_entry_t    *creation_row = NULL;

/* Variables that this MIb supports */
struct variable7 ssStaticMulticastGroup_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define STATICMCASTGROUPID        1
    {STATICMCASTGROUPID,  ASN_INTEGER,  RONLY,   var_staticMulticastCfgTable, 3,  { 1, 1, 1 }},
#define STATICMCASTGROUPMEMBER        2
    {STATICMCASTGROUPMEMBER,  ASN_OCTET_STR,  RWRITE,  var_staticMulticastCfgTable, 3,  { 1, 1, 2 }},
#define STATICMCASTVLANID        3
    {STATICMCASTVLANID,  ASN_INTEGER,  RWRITE,  var_staticMulticastCfgTable, 3,  { 1, 1, 3 }},
#define STATICMCASTMAC        4
    {STATICMCASTMAC,  ASN_OCTET_STR,  RWRITE,  var_staticMulticastCfgTable, 3,  { 1, 1, 4 }},
#define STATICMCASTSTATUS        5
    {STATICMCASTSTATUS,  ASN_INTEGER,  RWRITE,  var_staticMulticastCfgTable, 3,  { 1, 1, 5 }},
};


/** Initializes the ssStaticMulticastGroup module */
void
init_ssStaticMulticastGroup(void)
{
    AGENT_ENTER;

    /* Allocate memory for callback argument */
    statmcast_parms = (ngrmib_parameters_t *) \
                sal_alloc(sizeof(ngrmib_parameters_t), SS_STATICMCAST_GROUP_MIB);

    /* set group name */
    if (statmcast_parms != NULL) {
        statmcast_parms->group = GROUP_STAT_MCAST; 

        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(SS_STATICMCAST_GROUP_MIB, ssStaticMulticastGroup_variables, 
                     variable7,  ssStaticMulticastGroup_variables_oid);
    }
}

/*
 * var_ssStaticMulticastGroup(): This function is the top level MIB
 * entry function. It is just a path and supports no set/get variables
 */
unsigned char *
var_ssStaticMulticastGroup(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* Validate */
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
	}

    switch(vp->magic) {
        default:
          ERROR_MSG("");
    }

    AGENT_EXIT;
    return NULL;
}

static int
header_statmcastTable(struct variable *vp,
           oid *name,
           size_t *length,
           int exact,
           size_t *var_len,
           WriteMethod **write_method)
{
#define SS_STATMCAST_ID_LEN     12
#define SS_STATMCAST_TOTAL_LEN  13

    oid newname[MAX_OID_LEN];
    int i = 0, found = FALSE, result;
    int statmcast_id, statmcast_count; 

    /* In case we are creating a new row, we need to set write method first */
    *write_method = write_statmcastEntry;

    if (creation_row) {
        AGENT_DBG(("Static multicast under creation = %d", creation_row->smgid));
    } else {
        AGENT_DBG(("No Static multicast under creation"));
    }

    /* Get count of number of smcast entries on the switch */
    statmcast_parms->group_obj = SS_STATICMCAST_COUNT;
    statmcast_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_NETGEAR,
                        statmcast_parms);
    statmcast_count = statmcast_parms->value;
    AGENT_DBG(("Num entries %d", statmcast_count));

    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    /* Loop through all the indexes, plus one under creation */
    for (; i < (statmcast_count + 1); i++ ) {
        if (i == statmcast_count) {
         if (creation_row) {
            /* Get count of number of smcast entries on the switch */
            statmcast_parms->index = creation_row->smgid - 1;
            statmcast_parms->group_obj = SS_STATICMCAST_STATUS;
            statmcast_parms->callback_rw = CALLBACK_ACT_READ;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_NETGEAR,
                        statmcast_parms);
            if(statmcast_parms->value == 1) { /*Smid under creation status is active */
                sal_free (creation_row);
                creation_row = NULL;
				break;
            } else {
                creation_row->smgid = statmcast_count+1;
                statmcast_id = creation_row->smgid;
            }
            AGENT_DBG(("Num entries %d", statmcast_count));
          } else {
              break;
          }
        } else {
            statmcast_id = (i+1);
        }

        newname[SS_STATMCAST_ID_LEN] = (oid) statmcast_id ;

        result = snmp_oid_compare(name, *length, newname,
                                    (int) vp->namelen + 1);
        if ((exact && (result == 0)) || (!exact && (result < 0))) {
            found = TRUE;
			statmcast_parms->index = i;
            break;
        }
    }

    if (!found) {
        DEBUGMSGTL(("ssStatmcast", "Invalid Index"));
        AGENT_EXIT;
        return MATCH_FAILED;
    }

    memcpy( (char *)name,(char *)newname, ((int) vp->namelen +1) * sizeof(oid));

    *length = vp->namelen + 1;
    *var_len = sizeof(long);    /* default to 'long' results */

    AGENT_EXIT;
    return MATCH_SUCCEEDED;
}


/*
 * var_staticMulticastCfgTable(): This contains the static
 * multicast table readable and writable parameters.
 */
unsigned char *
var_staticMulticastCfgTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    static long long_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    int num_stat_entries = 0;


    /* Validate the input first  */
    snmp_mib_setInterfaceCount();
    if (header_statmcastTable(vp, name, length, exact, var_len,
                      write_method) == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    AGENT_DBG(("Static MCAST index is %d", statmcast_parms->index));
    switch(vp->magic) {
        case STATICMCASTGROUPID:
            long_ret = (int)name[SS_STATMCAST_ID_LEN];
            return (unsigned char *) &long_ret;

        case STATICMCASTGROUPMEMBER:
            if (creation_row && ((creation_row->smgid-1) == statmcast_parms->index)) {
                *var_len = (interface_count/INTERFACE_BYTE_COUNT);
                memcpy (string, creation_row->smports, *var_len);
                return (unsigned char *) string;
            }
            statmcast_parms->group_obj = SS_STATICMCAST_GROUPMEMBER;
            break;

        case STATICMCASTVLANID:
            if (creation_row && ((creation_row->smgid-1) == statmcast_parms->index)) {
                long_ret = creation_row->smvid;
                return (unsigned char *) &long_ret;
            }
            statmcast_parms->group_obj = SS_STATICMCAST_VLANID;
            break;

        case STATICMCASTMAC:
            if (creation_row && ((creation_row->smgid-1) == statmcast_parms->index)) {
                memcpy (string, creation_row->smmac, MAC_ADDRESS_LEN);
                *var_len = MAC_ADDRESS_LEN;
                return (unsigned char *) string;
            }
            statmcast_parms->group_obj = SS_STATICMCAST_MAC;
            break;

        case STATICMCASTSTATUS:
            if (creation_row && ((creation_row->smgid-1) == statmcast_parms->index)) {
                long_ret = creation_row->smstatus;
                return (unsigned char *) &long_ret;
            }
            statmcast_parms->group_obj = SS_STATICMCAST_STATUS;
            break;

        default:
            ERROR_MSG("Undefined MIB object reference in Static Multicast Cfg");
    }

    /* Callback */
    statmcast_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        statmcast_parms);

    /* Give the value back */
    switch(vp->magic) {
        case STATICMCASTGROUPMEMBER:
            *var_len = (interface_count/INTERFACE_BYTE_COUNT);
            memcpy (string, statmcast_parms->buf, *var_len);
            AGENT_EXIT;
            return (unsigned char *)string;

        case STATICMCASTMAC:
            memcpy (string, statmcast_parms->buf,MAC_ADDRESS_LEN);
            *var_len = MAC_ADDRESS_LEN;
            AGENT_EXIT;
            return (unsigned char *)string;
    }

    long_ret = (long)statmcast_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}


/*
 * Write/Create function for Static Multicast
 */
int
write_statmcastEntry(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    AGENT_ENTER;
    AGENT_OID_PRINT(name_len, name);
    long            value;
    int             var;
    int mcast_id;
    unsigned char  string[6]; 
    unsigned char mc_mac[] = {0x01, 0x00, 0x5e};
    unsigned char  temp_mac[6];

    memset (temp_mac, 0, 6);
         
    var = name[11];
    mcast_id = (int) name[SS_STATMCAST_ID_LEN];
    statmcast_parms->index  = mcast_id - 1;
	AGENT_DBG(("STAT_MCAST: Var being set = %d", var));

    /* Create an entry if createAndWait */
    value = *(long *) var_val;
    if (creation_row == NULL && 
           (var == STATICMCASTSTATUS && value == MIB_ROWSTATUS_CREATEANDWAIT)) {
        /* Creation index must not exist, and count must be one less than  
           index being created */
        statmcast_parms->group_obj = SS_STATICMCAST_COUNT;
        statmcast_parms->callback_rw = CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                            SNMPD_CALLBACK_NETGEAR,
                            statmcast_parms);
        if (statmcast_parms->value >= mcast_id || 
                            (mcast_id > (statmcast_parms->value + 1))) {
            AGENT_DBG(("Entry already exists"));
            return SNMP_ERR_WRONGVALUE;
        }

        creation_row = (ngr_statmcast_entry_t *) 
                                sal_alloc(sizeof(ngr_statmcast_entry_t), 
                                          SS_STATICMCAST_GROUP_MIB);

        if (creation_row == NULL) {
            AGENT_DBG(("Unable to allocate memory"));
            return SNMP_ERR_RESOURCEUNAVAILABLE;
        }

        /* zero out all fields */
        memset(creation_row, 0, sizeof(ngr_statmcast_entry_t));

        /* Set the index and vlan id */
        creation_row->smgid = mcast_id;
        creation_row->smvid = 1; /* Default VLAN ID */

        /* Set the row status to notReady */
        creation_row->smstatus = 3;
    } else if (creation_row != NULL && 
           (var == STATICMCASTSTATUS && value == MIB_ROWSTATUS_CREATEANDWAIT)) {
        /* We can have only one row under creation */
        if (mcast_id != creation_row->smgid) {
            AGENT_DBG(("We can have only one Mcast entry under creation"));
            return SNMP_ERR_RESOURCEUNAVAILABLE;
        }
    }


    switch ( action ) {
        case RESERVE1:
            /* No modification of existing entries allowed */
            if (var != STATICMCASTSTATUS && (creation_row == NULL ||
                              mcast_id != creation_row->smgid)) {
                return SNMP_ERR_NOTWRITABLE;
            }

            switch(var) {
                case STATICMCASTGROUPID:
                    if (var_val_type != ASN_INTEGER) {
                        fprintf(stderr, 
                                "write to StatMcast not ASN_INTEGER\n");
                        return SNMP_ERR_WRONGTYPE;
                    }
                    if (var_val_len > sizeof(long)) {
                        fprintf(stderr, "write to StatMcast: bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }
                    break;

                case STATICMCASTGROUPMEMBER:
                    if (var_val_type != ASN_OCTET_STR) {
                        fprintf(stderr, "write to StatMcast not ASN_OCTET_STR\n");
                        return SNMP_ERR_WRONGTYPE;
                    }

                    if (var_val_len != 
                        (sizeof(char)*(interface_count/INTERFACE_BYTE_COUNT))) {
                        fprintf(stderr, "write to StatMcast: bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }
                    
                    memset (statmcast_parms->buf, 0, sizeof(statmcast_parms->buf));
                    if (!memcmp(var_val, statmcast_parms->buf, var_val_len)) {
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;

                case STATICMCASTVLANID:
                   if (var_val_type != ASN_INTEGER) {
                        fprintf(stderr,
                                "write to StatMcast not ASN_INTEGER\n");
                        return SNMP_ERR_WRONGTYPE;
                    }
                    if (var_val_len > sizeof(long)) {
                        fprintf(stderr, "write to StatMcast: bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }
                    /* No write if port based vlan */
                    if (snmp_mib_getVlanMode() == MIB_VLAN_MODE_PORTBASED) {
                        return SNMP_ERR_NOTWRITABLE;
                    }

                    /* check if vlan exists */
                    statmcast_parms->group = GROUP_TVLAN;
                    statmcast_parms->group_obj = SS_TVLAN_VID_QUERY;
                    statmcast_parms->callback_rw = CALLBACK_ACT_READ;
                    statmcast_parms->value = value;
                    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                            SNMPD_CALLBACK_NETGEAR,
                                            statmcast_parms);
                    statmcast_parms->group = GROUP_STAT_MCAST;
                    if (statmcast_parms->value != MIB_ROWSTATUS_ACTIVE) {
                        AGENT_EXIT;
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;
                case STATICMCASTMAC:
                    if (var_val_type != ASN_OCTET_STR) {
                        fprintf(stderr, "write to StatMcast not ASN_OCTET_STR\n");
                        return SNMP_ERR_WRONGTYPE;
                    }

                    if (var_val_len != MAC_ADDRESS_LEN) {
                        fprintf(stderr, "write to StatMcast: bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }
                    
                    if (memcmp(var_val, mc_mac, 3) != 0) {
                        fprintf(stderr, "write to StatMcast: bad value\n");
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;

               case STATICMCASTSTATUS:
                    if (var_val_type != ASN_INTEGER) {
                        fprintf(stderr, "write to StatMcast not ASN_INTEGER\n");
                        return SNMP_ERR_WRONGTYPE;
                    }

                    if (var_val_len > sizeof(long)) {
                        fprintf(stderr, "write to StatMcast: bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }

                    if (value != MIB_ROWSTATUS_ACTIVE && 
                        value != MIB_ROWSTATUS_NOTREADY && 
                        value != MIB_ROWSTATUS_CREATEANDWAIT && 
                        value != MIB_ROWSTATUS_DESTROY) {
                        fprintf(stderr, "write to StatMcast: bad value\n");
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;
            }
        break;

        case RESERVE2:
        break;

        case FREE:
        break;

        case ACTION:
    		switch(var) {
		        case STATICMCASTSTATUS:
		            if (creation_row && (mcast_id != creation_row->smgid)) {
			            if (value == MIB_ROWSTATUS_ACTIVE) {
                     		statmcast_parms->index = creation_row->smgid - 1;
                            statmcast_parms->group_obj = SS_STATICMCAST_STATUS;
                            statmcast_parms->callback_rw = CALLBACK_ACT_READ;
                            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                                SNMPD_CALLBACK_NETGEAR,
                                                statmcast_parms);
                            if(statmcast_parms->value == MIB_ROWSTATUS_ACTIVE) { 
                                /*Smid under creation status is active */
                                sal_free (creation_row);
                                creation_row = NULL;
                                return SNMP_ERR_NOERROR;
                            }
				        }
			        }
					break;
				}
          break;

        case UNDO:
          break;

        case COMMIT:
            switch(var) {
                case STATICMCASTGROUPMEMBER:
                    if ((creation_row == NULL) || mcast_id != creation_row->smgid) {
                        statmcast_parms->group_obj = SS_STATICMCAST_GROUPMEMBER;
                        memcpy(statmcast_parms->buf, var_val, var_val_len);
                        statmcast_parms->value = var_val_len;
                    } else  if (creation_row) {
                        AGENT_DBG(("length = %d", var_val_len));
                        memset (creation_row->smports, 0, var_val_len);
                        memcpy(creation_row->smports, var_val, var_val_len);
                    }
                    break;

                case STATICMCASTVLANID:
                    if ((creation_row == NULL) || mcast_id != creation_row->smgid) {
                        statmcast_parms->group_obj = SS_STATICMCAST_VLANID;
                        statmcast_parms->value = * (unsigned short *) var_val;
                    } else  if (creation_row) {
                        creation_row->smvid = * (unsigned short *) var_val;
                    }
                    break;

               case STATICMCASTMAC:
                    if ((creation_row == NULL) || mcast_id != creation_row->smgid) {
                        statmcast_parms->group_obj = SS_STATICMCAST_MAC;
                        memcpy(statmcast_parms->buf, var_val, var_val_len);
                        statmcast_parms->value = var_val_len;
                    } else  if (creation_row) {
                        memset (creation_row->smmac, 0, var_val_len);
                        memcpy(creation_row->smmac, var_val, var_val_len);
                    }
                    break;


                case STATICMCASTSTATUS:
                    statmcast_parms->group_obj = SS_STATICMCAST_STATUS;
                    statmcast_parms->value = value;
                    if ((creation_row == NULL) 
                                        || mcast_id != creation_row->smgid) {
                        if (value == MIB_ROWSTATUS_ACTIVE) {
                            return SNMP_ERR_WRONGVALUE; /* Already active */
                        } else if (value == MIB_ROWSTATUS_NOTREADY || 
                                   value == MIB_ROWSTATUS_CREATEANDWAIT) {
                            /* Already active, so cant set not ready */
                            return SNMP_ERR_WRONGVALUE;
                        }
                    } else  if (creation_row && 
                                            mcast_id == creation_row->smgid) {
                        if (value == MIB_ROWSTATUS_DESTROY) { /* Destroy */
                            sal_free (creation_row);
                            creation_row = NULL;
                            return SNMP_ERR_NOERROR;
                        } else if (value == MIB_ROWSTATUS_ACTIVE) { /* active */
                            /* Check if valid port */
                            memset (statmcast_parms->buf, 0, 
                                                sizeof(statmcast_parms->buf));
                            if (!memcmp(statmcast_parms->buf, 
                                       creation_row->smports, 
                                       interface_count/INTERFACE_BYTE_COUNT)) {
                                return SNMP_ERR_WRONGVALUE;
                            }

                            if ( !creation_row->smvid ||
                                 !memcmp(creation_row->smmac, temp_mac,
                                                            MAC_ADDRESS_LEN)) {
                                return SNMP_ERR_WRONGVALUE;
                            }
                                     
                            memcpy (&statmcast_parms->group_data.smcast,
                                    creation_row, 
                                    sizeof(ngr_statmcast_entry_t));
                        }
                    }
                    break;
            }

            /* call backend if already existing row, or to activate row
               under creation */
            if ((creation_row == NULL || mcast_id != creation_row->smgid) ||
                (mcast_id == creation_row->smgid && 
                                      var == STATICMCASTSTATUS && value == 1)) {
                    statmcast_parms->callback_rw = CALLBACK_ACT_WRITE;
                    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                        SNMPD_CALLBACK_NETGEAR,
                                        statmcast_parms);
                    if (statmcast_parms->ret_val == MIB_STATUS_ERROR) {
                        AGENT_EXIT;
                        return SNMP_ERR_COMMITFAILED;
                    }
                    /* If activating createAndWait, free mem */
                    if (creation_row && mcast_id == creation_row->smgid) {
                        sal_free (creation_row);
                        creation_row = NULL;
                    }
            }
        break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

void free_statmcast_createAndWaitMemory(void) {
    if (creation_row) {
        sal_free(creation_row);
        creation_row = NULL;
    }

    return;
}
