/*
 * $Id: ssStormCtrlGroup.c,v 1.2 2007/01/25 17:58:36 ganesh Exp $
 *
 * $Copyright: (c) 2004 Broadcom Corp.
 * All Rights Reserved.$
 *
 */
#include <config.h>
#include <ctype.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"
#include <ucd-snmp/mibgroup/ng700ss/netgear_mib.h>
#include "mibgroup/ng700ss/ssStormCtrlGroup.h"

static ngrmib_parameters_t *ssStrom_grp_parms = NULL;

oid ssStormCtrlGroup_variables_oid[] = { 1,3,6,1,4,1,4526,11,17 };


struct variable4 ssStormCtrlGroup_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define CONFIGSTORMCONTROLSETTING		1
{CONFIGSTORMCONTROLSETTING,  ASN_INTEGER,  RWRITE,  var_ssStormCtrlGroup, 1,  { 1 }},
#define CONFIGLIMITSELECTION		2
{CONFIGLIMITSELECTION,  ASN_INTEGER,  RWRITE,  var_ssStormCtrlGroup, 1,  { 2 }},
#define CONFIGLIMITVALUE		3
{CONFIGLIMITVALUE,  ASN_INTEGER,  RWRITE,  var_ssStormCtrlGroup, 1,  { 3 }},

#define CONFIGPERPORTCTRLSTATUS		4
{CONFIGPERPORTCTRLSTATUS,  ASN_INTEGER,  RWRITE,  var_configPerPortCtrlTable, 3,  { 4, 1, 1 }},
#define STORMCTRLCURRENTRATELIMIT		5
{STORMCTRLCURRENTRATELIMIT,  ASN_INTEGER,  RONLY,   var_configPerPortCtrlTable, 3,  { 4, 1, 2 }},
};
/*    (L = length of the oidsuffix) */


void
init_ssStormCtrlGroup(void)
{

    DEBUGMSGTL(("ssStormCtrlGroup", "Initializing\n"));
    ssStrom_grp_parms= (ngrmib_parameters_t *) \
                           sal_alloc(sizeof(ngrmib_parameters_t),SS_STROM_GROUP_MIB);

    ssStrom_grp_parms->group=GROUP_STROM;
    /* register ourselves with the agent to handle our mib tree */
    REGISTER_MIB(SS_STROM_GROUP_MIB, ssStormCtrlGroup_variables, variable4,
            ssStormCtrlGroup_variables_oid);

}

unsigned char *
var_ssStormCtrlGroup(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case CONFIGSTORMCONTROLSETTING:
        *write_method = write_configStormControlSetting;
        ssStrom_grp_parms->group_obj=SS_CONFIGSTORMCONTROLSETTING;
        ssStrom_grp_parms->callback_rw=CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                SNMPD_CALLBACK_NETGEAR,
                ssStrom_grp_parms);
        /* Give the value back */
        long_ret = (long)ssStrom_grp_parms->value;
        return (u_char *)&long_ret;

    case CONFIGLIMITSELECTION:
        *write_method = write_configLimitSelection;
        ssStrom_grp_parms->group_obj=SS_CONFIGLIMITSELECTION;

        ssStrom_grp_parms->callback_rw=CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                SNMPD_CALLBACK_NETGEAR,
                ssStrom_grp_parms);
        /* Give the value back */
        long_ret = (long)ssStrom_grp_parms->value;
         return (u_char *)&long_ret;
    case CONFIGLIMITVALUE:
         *write_method = write_configLimitValue;
         ssStrom_grp_parms->group_obj=SS_CONFIGLIMITVALUE;

         ssStrom_grp_parms->callback_rw=CALLBACK_ACT_READ;
         snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                 SNMPD_CALLBACK_NETGEAR,
                 ssStrom_grp_parms);
         /* Give the value back */
         long_ret = (long)ssStrom_grp_parms->value;
         return (u_char *)&long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;

}


unsigned char *
var_configPerPortCtrlTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;

    snmp_mib_setInterfaceCount();
    if (header_simple_table(vp,name,length,exact,var_len,write_method,interface_count)
                                                == MATCH_FAILED )
    return NULL;
   ssStrom_grp_parms->index = name[(*length) - 1] ;
    

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case CONFIGPERPORTCTRLSTATUS:
        *write_method = write_configPerPortCtrlStatus;
        ssStrom_grp_parms->group_obj=SS_STORMCONFIGPERPORTCTRLSTATUS;
        ssStrom_grp_parms->callback_rw=CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                SNMPD_CALLBACK_NETGEAR,
                ssStrom_grp_parms);
        /* Give the value back */
        long_ret = (long)ssStrom_grp_parms->value;
        return (u_char *)&long_ret;

    case STORMCTRLCURRENTRATELIMIT:
        ssStrom_grp_parms->group_obj=SS_STORMCTRLCURRENTRATELIMIT;

        ssStrom_grp_parms->callback_rw=CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                SNMPD_CALLBACK_NETGEAR,
                ssStrom_grp_parms);
        /* Give the value back */
        long_ret = (long)ssStrom_grp_parms->value;
        return (u_char *)&long_ret;

    default:
      ERROR_MSG("");
    }
    return NULL;
}



int
write_configStormControlSetting(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    long value;
    int size;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssStormCtrlGroup not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssStormCtrlGroup: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          size  = var_val_len;
          value = * (long *) var_val;
          if((value<1) || (value>4))
          {
              return SNMP_ERR_WRONGVALUE;
          }

          break;

        case FREE:
          break;

        case ACTION:
          break;

        case UNDO:
          break;

        case COMMIT:
          value = * (int *) var_val;
          ssStrom_grp_parms->value = value;
          ssStrom_grp_parms->group_obj =SS_CONFIGSTORMCONTROLSETTING ;
          ssStrom_grp_parms->callback_rw = CALLBACK_ACT_WRITE;

          /* Callback */
          snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                  SNMPD_CALLBACK_NETGEAR,
                  ssStrom_grp_parms);
          if (ssStrom_grp_parms->ret_val == MIB_STATUS_ERROR) {
              return SNMP_ERR_COMMITFAILED;
          }
          break;
    }
    return SNMP_ERR_NOERROR;
}


int
write_configLimitSelection(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    long value;
    int size;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssStormCtrlGroup not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssStormCtrlGroup: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          size  = var_val_len;
          value = * (long *) var_val;
          if((value<1) || (value>6))
          {
              return SNMP_ERR_WRONGVALUE;
          }

          break;

        case FREE:
          break;

        case ACTION:
          break;

        case UNDO:
          break;

        case COMMIT:
          value = * (int *) var_val;
          ssStrom_grp_parms->value = value;
          ssStrom_grp_parms->group_obj =SS_CONFIGLIMITSELECTION;
          ssStrom_grp_parms->callback_rw = CALLBACK_ACT_WRITE;

          /* Callback */
          snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                  SNMPD_CALLBACK_NETGEAR,
                  ssStrom_grp_parms);
          if (ssStrom_grp_parms->ret_val == MIB_STATUS_ERROR) {
              return SNMP_ERR_COMMITFAILED;
          }

          break;
    }
    return SNMP_ERR_NOERROR;
}


int
write_configLimitValue(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    long value;
    int size;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssStormCtrlGroup not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssStormCtrlGroup: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          size  = var_val_len;
          value = * (long *) var_val;
          if((value<0) || (value>2147483647))
          {
              return SNMP_ERR_WRONGVALUE;
          }


          break;

        case FREE:
          break;

        case ACTION:
          break;

        case UNDO:
          break;

        case COMMIT:
          value = * (int *) var_val;
          ssStrom_grp_parms->value = value;
          ssStrom_grp_parms->group_obj =SS_CONFIGLIMITVALUE;
          ssStrom_grp_parms->callback_rw = CALLBACK_ACT_WRITE;
          /* Callback */
          snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                  SNMPD_CALLBACK_NETGEAR,
                  ssStrom_grp_parms);
          if (ssStrom_grp_parms->ret_val == MIB_STATUS_ERROR) {
              return SNMP_ERR_COMMITFAILED;
          }
          break;
    }
    return SNMP_ERR_NOERROR;
}

int
write_configPerPortCtrlStatus(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    long value;
    int size;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssStormCtrlGroup not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssStormCtrlGroup: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          size  = var_val_len;
          value = * (long *) var_val;
          break;

        case FREE:
          break;

        case ACTION:
          break;

        case UNDO:
          break;

        case COMMIT:
          value = * (int *) var_val;
          ssStrom_grp_parms->value = value;
          ssStrom_grp_parms->group_obj =SS_STORMCONFIGPERPORTCTRLSTATUS;
          ssStrom_grp_parms->callback_rw = CALLBACK_ACT_WRITE;

          /* Callback */
          snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                  SNMPD_CALLBACK_NETGEAR,
                  ssStrom_grp_parms);
          if (ssStrom_grp_parms->ret_val == MIB_STATUS_ERROR) {
              return SNMP_ERR_COMMITFAILED;
          }
          break;
    }
    return SNMP_ERR_NOERROR;
}
