/*
 * $Id: ssTVlanGroup.c,v 1.1 2006/12/11 06:25:14 ganesh Exp $
 */

/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include <ucd-snmp/mibgroup/ng700ss/netgear_mib.h>
#include "mibgroup/ng700ss/ssTVlanGroup.h"

#define C_TVLAN_NAME_LEN 8

/* callback parameters */
static ngrmib_parameters_t *sstvlangrp_parms = NULL;

/*  Path to this MIB WRT the global MIB tree */
oid ssTVlanGroup_variables_oid[] = { 1,3,6,1,4,1,4526,11,13 };

/* Row under creation */
static ngr_vlan_entry_t    *creation_row = NULL;

/* index of vlan under creation */
static int curr_vid_index = 0;

/* Variables that this MIB supports */
struct variable7 ssTVlanGroup_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define TVLANID        1
    {TVLANID,  ASN_INTEGER,  RONLY,   var_tvlanTable, 3,  { 1, 1, 1 }},
#define TVLANNAME        2
    {TVLANNAME,  ASN_OCTET_STR,  RWRITE,  var_tvlanTable, 3,  { 1, 1, 2 }},
#define TVLANMEMBER        3
    {TVLANMEMBER,  ASN_OCTET_STR,  RWRITE,  var_tvlanTable, 3,  { 1, 1, 3 }},
#define TVLANUNTAGGEDPORTS        4
    {TVLANUNTAGGEDPORTS,  ASN_OCTET_STR,  RWRITE,  var_tvlanTable, 3,  { 1, 1, 4 }},
#define TVLANSTATUS        5
    {TVLANSTATUS,  ASN_INTEGER,  RWRITE,  var_tvlanTable, 3,  { 1, 1, 5 }},
#define TVLANPORTPORTID        6
    {TVLANPORTPORTID,  ASN_INTEGER,  RONLY,   var_tvlanPortTable, 3,  { 2, 1, 1 }},
#define TVLANPORTVLANID        7
    {TVLANPORTVLANID,  ASN_INTEGER,  RWRITE,  var_tvlanPortTable, 3,  { 2, 1, 2 }},
#define TVLANPORTPRIORITY        8
    {TVLANPORTPRIORITY,  ASN_INTEGER,  RWRITE,  var_tvlanPortTable, 3,  { 2, 1, 3 }},
#define TVLANPORTTAGGEDMODE        9
    {TVLANPORTTAGGEDMODE,  ASN_INTEGER,  RONLY,   var_tvlanPortTable, 3,  { 2, 1, 4 }},
};

/** Initializes the ssTVlanGroup module */
void
init_ssTVlanGroup(void)
{
    AGENT_ENTER;

    /* Allocate memory for callback argument */
    sstvlangrp_parms = (ngrmib_parameters_t *) \
                sal_alloc(sizeof(ngrmib_parameters_t), SS_TVLAN_GROUP_MIB);

    /* set group name */
    if (sstvlangrp_parms != NULL) {
        AGENT_DBG(("Registering TVLAN"));
        sstvlangrp_parms->group = GROUP_TVLAN; 

        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(SS_TVLAN_GROUP_MIB, ssTVlanGroup_variables, variable7,
                   ssTVlanGroup_variables_oid);
    }
}

/*
 * This function is called when there is a request for a 
 * TVlanGroup object. If there is no element, nothing is done.
 */
unsigned char *
var_ssTVlanGroup(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    AGENT_ENTER;

    /* Validate request */
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
        default:
          ERROR_MSG("");
    }

    AGENT_EXIT;
    return NULL;
}

static int
header_tvlanTable(struct variable *vp,
           oid *name,
           size_t *length,
           int exact,
           size_t *var_len,
           WriteMethod **write_method)
{
#define SS_TVLAN_ID_LEN     12
#define SS_TVLAN_TOTAL_LEN  13

    oid newname[MAX_OID_LEN];
    oid lowest[MAX_OID_LEN];
    int i = 0, found = FALSE;
    int vlan_id, vlan_count; 

    /* In case we are creating a new row, we need to set write method first */
    *write_method = write_tvlanEntry;

    if (creation_row) {
        AGENT_DBG(("Vlan under creation = %d", creation_row->vid));
    } else {
        AGENT_DBG(("No vlan under creation"));
    }

    /* Get count of number of VLANs on the switch */
    sstvlangrp_parms->callback_rw = CALLBACK_ACT_READ;
    sstvlangrp_parms->group_obj = SS_TVLAN_COUNT;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_NETGEAR,
                        sstvlangrp_parms);
    vlan_count = sstvlangrp_parms->value;
    AGENT_DBG(("NUmber of VLANs %d", vlan_count));
    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    /* VLAN under creation because of createAndWait */
    if (creation_row) {
        sstvlangrp_parms->value = creation_row->vid;
        sstvlangrp_parms->group_obj = SS_TVLAN_VID_QUERY;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                            SNMPD_CALLBACK_NETGEAR,
                            sstvlangrp_parms);
        if (sstvlangrp_parms->value == 1) {
            sal_free(creation_row);
            creation_row = NULL;
        } else {
            i = -1;
        }
    }

    /* Loop through all the indexes, plus one under creation */
    for (; i < vlan_count; i++ ) {
        if (i == -1) { /* First, check VLAN under creation*/
            vlan_id = creation_row->vid;
        } else {
            sstvlangrp_parms->index = i;
            sstvlangrp_parms->group_obj = SS_TVLAN_ID;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                SNMPD_CALLBACK_NETGEAR,
                                sstvlangrp_parms);
            vlan_id = sstvlangrp_parms->value;
            AGENT_DBG(("VLAN ID = %d", vlan_id));
        }

        if (vlan_id < 1 || vlan_id > 4094) {
            continue;
        }

        newname[SS_TVLAN_ID_LEN] = (oid) vlan_id ;
        if (exact) {
            if (snmp_oid_compare(name, *length, newname, 
                        SS_TVLAN_TOTAL_LEN) == 0) {
                memcpy(lowest, newname, 
                                SS_TVLAN_TOTAL_LEN * sizeof(oid));
                found = TRUE;
                curr_vid_index = i;
                AGENT_DBG(("Index exact %d", i));
                break;
            }
        } else {
            if ((snmp_oid_compare(newname, SS_TVLAN_TOTAL_LEN, 
                   name, *length) > 0) && (found == FALSE || 
                       snmp_oid_compare(newname, 
                       SS_TVLAN_TOTAL_LEN, lowest, 
                           SS_TVLAN_TOTAL_LEN) < 0))  {
                memcpy((char *) lowest, (char *) newname,
                       SS_TVLAN_TOTAL_LEN * sizeof(oid));
                AGENT_DBG(("Index %d", i));
                curr_vid_index = i;
                found = TRUE;
            }
        }
    }

    if (!found) {
        DEBUGMSGTL(("ssTVlan", "Invalid Index"));
        AGENT_EXIT;
        return MATCH_FAILED;
    }

    memcpy( (char *)name,(char *)lowest, ((int) vp->namelen +1) * sizeof(oid));

    *length = vp->namelen + 1;
    *var_len = sizeof(long);    /* default to 'long' results */

    AGENT_EXIT;
    return MATCH_SUCCEEDED;
}

/*
 * This function is called when a set or get has to be done for
 * a VLAN created on the switch.
 */
unsigned char *
var_tvlanTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    AGENT_ENTER;
    AGENT_OID_PRINT(*length, name);

    /* return if not 802.1Q VLAN */
    if (snmp_mib_getVlanMode() != MIB_VLAN_MODE_TAGGED) {
        AGENT_EXIT;
        return NULL;
    }

    snmp_mib_setInterfaceCount();

    /* Validate the entry */
    if (header_tvlanTable(vp,name,length,exact,var_len,write_method)
                                                     == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    sstvlangrp_parms->index  = curr_vid_index;
    AGENT_DBG(("Current index is %d", curr_vid_index));
    switch(vp->magic) {
        case TVLANID:
            long_ret = (int)name[SS_TVLAN_ID_LEN];
            return (unsigned char *) &long_ret;
            break;

        case TVLANNAME:
            if (curr_vid_index == -1) {
                strcpy (string, creation_row->vname);
                *var_len = strlen(string);
                return (unsigned char *) string;
            }
            sstvlangrp_parms->group_obj = SS_TVLAN_NAME;
            break;

        case TVLANMEMBER:
            if (curr_vid_index == -1) {
                AGENT_DBG(("Row under creation, returning the same"));
                memcpy (string, creation_row->vports, 
                                    (interface_count/INTERFACE_BYTE_COUNT));
                *var_len = (interface_count/INTERFACE_BYTE_COUNT);
                return (unsigned char *) string;
            }
            sstvlangrp_parms->group_obj = SS_TVLAN_MEMBER;
            break;

        case TVLANUNTAGGEDPORTS:
            if (curr_vid_index == -1) {
                AGENT_DBG(("Row under creation, returning the same"));
                memcpy (string, creation_row->vuntagports, 
                                    (interface_count/INTERFACE_BYTE_COUNT));
                *var_len = (interface_count/INTERFACE_BYTE_COUNT);
                return (unsigned char *) string;
            }
            sstvlangrp_parms->group_obj = SS_TVLAN_UNTAGGEDPORTS;
            break;

        case TVLANSTATUS:
            if (curr_vid_index == -1) {
                long_ret = creation_row->vstatus;
                return (unsigned char *) &long_ret;
            }
            sstvlangrp_parms->group_obj = SS_TVLAN_STATUS;
            break;
        default:
            ERROR_MSG("Undefined MIB access in TVLAN table");
    }

    /* Callback */
    sstvlangrp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_NETGEAR,
                        sstvlangrp_parms);

    /* Give the value back */
    switch(vp->magic) {
        case TVLANNAME:
            *var_len = sstvlangrp_parms->value;
            memcpy(string, sstvlangrp_parms->buf, *var_len);
            AGENT_EXIT;
            return (unsigned char *) string;

        case TVLANMEMBER:
        case TVLANUNTAGGEDPORTS:
            *var_len = (interface_count/INTERFACE_BYTE_COUNT);
            memcpy(string, sstvlangrp_parms->buf, *var_len);
            AGENT_EXIT;
            return (unsigned char *) string;
    }

    long_ret = (long)sstvlangrp_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}

/*
 * This function is called when VLAN related port information
 * has to be queried.
 */
unsigned char *
var_tvlanPortTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    static long long_ret;
    AGENT_ENTER;
    AGENT_OID_PRINT(*length, name);


    /* return if not 802.1Q VLAN */
    if (snmp_mib_getVlanMode() != MIB_VLAN_MODE_TAGGED) {
        AGENT_EXIT;
        return NULL;
    }
    snmp_mib_setInterfaceCount();

    /* Validate request */
    if (header_simple_table(vp,name,length,exact,var_len,write_method,
                                  interface_count) == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    sstvlangrp_parms->index = (int)name[12];
    switch(vp->magic) {
        case TVLANPORTPORTID:
            sstvlangrp_parms->group_obj = SS_TVLAN_PORTPORTID;
            break;
        case TVLANPORTVLANID:
            *write_method = write_tvlanPortVlanId;
            sstvlangrp_parms->group_obj = SS_TVLAN_PORTVLANID;
            break;
        case TVLANPORTPRIORITY:
            *write_method = write_tvlanPortPriority;
            sstvlangrp_parms->group_obj = SS_TVLAN_PORTPRIORITY;
            break;
        case TVLANPORTTAGGEDMODE:
            sstvlangrp_parms->group_obj = SS_TVLAN_PORTTAGGEDMODE;
            break;
        default:
            ERROR_MSG("Undefined MIB reference in Port VLAN table");
    }

    /* Callback */
    sstvlangrp_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_NETGEAR,
                        sstvlangrp_parms);

    /* Give the value back */
    long_ret = (long)sstvlangrp_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}

/*
 * Write/Create function for tagged VLAN
 */
int
write_tvlanEntry(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    AGENT_ENTER;
    AGENT_OID_PRINT(name_len, name);
    long            value = 0;
    int             var;
    int vid;

    var = name[SS_TVLAN_ID_LEN-1];
    vid = (int) name[SS_TVLAN_ID_LEN];
    sstvlangrp_parms->index  = curr_vid_index;
    AGENT_DBG(("TVLAN: Var being set = %d", var));

    if (var_val)
        value = *(long *) var_val;

    /* Create an entry if createAndWait */
    AGENT_DBG(("index = %d", curr_vid_index));
    AGENT_DBG(("Var = %d", var));
    if (creation_row == NULL && (var == TVLANSTATUS && 
                                    value == MIB_ROWSTATUS_CREATEANDWAIT)) {
        if (vid == 0) {
            return SNMP_ERR_WRONGVALUE;
        }

        /*  Create if the VLAN does not already exist */
        sstvlangrp_parms->value = vid;
        AGENT_DBG(("VLAN ID is %d", vid));
        sstvlangrp_parms->group_obj = SS_TVLAN_VID_QUERY;
        sstvlangrp_parms->callback_rw = CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                            SNMPD_CALLBACK_NETGEAR,
                            sstvlangrp_parms);
        if (sstvlangrp_parms->value == MIB_ROWSTATUS_ACTIVE) {
            AGENT_DBG(("VLAN already exists"));
            return SNMP_ERR_WRONGVALUE;
        }

        creation_row = (ngr_vlan_entry_t *) 
                        sal_alloc(sizeof(ngr_vlan_entry_t), SS_TVLAN_GROUP_MIB);

        if (creation_row == NULL) {
            AGENT_DBG(("Unable to allocate memory"));
            return SNMP_ERR_RESOURCEUNAVAILABLE;
        }

        /* zero out all fields */
        memset(creation_row, 0, sizeof(ngr_vlan_entry_t));

        /* Get the vlan id*/
        creation_row->vid = vid;

        /* Set the row status to notReady */
        creation_row->vstatus = MIB_ROWSTATUS_NOTREADY;
    } else if (creation_row != NULL && (var == TVLANSTATUS && 
                                    value == MIB_ROWSTATUS_CREATEANDWAIT)) {
        /* We can have only one row under creation */
        if (vid != creation_row->vid) {
            AGENT_DBG(("We can have only one VLAN under creation"));
            return SNMP_ERR_RESOURCEUNAVAILABLE;
        }
    }


    switch ( action ) {
        case RESERVE1:
            switch(var) {
                case TVLANNAME:
                    if (var_val_type != ASN_OCTET_STR) {
                        fprintf(stderr, 
                                "write to TVlan not ASN_OCTET_STR\n");
                        return SNMP_ERR_WRONGTYPE;
                    }
                    if (var_val_len > sizeof(char) * C_TVLAN_NAME_LEN) {
                        fprintf(stderr, "write to TVlan: bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }
                    break;

                case TVLANMEMBER:
                    if (var_val_type != ASN_OCTET_STR) {
                        fprintf(stderr, "write to TVlan not ASN_OCTET_STR\n");
                        return SNMP_ERR_WRONGTYPE;
                    }

                    if (var_val_len != 
                        (sizeof(char)*(interface_count/INTERFACE_BYTE_COUNT))) {
                        fprintf(stderr, "write to TVlan: bad length\n");
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;

                case TVLANUNTAGGEDPORTS:
                    if (var_val_type != ASN_OCTET_STR) {
                        fprintf(stderr, "write to TVlan not ASN_OCTET_STR\n");
                        return SNMP_ERR_WRONGTYPE;
                    }

                    if (var_val_len != 
                        (sizeof(char)*(interface_count/INTERFACE_BYTE_COUNT))) {
                        fprintf(stderr, "write to TVlan: bad length\n");
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;

                case TVLANSTATUS:
                    if (var_val_type != ASN_INTEGER) {
                        fprintf(stderr, "write to TVlan not ASN_INTEGER\n");
                        return SNMP_ERR_WRONGTYPE;
                    }

                    if (var_val_len > sizeof(long)) {
                        fprintf(stderr, "write to TVlan: bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }

                    if (value != MIB_ROWSTATUS_ACTIVE && 
                            value != MIB_ROWSTATUS_NOTREADY && 
                            value != MIB_ROWSTATUS_CREATEANDWAIT && 
                            value != MIB_ROWSTATUS_DESTROY) {
                        fprintf(stderr, "write to TVlan: bad value\n");
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;
            }
        break;

        case RESERVE2:
        break;

        case FREE:
        break;

        case ACTION:
            switch (var) {
                case TVLANSTATUS:
                    /* Do not activate if entry does not exist */
                    sstvlangrp_parms->group_obj = SS_TVLAN_STATUS;
                    sstvlangrp_parms->value = value;
                    if ((creation_row == NULL) || vid != creation_row->vid) {
                        if (value == 1) {
                            sstvlangrp_parms->callback_rw = CALLBACK_ACT_READ;
                            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                                SNMPD_CALLBACK_NETGEAR,
                                                sstvlangrp_parms);
                            if (sstvlangrp_parms->value != 1) {
                                return SNMP_ERR_WRONGVALUE;
                            }
                        }
                    } else if (creation_row && vid == creation_row->vid) {
                        sstvlangrp_parms->group_obj = SS_TVLAN_VID_QUERY;
                        sstvlangrp_parms->callback_rw = CALLBACK_ACT_READ;
                        sstvlangrp_parms->value = vid;
                        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                            SNMPD_CALLBACK_NETGEAR,
                                            sstvlangrp_parms);
                        if (sstvlangrp_parms->value == 1) {
                            sal_free (creation_row);
                            creation_row = NULL;
                            return SNMP_ERR_WRONGVALUE;
                        }
					}
                break;
            }
          break;

        case UNDO:
          break;

        case COMMIT:
            switch(var) {
                case TVLANNAME:
                    AGENT_DBG(("Setting TVlan Name"));
                    if ((creation_row == NULL) || vid != creation_row->vid) {
                        sstvlangrp_parms->group_obj = SS_TVLAN_NAME;
                        memset(sstvlangrp_parms->buf, 0, 
                                             sizeof(sstvlangrp_parms->buf));
                        if (var_val) {
                            strcpy(sstvlangrp_parms->buf, var_val);
                        }
                    } else if (creation_row) {
                        memcpy(creation_row->vname, var_val, var_val_len);
                    }
                    break;

                case TVLANMEMBER:
                    AGENT_DBG(("Setting TVLAN Member"));
                    if ((creation_row == NULL) || vid != creation_row->vid) {
                        sstvlangrp_parms->group_obj = SS_TVLAN_MEMBER;
                        memset(sstvlangrp_parms->buf, 0, 
                                             sizeof(sstvlangrp_parms->buf));
                        memcpy(sstvlangrp_parms->buf, var_val, var_val_len);
                        sstvlangrp_parms->value = var_val_len;
                    } else  if (creation_row) {
                        memcpy(creation_row->vports, var_val, var_val_len);
                    }
                    break;

                case TVLANUNTAGGEDPORTS:
                    if ((creation_row == NULL) || vid != creation_row->vid) {
                        sstvlangrp_parms->group_obj = SS_TVLAN_UNTAGGEDPORTS;
                        memset(sstvlangrp_parms->buf, 0, 
                                             sizeof(sstvlangrp_parms->buf));
                        memcpy(sstvlangrp_parms->buf, var_val, var_val_len);
                        sstvlangrp_parms->value = var_val_len;
                    } else  if (creation_row) {
                        AGENT_DBG(("Setting untag member"));
                        memcpy(creation_row->vuntagports, var_val,
                                                           var_val_len);
                    }
                    break;

                case TVLANSTATUS:
                    sstvlangrp_parms->group_obj = SS_TVLAN_STATUS;
                    sstvlangrp_parms->value = value;
                    if ((creation_row == NULL) || vid != creation_row->vid) {
                        if (value == 1) {
                            return SNMP_ERR_NOERROR; /* Already active */
                        } else if (value == MIB_ROWSTATUS_NOTREADY || 
                                   value == MIB_ROWSTATUS_CREATEANDWAIT || 
                                   (value == MIB_ROWSTATUS_DESTROY && vid == 1)) {
                            /* Active or default VLAN */
                            return SNMP_ERR_WRONGVALUE;
                        } 
                    } else  if (creation_row) { /* row under creation */
                        if (value == MIB_ROWSTATUS_DESTROY) { /* Destroy */
                            sal_free (creation_row);
                            creation_row = NULL;
                            return SNMP_ERR_NOERROR;
                        } else if (value == MIB_ROWSTATUS_ACTIVE) { /* active */
                            memcpy (&sstvlangrp_parms->group_data.tvlan,
                                    creation_row, 
                                    sizeof(ngr_vlan_entry_t));
                        }
                    }
                    break;
            }

            /* call backend if already existing row, or to activate row
               under creation */
            if ((creation_row == NULL || vid != creation_row->vid) ||
               (vid == creation_row->vid && var == TVLANSTATUS && 
                                            value == MIB_ROWSTATUS_ACTIVE)) {
                    sstvlangrp_parms->callback_rw = CALLBACK_ACT_WRITE;
                    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                        SNMPD_CALLBACK_NETGEAR,
                                        sstvlangrp_parms);
                    if (sstvlangrp_parms->ret_val == MIB_STATUS_ERROR) {
                        AGENT_EXIT;
                        return SNMP_ERR_COMMITFAILED;
                    }
                    /* If activating createAndWait, free mem */
                    if (creation_row && vid == creation_row->vid) {
                        AGENT_DBG(("Freeing memory"));
                        sal_free (creation_row);
                        creation_row = NULL;
                    }
            }
        break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

/* Function to set or modify the port VLAN ID */
int
write_tvlanPortVlanId(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    AGENT_ENTER;
    AGENT_OID_PRINT(name_len, name);

    sstvlangrp_parms->index = (int) name[12];
    sstvlangrp_parms->group_obj = SS_TVLAN_PORTVLANID;
    sstvlangrp_parms->value = * (int *) var_val;
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssTVlanGroup not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }

          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssTVlanGroup: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          if(sstvlangrp_parms->value < 1 || sstvlangrp_parms->value > 4094) {
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
            sstvlangrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                SNMPD_CALLBACK_NETGEAR,
                                sstvlangrp_parms);
            if (sstvlangrp_parms->ret_val == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
          break;
    }
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

/*
 * Function to set or modify the Port priority
 */
int
write_tvlanPortPriority(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    AGENT_ENTER;
    AGENT_OID_PRINT(name_len, name);

    sstvlangrp_parms->index = (int) name[12];
    sstvlangrp_parms->group_obj = SS_TVLAN_PORTPRIORITY;
    sstvlangrp_parms->value = * (int *) var_val;
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to ssTVlanGroup not ASN_INTEGER\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to ssTVlanGroup: bad length\n");
              AGENT_EXIT;
              return SNMP_ERR_WRONGLENGTH;
          }
          if (sstvlangrp_parms->value < SS_TVLAN_PORT_PRIO_MIN || 
                        sstvlangrp_parms->value > SS_TVLAN_PORT_PRIO_MAX) {
              AGENT_EXIT;
              return SNMP_ERR_WRONGVALUE;
          }
          break;

        case RESERVE2:
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
          break;

        case UNDO:
          break;

        case COMMIT:
            sstvlangrp_parms->callback_rw = CALLBACK_ACT_WRITE;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                SNMPD_CALLBACK_NETGEAR,
                                sstvlangrp_parms);
            if (sstvlangrp_parms->ret_val == MIB_STATUS_ERROR) {
                AGENT_EXIT;
                return SNMP_ERR_COMMITFAILED;
            }
          break;
    }
    
    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

void free_tvlan_createAndWaitMemory(void) {
    if (creation_row) {
        sal_free(creation_row);
        creation_row = NULL;
    }

    return;
}

