/*
 * $Id: ssTrustMACGroup.c,v 1.1 2006/12/11 06:25:14 ganesh Exp $
 */

/* This should always be included first before anything else */
#include <config.h>

/* minimal include directives */
#include "mibincl.h"
#include "mibgroup/util_funcs.h"

#include <ucd-snmp/mibgroup/ng700ss/netgear_mib.h>
#include "mibgroup/ng700ss/ssTrustMACGroup.h"

/* Will be used for callback */
static ngrmib_parameters_t *trustmac_parms = NULL;

/* Path to the MIB */
oid ssTrustMACGroup_variables_oid[] = { 1,3,6,1,4,1,4526,11,21 };

/* Used for creating a new row */
static ngr_trustmac_entry_t         *creation_row = NULL;

/* Variables that this MIB supports */
struct variable7 ssTrustMACGroup_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define TRUSTEDID                1
    {TRUSTEDID,     ASN_INTEGER,  RONLY,   var_trustedmacTable, 3,  { 1, 1, 1 }},
#define TRUSTEDMAC                2
    {TRUSTEDMAC,    ASN_OCTET_STR,  RWRITE,  var_trustedmacTable, 3,  { 1, 1, 2 }},
#define TRUSTEDPORT                3
    {TRUSTEDPORT,   ASN_INTEGER,  RWRITE,  var_trustedmacTable, 3,  { 1, 1, 3 }},
#define TRUSTEDVID                4
    {TRUSTEDVID,    ASN_INTEGER,  RWRITE,  var_trustedmacTable, 3,  { 1, 1, 4 }},
#define TRUSTEDSTATUS            5
    {TRUSTEDSTATUS, ASN_INTEGER,  RWRITE,  var_trustedmacTable, 3,  { 1, 1, 5 }},
#define TRUSTEDMACGLOBALCFG        6
    {TRUSTEDMACGLOBALCFG,  ASN_INTEGER,  RWRITE,  var_ssTrustMACGroup, 1,  { 2 }},
};


/** Initializes the ssTrustMACGroup mib */
void
init_ssTrustMACGroup(void)
{
    AGENT_ENTER;

    /* Allocate memory for callback argument */
    trustmac_parms = (ngrmib_parameters_t *) \
                sal_alloc(sizeof(ngrmib_parameters_t), SS_TRUSTMAC_GROUP_MIB);

    if (trustmac_parms != NULL) {
        trustmac_parms->group = GROUP_TRUST_MAC;

        /* register ourselves with the agent to handle our mib tree */
        REGISTER_MIB(SS_TRUSTMAC_GROUP_MIB, ssTrustMACGroup_variables, variable7,
                   ssTrustMACGroup_variables_oid);
    }
}

/*
 * var_ssTrustMACGroup(): This function is called whenever there
 * is a request for current trust MAC status
 * 
 * NOTE: TRUSTEDMACGLOBALCFG is currently not writable. If there is one
 * or more entry in the trusted MAC table, it will be treated as 
 * enabled. Else, it will be treated as disabled.
 */
unsigned char *
var_ssTrustMACGroup(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    static long long_ret;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    switch(vp->magic) {
        case TRUSTEDMACGLOBALCFG:
#if 0 
             *write_method = write_trustedMacGlobalCfg;
#else
             *write_method = OID_NOT_WRITABLE;
#endif
             /* 1 if enabled, 2 if disabled */
             trustmac_parms->group_obj = SS_TRUSTEDMAC_GLOBALCFG;
        break;
        
        default:
              ERROR_MSG("Undefined OID in TrustMacGroup Global Variable");
        break;
    }

    /* Callback */
    trustmac_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        trustmac_parms);
    
    /* Give the value back */
    long_ret = (long)trustmac_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}


static int
header_trustMacTable(struct variable *vp,
           oid *name,
           size_t *length,
           int exact,
           size_t *var_len,
           WriteMethod **write_method)
{
#define SS_TMAC_ID_LEN     12
#define SS_TMAC_TOTAL_LEN  13

    oid newname[MAX_OID_LEN];
    int i = 0, result, found = FALSE;
    int tmac_count, tmac_index; 

    /* In case we are creating a new row, we need to set write method first */
    *write_method = write_trustMacEntry;

    if (creation_row) {
        AGENT_DBG(("Index under creation = %d", creation_row->tmid));
    } else {
        AGENT_DBG(("No MAC Address to add"));
    }

    /* Get count of number of trust mac entries on the switch */
    trustmac_parms->group_obj = SS_TRUSTEDMAC_COUNT;
    trustmac_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        trustmac_parms);
    tmac_count = trustmac_parms->value;
    AGENT_DBG(("Number of entries in TMAC Table %d", tmac_count));

    /* Initialize the new OID */
    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    /* Loop through all the indexes, plus one under creation */
    for (; i <= tmac_count; i++ ) {
      if (i == tmac_count) {
         if (creation_row) {
            trustmac_parms->index = creation_row->tmid;
            trustmac_parms->group_obj = SS_TRUSTEDSTATUS;
            trustmac_parms->callback_rw = CALLBACK_ACT_READ;
            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_NETGEAR,
                        trustmac_parms);
            if(trustmac_parms->value == MIB_ROWSTATUS_ACTIVE) {
                /*Tmid under creation status is active */
                sal_free (creation_row);
                creation_row = NULL;
                break;
            } else {
                creation_row->tmid = tmac_count+1;
                tmac_index = creation_row->tmid;
            }
            AGENT_DBG(("Num entries %d", tmac_count));
          } else {
              break;
          }
        } else {
            tmac_index = (i+1);
        }

        AGENT_DBG(("Index = %d", tmac_index));

        newname[SS_TMAC_ID_LEN] = (oid) tmac_index ;
        result = snmp_oid_compare(name, *length, newname, 
                                    (int) vp->namelen + 1);
        if ((exact && (result == 0)) || (!exact && (result < 0))) {
            found = TRUE;
            break;
        }
    }

    if (!found) {
        DEBUGMSGTL(("ssTrustMac", "Invalid Index"));
        AGENT_EXIT;
        return MATCH_FAILED;
    }

    memcpy( (char *)name,(char *)newname, ((int) vp->namelen +1) * sizeof(oid));

    *length = SS_TMAC_TOTAL_LEN;
    *var_len = sizeof(long);    /* default to 'long' results */

    AGENT_EXIT;
    return MATCH_SUCCEEDED;
}

/*
 * var_trustedmacTable(): This table contains all the entries added to
 * the trust MAC table
 */
unsigned char *
var_trustedmacTable(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static unsigned char string[SPRINT_MAX_LEN];

    snmp_mib_setInterfaceCount();

    /* Validate */
    if (header_trustMacTable(vp,name,length,exact, var_len,write_method) 
                                                        == MATCH_FAILED ) {
        AGENT_EXIT;
        return NULL;
    }

    trustmac_parms->index = (int) name[SS_TMAC_ID_LEN];
    AGENT_DBG(("Index of current entry is %d", trustmac_parms->index));
    switch(vp->magic) {
        case TRUSTEDID:
            long_ret = trustmac_parms->index;
            return (unsigned char *) &long_ret;

        case TRUSTEDMAC:
            if (creation_row && trustmac_parms->index == creation_row->tmid) {
                *var_len = MAC_ADDRESS_LEN;
                memcpy (trustmac_parms->buf, creation_row->tmac, *var_len);
                memcpy (string, trustmac_parms->buf,  *var_len);
                return (unsigned char*) string;
            }
            trustmac_parms->group_obj = SS_TRUSTEDMAC;
            break;
        case TRUSTEDPORT:
            if (creation_row && trustmac_parms->index == creation_row->tmid) {
                trustmac_parms->value = creation_row->tmport;
                long_ret = trustmac_parms->value;
                return (unsigned char*) &long_ret;
            }
            trustmac_parms->group_obj = SS_TRUSTEDPORT;
            break;
        case TRUSTEDVID:
            if (creation_row && trustmac_parms->index == creation_row->tmid) {
                trustmac_parms->value = creation_row->tmvid;
                long_ret = trustmac_parms->value;
                return (unsigned char*) &long_ret;
            }
            trustmac_parms->group_obj = SS_TRUSTEDVID;
            break;
        case TRUSTEDSTATUS:
            if (creation_row && trustmac_parms->index == creation_row->tmid) {
                trustmac_parms->value = creation_row->tmstatus;
                long_ret = trustmac_parms->value;
                return (unsigned char*) &long_ret;
            }
            trustmac_parms->group_obj = SS_TRUSTEDSTATUS;
            break;

        default:
          ERROR_MSG("Undefined MIB reference in trusted MAC Table");
    }

    /* Callback */
    trustmac_parms->callback_rw = CALLBACK_ACT_READ;
    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                        SNMPD_CALLBACK_NETGEAR,
                        trustmac_parms);
    
    
    /* Give the value back */
    switch(vp->magic) {
        case TRUSTEDMAC:
            *var_len = MAC_ADDRESS_LEN;
            memcpy (string, trustmac_parms->buf, *var_len);
            AGENT_EXIT;
            return (unsigned char *)string;
    }

    long_ret = (long)trustmac_parms->value;
    AGENT_EXIT;
    return (unsigned char *)&long_ret;
}

/* Common Write/Create function for Trust MAC Table */
int
write_trustMacEntry(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
#define TMAC_GET_CREATION_ROW_VAL    1
#define TMAC_GET_CURR_ROW_VAL        2
    AGENT_ENTER;
    AGENT_OID_PRINT(name_len, name);
    long            value;
    int             var;
    int             val_type = 0;
    int             vlan_mode = 0;
    unsigned char   temp_mac[MAC_ADDRESS_LEN];
    
    memset (temp_mac, 0, MAC_ADDRESS_LEN);

    var = name[SS_TMAC_ID_LEN - 1];
    trustmac_parms->index  = name[SS_TMAC_ID_LEN];
    AGENT_DBG(("TMAC: Var being set = %d", var));

    vlan_mode = snmp_mib_getVlanMode();

    /* Create an entry if createAndWait */
    value = *(long *) var_val;
    if (creation_row == NULL && (var == TRUSTEDSTATUS && 
                                        value == MIB_ROWSTATUS_CREATEANDWAIT)) {
        /* The specified index must not already exist */
        trustmac_parms->group_obj = SS_TRUSTEDMAC_COUNT;
        trustmac_parms->callback_rw = CALLBACK_ACT_READ;
        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, 
                            SNMPD_CALLBACK_NETGEAR,
                            trustmac_parms);
        AGENT_DBG(("Value = %d index = %d", 
                               trustmac_parms->value,
                               trustmac_parms->index));
        if ((trustmac_parms->value >= trustmac_parms->index) ||
            (trustmac_parms->index > (trustmac_parms->value+1))) {
            AGENT_DBG(("Trust MAC entry already exists"));
            return SNMP_ERR_WRONGVALUE;
        }

        creation_row = (ngr_trustmac_entry_t *) 
                                sal_alloc(sizeof(ngr_trustmac_entry_t), 
                                SS_TRUSTMAC_GROUP_MIB);

        if (creation_row == NULL) {
            AGENT_DBG(("Unable to allocate memory"));
            return SNMP_ERR_RESOURCEUNAVAILABLE;
        }

        /* zero out all fields */
        memset(creation_row, 0, sizeof(ngr_trustmac_entry_t));

        /* Set the index */
        creation_row->tmid = trustmac_parms->index;

        /* Set the row status to notReady */
        creation_row->tmstatus = MIB_ROWSTATUS_NOTREADY;
    } else if (creation_row != NULL && (var == TRUSTEDSTATUS && 
                                        value == MIB_ROWSTATUS_CREATEANDWAIT)) {
        /* We can have only one row under creation */
        if (trustmac_parms->index != creation_row->tmid) {
            AGENT_DBG(("We can have only one creation at a time"));
            return SNMP_ERR_RESOURCEUNAVAILABLE;
        }
    }

    val_type = (creation_row == NULL || 
                   trustmac_parms->index != creation_row->tmid)?
                        TMAC_GET_CURR_ROW_VAL:TMAC_GET_CREATION_ROW_VAL;
    switch ( action ) {
        case RESERVE1:
            if (var != TRUSTEDSTATUS && val_type == TMAC_GET_CURR_ROW_VAL) {
                return SNMP_ERR_NOTWRITABLE;
            }

            switch(var) {
                case TRUSTEDMAC:
                    if (var_val_type != ASN_OCTET_STR) {
                      fprintf(stderr, "write to ssTrustMACGroup not ASN_OCTET_STR\n");
                      AGENT_EXIT;
                      return SNMP_ERR_WRONGTYPE;
                    }
                    if (var_val_len != sizeof(char) * MAC_ADDRESS_LEN) {
                      fprintf(stderr,"write to ssTrustMACGroup: bad length\n");
                      AGENT_EXIT;
                      return SNMP_ERR_WRONGLENGTH;
                    }
                    if (!memcmp(var_val, temp_mac, var_val_len)) {
                        AGENT_EXIT;
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;

                case TRUSTEDPORT:
                    if (var_val_type != ASN_INTEGER) {
                      fprintf(stderr, "write to ssTrustMACGroup not ASN_INTEGER\n");
                      AGENT_EXIT;
                      return SNMP_ERR_WRONGTYPE;
                    }
                    if (var_val_len > sizeof(long)) {
                      fprintf(stderr,"write to ssTrustMACGroup: bad length\n");
                      AGENT_EXIT;
                      return SNMP_ERR_WRONGLENGTH;
                    }
                    if (value < 1 || value > interface_count) {
                      fprintf(stderr,"write to ssTrustMACGroup: Wrong Value\n");
                      AGENT_EXIT;
                      return SNMP_ERR_WRONGVALUE;
                    }
                    break;

                case TRUSTEDVID:
                    if (var_val_type != ASN_INTEGER) {
                      fprintf(stderr, "write to ssTrustMACGroup not ASN_INTEGER\n");
                      AGENT_EXIT;
                      return SNMP_ERR_WRONGTYPE;
                    }
                    if (var_val_len > sizeof(long)) {
                      fprintf(stderr,"write to ssTrustMACGroup: bad length\n");
                      AGENT_EXIT;
                      return SNMP_ERR_WRONGLENGTH;
                    }

                    /* do not allow if mode is port based vlan */
                    if (vlan_mode == MIB_VLAN_MODE_PORTBASED) {
                        return SNMP_ERR_NOTWRITABLE;
                    }

                    /* check if vlan exists */
                    trustmac_parms->group = GROUP_TVLAN;
                    trustmac_parms->group_obj = SS_TVLAN_VID_QUERY;
                    trustmac_parms->callback_rw = CALLBACK_ACT_READ;
                    trustmac_parms->value = value;
                    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                            SNMPD_CALLBACK_NETGEAR,
                                            trustmac_parms);
                    trustmac_parms->group = GROUP_TRUST_MAC;
                    if (trustmac_parms->value != 1) {
                        AGENT_EXIT;
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;

                case TRUSTEDSTATUS:
                    if (var_val_type != ASN_INTEGER) {
                        fprintf(stderr, "write to TrustMAC not ASN_INTEGER\n");
                        return SNMP_ERR_WRONGTYPE;
                    }

                    if (var_val_len > sizeof(long)) {
                        fprintf(stderr, "write to TrustMAC: bad length\n");
                        return SNMP_ERR_WRONGLENGTH;
                    }

                    if (value != MIB_ROWSTATUS_ACTIVE && 
                        value != MIB_ROWSTATUS_NOTREADY && 
                        value != MIB_ROWSTATUS_CREATEANDWAIT && 
                        value != MIB_ROWSTATUS_DESTROY) {
                        fprintf(stderr, "write to TrustMAC: bad value\n");
                        return SNMP_ERR_WRONGVALUE;
                    }
                    break;
            }
        break;

        case RESERVE2:
        break;

        case FREE:
        break;

        case ACTION:
            switch(var) {
                case TRUSTEDSTATUS:
                    if (creation_row) {
                        if (value == MIB_ROWSTATUS_ACTIVE) {
                            trustmac_parms->index = creation_row->tmid;
                            trustmac_parms->group_obj = SS_TRUSTEDSTATUS; 
                            trustmac_parms->callback_rw = CALLBACK_ACT_READ;
                            snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                                SNMPD_CALLBACK_NETGEAR,
                                                trustmac_parms);
                            if(trustmac_parms->value == MIB_ROWSTATUS_ACTIVE) {
                                /*tmid under creation status is active */
                                sal_free (creation_row);
                                creation_row = NULL;
                                return SNMP_ERR_NOERROR;
                            }
                        }
                    }
                break;
            }
          break;

        case UNDO:
          break;

        case COMMIT:
            switch(var) {
                case TRUSTEDMAC:
                    if (val_type == TMAC_GET_CURR_ROW_VAL) {
                        trustmac_parms->group_obj = SS_TRUSTEDMAC;
                        memcpy(trustmac_parms->buf, var_val, var_val_len);
                        trustmac_parms->value = var_val_len;
                    } else if (creation_row) {
                        memset(creation_row->tmac, 0, var_val_len);
                        if (!memcmp(creation_row->tmac, var_val, var_val_len)) {
                            return SNMP_ERR_WRONGVALUE;
                        }
                        memcpy(creation_row->tmac, var_val, var_val_len);
                    }
                    break;

                case TRUSTEDPORT:
                    if (val_type == TMAC_GET_CURR_ROW_VAL) {
                        trustmac_parms->group_obj = SS_TRUSTEDPORT;
                        trustmac_parms->value = value;
                    } else  if (creation_row) {
                        creation_row->tmport = value;
                    }
                    break;

                case TRUSTEDVID:
                    if (val_type == TMAC_GET_CURR_ROW_VAL) {
                        trustmac_parms->group_obj = SS_TRUSTEDVID;
                        trustmac_parms->value = value;
                    } else  if (creation_row) {
                        creation_row->tmvid = value;
                    }
                    break;

                case TRUSTEDSTATUS:
                    trustmac_parms->group_obj = SS_TRUSTEDSTATUS;
                    trustmac_parms->value = value;
                    if (val_type == TMAC_GET_CURR_ROW_VAL) {
                        if (value == 1) {
                            return SNMP_ERR_NOERROR; /* Already active */
                        } else if (value == MIB_ROWSTATUS_NOTREADY || 
                                        value == MIB_ROWSTATUS_CREATEANDWAIT) {
                            /* Already active, so cant set not ready */
                            return SNMP_ERR_WRONGVALUE;
                        }
                    } else  if (creation_row) { /* row under creation */
                        if (value == MIB_ROWSTATUS_DESTROY) { /* Destroy */
                            sal_free (creation_row);
                            creation_row = NULL;
                            return SNMP_ERR_NOERROR;
                        } else if (value == MIB_ROWSTATUS_ACTIVE) { /* active */
                            memset (trustmac_parms->buf, 0, sizeof(trustmac_parms->buf));
                            if (!creation_row->tmport || 
                                (vlan_mode == MIB_VLAN_MODE_TAGGED && !creation_row->tmvid) || 
                                !(memcmp(creation_row->tmac, trustmac_parms->buf, MAC_ADDRESS_LEN))) {
                                return SNMP_ERR_WRONGVALUE;
                            }
                            memcpy (&trustmac_parms->group_data.trustmac,
                                    creation_row, 
                                    sizeof(ngr_trustmac_entry_t));
                        }
                    }
                    break;
            }

            /* call backend if already existing row, or to activate row
               under creation */
            if (val_type == TMAC_GET_CURR_ROW_VAL ||
                (trustmac_parms->index == creation_row->tmid && 
                                      var == TRUSTEDSTATUS && 
                                      value == MIB_ROWSTATUS_ACTIVE)) {
                    trustmac_parms->callback_rw = CALLBACK_ACT_WRITE;
                    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                                        SNMPD_CALLBACK_NETGEAR,
                                        trustmac_parms);
                    if (trustmac_parms->ret_val == MIB_STATUS_ERROR) {
                        AGENT_EXIT;
                        return SNMP_ERR_COMMITFAILED;
                    }

                    /* If activating createAndWait, free mem */
                    if (creation_row && 
                             trustmac_parms->index == creation_row->tmid) {
                        sal_free (creation_row);
                        creation_row = NULL;
                    }
            }
        break;
    }

    AGENT_EXIT;
    return SNMP_ERR_NOERROR;
}

void free_trustmac_createAndWaitMemory(void) {
    if (creation_row) {
        sal_free(creation_row);
        creation_row = NULL;
    }

    return;
}

